@@ -17,7 +17,7 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -186,6 +186,9 @@ static LinkedListInfo *AcquireConfigureCache(const char *filename,
     }
     options=DestroyConfigureOptions(options);
   }
+#else
+  magick_unreferenced(filename);
+  magick_unreferenced(exception);
 #endif
   /*
     Load built-in configure.
@@ -253,7 +256,7 @@ MagickPrivate MagickBooleanType ConfigureComponentGenesis(void)
 
 static void *DestroyConfigureElement(void *configure_info)
 {
-  register ConfigureInfo
+  ConfigureInfo
     *p;
 
   p=(ConfigureInfo *) configure_info;
@@ -349,7 +352,7 @@ MagickExport LinkedListInfo *DestroyConfigureOptions(LinkedListInfo *options)
 MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
   ExceptionInfo *exception)
 {
-  register const ConfigureInfo
+  const ConfigureInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -436,10 +439,10 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
   const ConfigureInfo
     **options;
 
-  register const ConfigureInfo
+  const ConfigureInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -511,7 +514,7 @@ extern "C" {
 
 static int ConfigureCompare(const void *x,const void *y)
 {
-  register char
+  char
     **p,
     **q;
 
@@ -530,10 +533,10 @@ MagickExport char **GetConfigureList(const char *pattern,
   char
     **options;
 
-  register const ConfigureInfo
+  const ConfigureInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -759,7 +762,7 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     configure_path=GetEnvironmentValue("MAGICK_CONFIGURE_PATH");
     if (configure_path != (char *) NULL)
       {
-        register char
+        char
           *p,
           *q;
 
@@ -1053,7 +1056,7 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
   const ConfigureInfo
     **configure_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
