@@ -16,7 +16,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -80,6 +80,14 @@
 /*
   Define declarations.
 */
+#if defined(__APPLE__)
+  #include "TargetConditionals.h"
+  #if TARGET_OS_IOS || TARGET_OS_WATCH || TARGET_OS_TV
+    #define system(s) ((s)==NULL ? 0 : -1)
+  #endif // end iOS
+#elif defined(__ANDROID__)
+  #define system(s) ((s)==NULL ? 0 : -1)
+#endif
 #define DelegateFilename  "delegates.xml"
 
 /*
@@ -145,8 +153,8 @@ static const char
     "  <delegate decode=\"xps:cmyk\" stealth=\"True\" command=\"&quot;gxps&quot; -dQUIET -dSAFER -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=2 &quot;-sDEVICE=bmpsep8&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
     "  <delegate decode=\"xps:color\" stealth=\"True\" command=\"&quot;gxps&quot; -dQUIET -dSAFER -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=2 &quot;-sDEVICE=ppmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
     "  <delegate decode=\"xps:mono\" stealth=\"True\" command=\"&quot;gxps&quot; -dQUIET -dSAFER -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=2 &quot;-sDEVICE=pbmraw&quot; -dTextAlphaBits=%u -dGraphicsAlphaBits=%u &quot;-r%s&quot; %s &quot;-sOutputFile=%s&quot; &quot;%s&quot;\"/>"
-    "  <delegate decode=\"mpeg:decode\" command=\"&quot;ffmpeg&quot; -v -1 -i &quot;%i&quot; -vframes %S -vcodec pam -an -f rawvideo -y &quot;%u.pam&quot; 2&gt; &quot;%u&quot;\"/>"
-    "  <delegate encode=\"mpeg:encode\" stealth=\"True\" command=\"&quot;ffmpeg&quot; -v -1 -i &quot;%M%%d.jpg&quot; -plays %I &quot;%u.%m&quot; 2&gt; &quot;%u&quot;\"/>"
+    "  <delegate decode=\"video:decode\" command=\"&quot;ffmpeg&quot; -nostdin -loglevel error -i &quot;%s&quot; -an -f rawvideo -y %s &quot;%s&quot;\"/>"
+    "  <delegate encode=\"video:encode\" stealth=\"True\" command=\"&quot;ffmpeg&quot; -nostdin -loglevel error -i &quot;%s%%d.%s&quot; %s &quot;%s.%s&quot;\"/>"
     "</delegatemap>";
 
 /*
@@ -198,11 +206,7 @@ static LinkedListInfo *AcquireDelegateCache(const char *filename,
   LinkedListInfo
     *cache;
 
-  MagickStatusType
-    status;
-
   cache=NewLinkedList(0);
-  status=MagickTrue;
 #if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     const StringInfo
@@ -215,15 +219,17 @@ static LinkedListInfo *AcquireDelegateCache(const char *filename,
     option=(const StringInfo *) GetNextValueInLinkedList(options);
     while (option != (const StringInfo *) NULL)
     {
-      status&=LoadDelegateCache(cache,(const char *)
+      (void) LoadDelegateCache(cache,(const char *)
         GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
       option=(const StringInfo *) GetNextValueInLinkedList(options);
     }
     options=DestroyConfigureOptions(options);
   }
+#else
+  magick_unreferenced(filename);
 #endif
   if (IsLinkedListEmpty(cache) != MagickFalse)
-    status&=LoadDelegateCache(cache,DelegateMap,"built-in",0,exception);
+    (void) LoadDelegateCache(cache,DelegateMap,"built-in",0,exception);
   return(cache);
 }
 
@@ -273,7 +279,7 @@ MagickPrivate MagickBooleanType DelegateComponentGenesis(void)
 
 static void *DestroyDelegate(void *delegate_info)
 {
-  register DelegateInfo
+  DelegateInfo
     *p;
 
   p=(DelegateInfo *) delegate_info;
@@ -356,7 +362,7 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
   PolicyRights
     rights;
 
-  register ssize_t
+  ssize_t
     i;
 
   status=(-1);
@@ -393,55 +399,92 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
   if (message != (char *) NULL)
     *message='\0';
 #if defined(MAGICKCORE_POSIX_SUPPORT)
-#if !defined(MAGICKCORE_HAVE_EXECVP)
-  status=system(sanitize_command);
-#else
-  if ((asynchronous != MagickFalse) ||
-      (strpbrk(sanitize_command,"&;<>|") != (char *) NULL))
-    status=system(sanitize_command);
-  else
+#if defined(MAGICKCORE_HAVE_POPEN)
+  if ((asynchronous == MagickFalse) && (message !=  (char *) NULL))
     {
-      pid_t
-        child_pid;
+      char
+        buffer[MagickPathExtent];
 
-      /*
-        Call application directly rather than from a shell.
-      */
-      child_pid=(pid_t) fork();
-      if (child_pid == (pid_t) -1)
+      FILE
+        *file;
+
+      size_t
+        offset;
+
+      offset=0;
+      file=popen_utf8(sanitize_command,"r");
+      if (file == (FILE *) NULL)
         status=system(sanitize_command);
       else
-        if (child_pid == 0)
+        {
+          while (fgets(buffer,(int) sizeof(buffer),file) != NULL)
           {
-            status=execvp(arguments[1],arguments+1);
-            _exit(1);
-          }
-        else
-          {
-            int
-              child_status;
+            size_t
+              length;
 
-            pid_t
-              pid;
-
-            child_status=0;
-            pid=(pid_t) waitpid(child_pid,&child_status,0);
-            if (pid == -1)
-              status=(-1);
-            else
+            length=MagickMin(MagickPathExtent-offset,strlen(buffer)+1);
+            if (length > 0)
               {
-                if (WIFEXITED(child_status) != 0)
-                  status=WEXITSTATUS(child_status);
-                else
-                  if (WIFSIGNALED(child_status))
-                    status=(-1);
+                (void) CopyMagickString(message+offset,buffer,length);
+                offset+=length-1;
               }
           }
+          status=pclose(file);
+        }
     }
+  else
 #endif
+    {
+#if !defined(MAGICKCORE_HAVE_EXECVP)
+      status=system(sanitize_command);
+#else
+      if ((asynchronous != MagickFalse) ||
+          (strpbrk(sanitize_command,"&;<>|") != (char *) NULL))
+        status=system(sanitize_command);
+      else
+        {
+          pid_t
+            child_pid;
+
+          /*
+            Call application directly rather than from a shell.
+          */
+          child_pid=(pid_t) fork();
+          if (child_pid == (pid_t) -1)
+            status=system(sanitize_command);
+          else
+            if (child_pid == 0)
+              {
+                status=execvp(arguments[1],arguments+1);
+                _exit(1);
+              }
+            else
+              {
+                int
+                  child_status;
+
+                pid_t
+                  pid;
+
+                child_status=0;
+                pid=(pid_t) waitpid(child_pid,&child_status,0);
+                if (pid == -1)
+                  status=(-1);
+                else
+                  {
+                    if (WIFEXITED(child_status) != 0)
+                      status=WEXITSTATUS(child_status);
+                    else
+                      if (WIFSIGNALED(child_status))
+                        status=(-1);
+                  }
+              }
+        }
+#endif
+    }
 #elif defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
-    register char
+    char
       *p;
 
     /*
@@ -462,8 +505,6 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
       }
   }
   status=NTSystemCommand(sanitize_command,message);
-#elif defined(macintosh)
-  status=MACSystemCommand(sanitize_command);
 #elif defined(vms)
   status=system(sanitize_command);
 #else
@@ -523,36 +564,6 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
 %
 */
 
-static char *SanitizeDelegateString(const char *source)
-{
-  char
-    *sanitize_source;
-
-  const char
-    *q;
-
-  register char
-    *p;
-
-  static char
-#if defined(MAGICKCORE_WINDOWS_SUPPORT)
-    whitelist[] =
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "
-      "$-_.+!;*(),{}|^~[]`\'><#%/?:@&=";
-#else
-    whitelist[] =
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "
-      "$-_.+!;*(),{}|\\^~[]`\"><#%/?:@&=";
-#endif
-
-  sanitize_source=AcquireString(source);
-  p=sanitize_source;
-  q=sanitize_source+strlen(sanitize_source);
-  for (p+=strspn(p,whitelist); p != q; p+=strspn(p,whitelist))
-    *p='_';
-  return(sanitize_source);
-}
-
 static char *GetMagickPropertyLetter(ImageInfo *image_info,Image *image,
   const char letter,ExceptionInfo *exception)
 {
@@ -685,8 +696,6 @@ static char *GetMagickPropertyLetter(ImageInfo *image_info,Image *image,
 
       WarnNoImageReturn("\"%%%c\"",letter);
       colorspace=image->colorspace;
-      if (SetImageGray(image,exception) != MagickFalse)
-        colorspace=GRAYColorspace;   /* FUTURE: this is IMv6 not IMv7 */
       (void) FormatLocaleString(value,MagickPathExtent,"%s %s %s",
         CommandOptionToMnemonic(MagickClassOptions,(ssize_t)
         image->storage_class),CommandOptionToMnemonic(MagickColorspaceOptions,
@@ -724,14 +733,18 @@ static char *GetMagickPropertyLetter(ImageInfo *image_info,Image *image,
     {
       WarnNoImageReturn("\"%%%c\"",letter);
       (void) FormatLocaleString(value,MagickPathExtent,"%.20g",
-        fabs(image->resolution.x) > MagickEpsilon ? image->resolution.x : 72.0);
+        fabs(image->resolution.x) > MagickEpsilon ? image->resolution.x :
+        image->units == PixelsPerCentimeterResolution ? DefaultResolution/2.54 :
+        DefaultResolution);
       break;
     }
     case 'y': /* Image vertical resolution (with units) */
     {
       WarnNoImageReturn("\"%%%c\"",letter);
       (void) FormatLocaleString(value,MagickPathExtent,"%.20g",
-        fabs(image->resolution.y) > MagickEpsilon ? image->resolution.y : 72.0);
+        fabs(image->resolution.y) > MagickEpsilon ? image->resolution.y :
+        image->units == PixelsPerCentimeterResolution ? DefaultResolution/2.54 :
+        DefaultResolution);
       break;
     }
     case 'z': /* Image depth as read in */
@@ -910,7 +923,7 @@ static char *InterpretDelegateProperties(ImageInfo *image_info,
     { \
       extent+=length; \
       interpret_text=(char *) ResizeQuantumMemory(interpret_text,extent+ \
-        MaxTextExtent,sizeof(*interpret_text)); \
+        MagickPathExtent,sizeof(*interpret_text)); \
       if (interpret_text == (char *) NULL) \
         return((char *) NULL); \
       q=interpret_text+strlen(interpret_text); \
@@ -924,7 +937,7 @@ static char *InterpretDelegateProperties(ImageInfo *image_info,
     { \
       extent+=length; \
       interpret_text=(char *) ResizeQuantumMemory(interpret_text,extent+ \
-        MaxTextExtent,sizeof(*interpret_text)); \
+        MagickPathExtent,sizeof(*interpret_text)); \
       if (interpret_text == (char *) NULL) \
         return((char *) NULL); \
       q=interpret_text+strlen(interpret_text); \
@@ -939,7 +952,7 @@ static char *InterpretDelegateProperties(ImageInfo *image_info,
     { \
       extent+=length; \
       interpret_text=(char *) ResizeQuantumMemory(interpret_text,extent+ \
-        MaxTextExtent,sizeof(*interpret_text)); \
+        MagickPathExtent,sizeof(*interpret_text)); \
       if (interpret_text == (char *) NULL) \
         return((char *) NULL); \
       q=interpret_text+strlen(interpret_text); \
@@ -952,10 +965,10 @@ static char *InterpretDelegateProperties(ImageInfo *image_info,
     *interpret_text,
     *string;
 
-  register char
+  char
     *q;  /* current position in interpret_text */
 
-  register const char
+  const char
     *p;  /* position in embed_text string being expanded */
 
   size_t
@@ -1110,7 +1123,7 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
   const DelegateInfo
     *delegate_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(image_info != (ImageInfo *) NULL);
@@ -1212,7 +1225,7 @@ MagickExport const char *GetDelegateCommands(const DelegateInfo *delegate_info)
 MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
   const char *encode,ExceptionInfo *exception)
 {
-  register const DelegateInfo
+  const DelegateInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -1331,10 +1344,10 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
   const DelegateInfo
     **delegates;
 
-  register const DelegateInfo
+  const DelegateInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1409,7 +1422,7 @@ extern "C" {
 
 static int DelegateCompare(const void *x,const void *y)
 {
-  register const char
+  const char
     **p,
     **q;
 
@@ -1428,10 +1441,10 @@ MagickExport char **GetDelegateList(const char *pattern,
   char
     **delegates;
 
-  register const DelegateInfo
+  const DelegateInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1611,7 +1624,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
   MagickBooleanType
     status;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -1693,7 +1706,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   PolicyRights
     rights;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1706,14 +1719,16 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   rights=ExecutePolicyRights;
-  if (IsRightsAuthorized(DelegatePolicyDomain,rights,decode) == MagickFalse)
+  if ((decode != (const char *) NULL) &&
+      (IsRightsAuthorized(DelegatePolicyDomain,rights,decode) == MagickFalse))
     {
       errno=EPERM;
       (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
         "NotAuthorized","`%s'",decode);
       return(MagickFalse);
     }
-  if (IsRightsAuthorized(DelegatePolicyDomain,rights,encode) == MagickFalse)
+  if ((encode != (const char *) NULL) &&
+      (IsRightsAuthorized(DelegatePolicyDomain,rights,encode) == MagickFalse))
     {
       errno=EPERM;
       (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
@@ -1760,7 +1775,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       ImageInfo
         *clone_info;
 
-      register Image
+      Image
         *p;
 
       /*
@@ -1952,7 +1967,7 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
   const char
     *path;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1994,11 +2009,11 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
     (void) FormatLocaleFile(file,"%11s%c=%c%s  ",delegate_info[i]->decode ?
       delegate_info[i]->decode : "",delegate_info[i]->mode <= 0 ? '<' : ' ',
       delegate_info[i]->mode >= 0 ? '>' : ' ',delegate);
-    StripString(commands[0]);
+    (void) StripMagickString(commands[0]);
     (void) FormatLocaleFile(file,"\"%s\"\n",commands[0]);
     for (j=1; commands[j] != (char *) NULL; j++)
     {
-      StripString(commands[j]);
+      (void) StripMagickString(commands[j]);
       (void) FormatLocaleFile(file,"                     \"%s\"\n",commands[j]);
     }
     for (j=0; commands[j] != (char *) NULL; j++)
@@ -2203,6 +2218,8 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
             (void) SubstituteString((char **) &commands,"&amp;","&");
             (void) SubstituteString((char **) &commands,"&gt;",">");
             (void) SubstituteString((char **) &commands,"&lt;","<");
+            if (delegate_info->commands != (char *) NULL)
+              delegate_info->commands=DestroyString(delegate_info->commands);
             delegate_info->commands=commands;
             break;
           }
