@@ -17,7 +17,7 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -154,6 +154,12 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
         (void) CopyMagickString(p,p+1,MagickPathExtent);
         break;
       }
+      case '#':
+      {
+        flags|=MaximumValue;
+        (void) CopyMagickString(p,p+1,MagickPathExtent);
+        break;
+      }
       case '^':
       {
         flags|=MinimumValue;
@@ -462,14 +468,14 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
     };
 
   char
-    page[MaxTextExtent];
+    page[MagickPathExtent];
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(page_geometry != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",page_geometry);
-  (void) CopyMagickString(page,page_geometry,MaxTextExtent);
+  (void) CopyMagickString(page,page_geometry,MagickPathExtent);
   for (i=0; i < (ssize_t) (sizeof(PageSizes)/sizeof(PageSizes[0])); i++)
   {
     int
@@ -487,12 +493,12 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
         /*
           Replace mneumonic with the equivalent size in dots-per-inch.
         */
-        (void) FormatLocaleString(page,MaxTextExtent,"%s%.80s",
+        (void) FormatLocaleString(page,MagickPathExtent,"%s%.80s",
           PageSizes[i].geometry,page_geometry+PageSizes[i].extent);
         flags=GetGeometry(page,&geometry.x,&geometry.y,&geometry.width,
           &geometry.height);
         if ((flags & GreaterValue) == 0)
-          (void) ConcatenateMagickString(page,">",MaxTextExtent);
+          (void) ConcatenateMagickString(page,">",MagickPathExtent);
         break;
       }
   }
@@ -663,7 +669,12 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
     return(MagickFalse);
   p=(char *) geometry;
   value=StringToDouble(geometry,&p);
-  (void) value;
+  if (IsNaN(value) != 0)
+    return(MagickFalse);
+  if (value > (double) MAGICK_SSIZE_MAX)
+    return(MagickFalse);
+  if (value < (double) MAGICK_SSIZE_MIN)
+    return(MagickFalse);
   if (p == geometry)
     return(MagickFalse);
   if (strspn(geometry,"0123456789-, ") != strlen(geometry))
@@ -758,7 +769,7 @@ MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
   MagickStatusType
     flags;
 
-  register ssize_t
+  ssize_t
     i;
 
   GetAffineMatrix(affine_matrix);
@@ -900,7 +911,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   for (p=pedantic_geometry; *p != '\0'; )
   {
     c=(int) ((unsigned char) *p);
-    if (isspace(c) != 0)
+    if (isspace((int) ((unsigned char) c)) != 0)
       {
         (void) CopyMagickString(p,p+1,MagickPathExtent);
         continue;
@@ -931,6 +942,12 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         (void) CopyMagickString(p,p+1,MagickPathExtent);
         break;
       }
+      case '#':
+      {
+        flags|=MaximumValue;
+        (void) CopyMagickString(p,p+1,MagickPathExtent);
+        break;
+      }
       case '^':
       {
         flags|=MinimumValue;
@@ -1300,18 +1317,24 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
       (void) ParseGeometry(geometry,&geometry_info);
       geometry_ratio=geometry_info.rho;
       image_ratio=(double) image->columns/image->rows;
-      if (geometry_ratio >= image_ratio)
+      region_info->width=image->columns;
+      region_info->height=image->rows;
+      if ((flags & MaximumValue) != 0)
         {
-          region_info->width=image->columns;
-          region_info->height=(size_t) floor((double) (image->rows*image_ratio/
-            geometry_ratio)+0.5);
+          if (geometry_ratio < image_ratio)
+            region_info->height=(size_t) floor((double) (image->rows*
+              image_ratio/geometry_ratio)+0.5);
+          else
+            region_info->width=(size_t) floor((double) (image->columns*
+              geometry_ratio/image_ratio)+0.5);
         }
       else
-        {
+        if (geometry_ratio >= image_ratio)
+          region_info->height=(size_t) floor((double) (image->rows*image_ratio/
+            geometry_ratio)+0.5);
+        else
           region_info->width=(size_t) floor((double) (image->columns*
             geometry_ratio/image_ratio)+0.5);
-          region_info->height=image->rows;
-        }
     }
   /*
     Adjust offset according to gravity setting.
@@ -1419,8 +1442,8 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       scale.y=geometry_info.sigma;
       if ((percent_flags & SigmaValue) == 0)
         scale.y=scale.x;
-      *width=(size_t) MagickMax(floor(scale.x*former_width/100.0+0.5),1.0);
-      *height=(size_t) MagickMax(floor(scale.y*former_height/100.0+0.5),1.0);
+      *width=(size_t) floor(scale.x*former_width/100.0+0.5);
+      *height=(size_t) floor(scale.y*former_height/100.0+0.5);
       former_width=(*width);
       former_height=(*height);
     }
