@@ -17,7 +17,7 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,6 +41,7 @@
   Include declarations.
 */
 #include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
 #include "MagickCore/blob.h"
 #include "MagickCore/blob-private.h"
 #include "MagickCore/exception.h"
@@ -49,6 +50,7 @@
 #include "MagickCore/list.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -77,7 +79,7 @@
 */
 MagickExport void AppendImageToList(Image **images,const Image *append)
 {
-  register Image
+  Image
     *p,
     *q;
 
@@ -129,7 +131,7 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
     *clone,
     *image;
 
-  register Image
+  Image
     *p;
 
   if (images == (Image *) NULL)
@@ -207,6 +209,9 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
   char
     *p;
 
+  const char
+    *artifact;
+
   const Image
     *next;
 
@@ -214,7 +219,7 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
     *clone_images,
     *image;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -234,6 +239,7 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
   assert(exception->signature == MagickCoreSignature);
   clone_images=NewImageList();
   images=GetFirstImageInList(images);
+  artifact=GetImageArtifact(images,"frames:step");
   length=GetImageListLength(images);
   for (p=(char *) scenes; *p != '\0';)
   {
@@ -248,6 +254,7 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
     else
       if (first > (ssize_t) length)
         first=(ssize_t) length;
+    first%=(length << 1);
     last=first;
     while (isspace((int) ((unsigned char) *p)) != 0)
       p++;
@@ -260,9 +267,17 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
           if (last > (ssize_t) length)
             last=(ssize_t) length;
       }
+    last%=(length << 1);
     match=MagickFalse;
-    step=(ssize_t) (first > last ? -1 : 1);
-    for ( ; first != (last+step); first+=step)
+    step=1;
+    if (artifact != (const char *) NULL)
+      {
+        step=(ssize_t) StringToLong(artifact);
+        if (step == 0)
+          step=1;
+      }
+    step=(ssize_t) (first > last ? -step : step);
+    for ( ; step > 0 ? (last-first) >= 0 : (last-first) <= 0; first+=step)
     {
       i=0;
       for (next=images; next != (Image *) NULL; next=GetNextImageInList(next))
@@ -370,7 +385,7 @@ MagickExport void DeleteImages(Image **images,const char *scenes,
   MagickBooleanType
     *delete_list;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -514,7 +529,7 @@ MagickExport Image *DuplicateImages(Image *images,
     *clone_images,
     *duplicate_images;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -560,7 +575,7 @@ MagickExport Image *DuplicateImages(Image *images,
 */
 MagickExport Image *GetFirstImageInList(const Image *images)
 {
-  register const Image
+  const Image
     *p;
 
   if (images == (Image *) NULL)
@@ -604,10 +619,10 @@ MagickExport Image *GetFirstImageInList(const Image *images)
 */
 MagickExport Image *GetImageFromList(const Image *images,const ssize_t index)
 {
-  register const Image
+  const Image
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   if (images == (Image *) NULL)
@@ -656,7 +671,7 @@ MagickExport Image *GetImageFromList(const Image *images,const ssize_t index)
 */
 MagickExport ssize_t GetImageIndexInList(const Image *images)
 {
-  register ssize_t
+  ssize_t
     i;
 
   if (images == (const Image *) NULL)
@@ -695,7 +710,7 @@ MagickExport ssize_t GetImageIndexInList(const Image *images)
 */
 MagickExport size_t GetImageListLength(const Image *images)
 {
-  register ssize_t
+  ssize_t
     i;
 
   if (images == (Image *) NULL)
@@ -736,7 +751,7 @@ MagickExport size_t GetImageListLength(const Image *images)
 */
 MagickExport Image *GetLastImageInList(const Image *images)
 {
-  register const Image
+  const Image
     *p;
 
   if (images == (Image *) NULL)
@@ -847,7 +862,7 @@ MagickExport Image **ImageListToArray(const Image *images,
   Image
     **group;
 
-  register ssize_t
+  ssize_t
     i;
 
   if (images == (Image *) NULL)
@@ -1002,7 +1017,7 @@ MagickExport void PrependImageToList(Image **images,Image *prepend)
 */
 MagickExport Image *RemoveImageFromList(Image **images)
 {
-  register Image
+  Image
     *p;
 
   assert(images != (Image **) NULL);
@@ -1173,20 +1188,23 @@ MagickExport void ReplaceImageInList(Image **images,Image *replace)
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickCoreSignature);
-
-  /* link next pointer */
+  /*
+    Link next pointer.
+  */
   replace=GetLastImageInList(replace);
   replace->next=(*images)->next;
   if (replace->next != (Image *) NULL)
     replace->next->previous=replace;
-
-  /* link previous pointer - set images position to first replacement image */
+  /*
+    Link previous pointer - set images position to first replacement image.
+  */
   replace=GetFirstImageInList(replace);
   replace->previous=(*images)->previous;
   if (replace->previous != (Image *) NULL)
     replace->previous->next=replace;
-
-  /* destroy the replaced image that was in images */
+  /*
+    Destroy the replaced image that was in images.
+  */
   (void) DestroyImage(*images);
   (*images)=replace;
 }
@@ -1231,20 +1249,23 @@ MagickExport void ReplaceImageInListReturnLast(Image **images,Image *replace)
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickCoreSignature);
-
-  /* link previous pointer */
+  /*
+    Link previous pointer.
+  */
   replace=GetFirstImageInList(replace);
   replace->previous=(*images)->previous;
   if (replace->previous != (Image *) NULL)
     replace->previous->next=replace;
-
-  /* link next pointer - set images position to last replacement image */
+  /*
+    Link next pointer - set images position to last replacement image.
+  */
   replace=GetLastImageInList(replace);
   replace->next=(*images)->next;
   if (replace->next != (Image *) NULL)
     replace->next->previous=replace;
-
-  /* destroy the replaced image that was in images */
+  /*
+    Destroy the replaced image that was in images.
+  */
   (void) DestroyImage(*images);
   (*images)=replace;
 }
@@ -1277,7 +1298,7 @@ MagickExport void ReverseImageList(Image **images)
   Image
     *next;
 
-  register Image
+  Image
     *p;
 
   assert(images != (Image **) NULL);
@@ -1332,7 +1353,7 @@ MagickExport Image *SpliceImageIntoList(Image **images,
     *image,
     *split;
 
-  register size_t
+  size_t
     i;
 
   assert(images != (Image **) NULL);
@@ -1410,7 +1431,7 @@ MagickExport Image *SplitImageList(Image *images)
 */
 MagickExport void SyncImageList(Image *images)
 {
-  register Image
+  Image
     *p,
     *q;
 
