@@ -16,7 +16,7 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -319,7 +319,7 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
 #if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 static ssize_t ReadRandom(int file,unsigned char *source,size_t length)
 {
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -413,7 +413,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     if (gettimeofday(&timer,(struct timezone *) NULL) == 0)
       {
         seconds=(size_t) timer.tv_sec;
-        nanoseconds=(size_t) (1000UL*timer.tv_usec);
+        nanoseconds=((size_t) 1000UL*timer.tv_usec);
       }
   }
 #endif
@@ -454,22 +454,22 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
     double
-      seconds;
+      nt_seconds;
 
     LARGE_INTEGER
-      nanoseconds;
+      nt_nanoseconds;
 
     /*
       Not crytographically strong but better than nothing.
     */
-    seconds=NTElapsedTime()+NTUserTime();
-    SetStringInfoLength(chaos,sizeof(seconds));
-    SetStringInfoDatum(chaos,(unsigned char *) &seconds);
+    nt_seconds=NTElapsedTime()+NTUserTime();
+    SetStringInfoLength(chaos,sizeof(nt_seconds));
+    SetStringInfoDatum(chaos,(unsigned char *) &nt_seconds);
     ConcatenateStringInfo(entropy,chaos);
-    if (QueryPerformanceCounter(&nanoseconds) != 0)
+    if (QueryPerformanceCounter(&nt_nanoseconds) != 0)
       {
-        SetStringInfoLength(chaos,sizeof(nanoseconds));
-        SetStringInfoDatum(chaos,(unsigned char *) &nanoseconds);
+        SetStringInfoLength(chaos,sizeof(nt_nanoseconds));
+        SetStringInfoDatum(chaos,(unsigned char *) &nt_nanoseconds);
         ConcatenateStringInfo(entropy,chaos);
       }
     /*
@@ -498,7 +498,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     */
     if (environ != (char **) NULL)
       {
-        register ssize_t
+        ssize_t
           i;
 
         /*
@@ -831,7 +831,7 @@ MagickPrivate void RandomComponentTerminus(void)
 
 static inline void IncrementRandomNonce(StringInfo *nonce)
 {
-  register ssize_t
+  ssize_t
     i;
 
   unsigned char
@@ -850,10 +850,10 @@ static inline void IncrementRandomNonce(StringInfo *nonce)
 MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
   unsigned char *key)
 {
-  register size_t
+  size_t
     i;
 
-  register unsigned char
+  unsigned char
     *p;
 
   SignatureInfo
