@@ -17,7 +17,7 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the license.  You may  %
@@ -37,7 +37,7 @@
 */
 
 /*
-  include declarations.
+  Include declarations.
 */
 #include "MagickCore/studio.h"
 #include "MagickCore/blob.h"
@@ -74,7 +74,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AcquireString() returns an new extented string, containing a clone of the
+%  AcquireString() returns an new extended string, containing a clone of the
 %  given string.
 %
 %  An extended string is the string length, plus an extra MagickPathExtent space
@@ -108,7 +108,6 @@ MagickExport char *AcquireString(const char *source)
     sizeof(*destination));
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *destination='\0';
   if (source != (char *) NULL)
     (void) memcpy(destination,source,length*sizeof(*destination));
   destination[length]='\0';
@@ -393,13 +392,13 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 MagickExport size_t ConcatenateMagickString(char *magick_restrict destination,
   const char *magick_restrict source,const size_t length)
 {
-  register char
+  char
     *magick_restrict q;
 
-  register const char
+  const char
     *magick_restrict p;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -480,8 +479,8 @@ MagickExport MagickBooleanType ConcatenateString(
   length+=source_length;
   if (~length < MagickPathExtent)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
-  *destination=(char *) ResizeQuantumMemory(*destination,length+
-    MagickPathExtent,sizeof(**destination));
+  *destination=(char *) ResizeQuantumMemory(*destination,
+    OverAllocateMemory(length+MagickPathExtent),sizeof(**destination));
   if (*destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
   if (source_length != 0)
@@ -528,8 +527,20 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
   length=string_info->length;
   if (~length < source->length)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
-  SetStringInfoLength(string_info,length+source->length);
-  (void) memcpy(string_info->datum+length,source->datum,source->length);
+  length+=source->length;
+  if (~length < MagickPathExtent)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  if (string_info->datum == (unsigned char *) NULL)
+    string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
+      MagickPathExtent,sizeof(*string_info->datum));
+  else
+    string_info->datum=(unsigned char *) ResizeQuantumMemory(
+      string_info->datum,OverAllocateMemory(length+MagickPathExtent),
+      sizeof(*string_info->datum));
+  if (string_info->datum == (unsigned char *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memcpy(string_info->datum+string_info->length,source->datum,source->length);
+  string_info->length=length;
 }
 
 /*
@@ -604,7 +615,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
     }
   else
     {
-      register size_t
+      size_t
         i;
 
       ssize_t
@@ -614,7 +625,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
       for (i=0; i < length; i+=count)
       {
         count=read(file,string+i,(size_t) MagickMin(length-i,(size_t)
-          SSIZE_MAX));
+          MAGICK_SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -680,7 +691,6 @@ MagickExport char *ConstantString(const char *source)
     destination=(char *) AcquireQuantumMemory(length+1UL,sizeof(*destination));
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *destination='\0';
   if (source != (char *) NULL)
     (void) memcpy(destination,source,length*sizeof(*destination));
   destination[length]='\0';
@@ -721,13 +731,13 @@ MagickExport char *ConstantString(const char *source)
 MagickExport size_t CopyMagickString(char *magick_restrict destination,
   const char *magick_restrict source,const size_t length)
 {
-  register char
+  char
     *magick_restrict q;
 
-  register const char
+  const char
     *magick_restrict p;
 
-  register size_t
+  size_t
     n;
 
   p=source;
@@ -842,7 +852,7 @@ MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 */
 MagickExport char **DestroyStringList(char **list)
 {
-  register ssize_t
+  ssize_t
     i;
 
   assert(list != (char **) NULL);
@@ -886,10 +896,10 @@ MagickExport char *EscapeString(const char *source,const char escape)
   char
     *destination;
 
-  register char
+  char
     *q;
 
-  register const char
+  const char
     *p;
 
   size_t
@@ -1060,7 +1070,7 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
     bytes,
     extent;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1083,11 +1093,7 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
       bytes=1024.0;
       units=bi_units;
     }
-#if defined(_MSC_VER) && (_MSC_VER == 1200)
-  extent=(double) ((MagickOffsetType) size);
-#else
   extent=(double) size;
-#endif
   (void) FormatLocaleString(p,MagickPathExtent,"%.*g",GetMagickPrecision(),
     extent);
   (void) FormatLocaleString(q,MagickPathExtent,"%.20g",extent);
@@ -1463,10 +1469,10 @@ MagickExport MagickBooleanType IsStringFalse(const char *value)
 MagickExport void PrintStringInfo(FILE *file,const char *id,
   const StringInfo *string_info)
 {
-  register const char
+  const char
     *p;
 
-  register size_t
+  size_t
     i,
     j;
 
@@ -1562,7 +1568,7 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SanitizeString() returns n new string removes all characters except
+%  SanitizeString() returns a new string with all characters removed except
 %  letters, digits and !#$%&'*+-=?^_`{|}~@.[].
 %
 %  Free the sanitized string with DestroyString().
@@ -1584,18 +1590,18 @@ MagickExport char *SanitizeString(const char *source)
   const char
     *q;
 
-  register char
+  char
     *p;
 
   static char
-    whitelist[] =
+    allowlist[] =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "
       "$-_.+!*'(),{}|\\^~[]`\"><#%;/?:@&=";
 
   sanitize_source=AcquireString(source);
   p=sanitize_source;
   q=sanitize_source+strlen(sanitize_source);
-  for (p+=strspn(p,whitelist); p != q; p+=strspn(p,whitelist))
+  for (p+=strspn(p,allowlist); p != q; p+=strspn(p,allowlist))
     *p='_';
   return(sanitize_source);
 }
@@ -1826,41 +1832,37 @@ MagickExport StringInfo *SplitStringInfo(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S t r i n g I n f o T o S t r i n g                                       %
+%   S t r i n g I n f o T o D i g e s t                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  StringInfoToString() converts a string info string to a C string.
+%  StringInfoToDigest() converts a string info string to a hex digest.
 %
 %  The format of the StringInfoToString method is:
 %
-%      char *StringInfoToString(const StringInfo *string_info)
+%      char *StringInfoToDigest(const StringInfo *signature)
 %
 %  A description of each parameter follows:
 %
 %    o string_info: the string.
 %
 */
-MagickExport char *StringInfoToString(const StringInfo *string_info)
+MagickExport char *StringInfoToDigest(const StringInfo *signature)
 {
   char
-    *string;
+    *digest;
 
-  size_t
-    length;
+  SignatureInfo
+    *signature_info;
 
-  string=(char *) NULL;
-  length=string_info->length;
-  if (~length >= (MagickPathExtent-1))
-    string=(char *) AcquireQuantumMemory(length+MagickPathExtent,
-      sizeof(*string));
-  if (string == (char *) NULL)
-    return((char *) NULL);
-  (void) memcpy(string,(char *) string_info->datum,length*sizeof(*string));
-  string[length]='\0';
-  return(string);
+  signature_info=AcquireSignatureInfo();
+  UpdateSignature(signature_info,signature);
+  FinalizeSignature(signature_info);
+  digest=StringInfoToHexString(GetSignatureDigest(signature_info));
+  signature_info=DestroySignatureInfo(signature_info);
+  return(digest);
 }
 
 /*
@@ -1890,13 +1892,13 @@ MagickExport char *StringInfoToHexString(const StringInfo *string_info)
   char
     *string;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -1945,6 +1947,48 @@ MagickExport char *StringInfoToHexString(const StringInfo *string_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S t r i n g I n f o T o S t r i n g                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringInfoToString() converts a string info string to a C string.
+%
+%  The format of the StringInfoToString method is:
+%
+%      char *StringInfoToString(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string.
+%
+*/
+MagickExport char *StringInfoToString(const StringInfo *string_info)
+{
+  char
+    *string;
+
+  size_t
+    length;
+
+  string=(char *) NULL;
+  length=string_info->length;
+  if (~length >= (MagickPathExtent-1))
+    string=(char *) AcquireQuantumMemory(length+MagickPathExtent,
+      sizeof(*string));
+  if (string == (char *) NULL)
+    return((char *) NULL);
+  (void) memcpy(string,(char *) string_info->datum,length*sizeof(*string));
+  string[length]='\0';
+  return(string);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  S t r i n g T o A r g v                                                    %
 %                                                                             %
 %                                                                             %
@@ -1979,11 +2023,11 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   char
     **argv;
 
-  register const char
+  const char
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   *argc=0;
@@ -2101,7 +2145,7 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
   double
     *array;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -2173,7 +2217,7 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
 %  delimiter character found, or NULL.  A pointer to the start of the
 %  string is returned, representing the token before the delimiter.
 %
-%  In may ways this is equivent to the strtok() C library function, but with
+%  StringToken() is similar to the strtok() C library method, but with
 %  multiple delimiter characters rather than a delimiter string.
 %
 %  The format of the StringToken method is:
@@ -2193,13 +2237,13 @@ MagickExport char *StringToken(const char *delimiters,char **string)
   char
     *q;
 
-  register char
+  char
     *p;
 
-  register const char
+  const char
     *r;
 
-  register int
+  int
     c,
     d;
 
@@ -2287,10 +2331,10 @@ MagickExport char **StringToStrings(const char *text,size_t *count)
   char
     **textlist;
 
-  register const char
+  const char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2308,7 +2352,7 @@ MagickExport char **StringToStrings(const char *text,size_t *count)
       break;
   if (*p == '\0')
     {
-      register const char
+      const char
         *q;
 
       /*
@@ -2344,10 +2388,10 @@ MagickExport char **StringToStrings(const char *text,size_t *count)
       char
         hex_string[MagickPathExtent];
 
-      register char
+      char
         *q;
 
-      register ssize_t
+      ssize_t
         j;
 
       /*
@@ -2447,27 +2491,33 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S t r i p S t r i n g                                                     %
+%   S t r i p M a g i c k S t r i n g                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  StripString() strips any whitespace or quotes from the beginning and end of
-%  a string of characters.
+%  StripMagickString() strips any whitespace or quotes from the beginning and
+%  end of a string of characters. It returns the stripped string length.
 %
-%  The format of the StripString method is:
+%  The format of the StripMagickString method is:
 %
-%      void StripString(char *message)
+%      size_t StripMagickString(char *message)
 %
 %  A description of each parameter follows:
 %
 %    o message: Specifies an array of characters.
 %
 */
+
 MagickExport void StripString(char *message)
 {
-  register char
+  (void) StripMagickString(message);
+}
+
+MagickExport size_t StripMagickString(char *message)
+{
+  char
     *p,
     *q;
 
@@ -2476,8 +2526,10 @@ MagickExport void StripString(char *message)
 
   assert(message != (char *) NULL);
   if (*message == '\0')
-    return;
+    return(0);
   length=strlen(message);
+  if (length == 1)
+    return(1);
   p=message;
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
@@ -2494,6 +2546,7 @@ MagickExport void StripString(char *message)
   for (p=message; *p != '\0'; p++)
     if (*p == '\n')
       *p=' ';
+  return((size_t) (q-p+1));
 }
 
 /*
@@ -2533,7 +2586,7 @@ MagickExport MagickBooleanType SubstituteString(char **string,
   MagickBooleanType
     status;
 
-  register char
+  char
     *p;
 
   size_t
@@ -2566,8 +2619,8 @@ MagickExport MagickBooleanType SubstituteString(char **string,
         */
         offset=(ssize_t) (p-(*string));
         extent=strlen(*string)+replace_extent-search_extent+1;
-        *string=(char *) ResizeQuantumMemory(*string,extent+MagickPathExtent,
-          sizeof(*p));
+        *string=(char *) ResizeQuantumMemory(*string,
+          OverAllocateMemory(extent+MagickPathExtent),sizeof(*p));
         if (*string == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
         p=(*string)+offset;
