@@ -17,7 +17,7 @@
 %                                 October 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,7 @@
   Include declarations.
 */
 #include "MagickCore/studio.h"
-#include "MagickCore/property.h"
+#include "MagickCore/artifact.h"
 #include "MagickCore/blob.h"
 #include "MagickCore/cache-view.h"
 #include "MagickCore/color.h"
@@ -59,6 +59,7 @@
 #include "MagickCore/effect.h"
 #include "MagickCore/fx.h"
 #include "MagickCore/gem.h"
+#include "MagickCore/gem-private.h"
 #include "MagickCore/geometry.h"
 #include "MagickCore/image-private.h"
 #include "MagickCore/list.h"
@@ -69,7 +70,7 @@
 #include "MagickCore/montage.h"
 #include "MagickCore/option.h"
 #include "MagickCore/pixel-accessor.h"
-#include "MagickCore/pixel-private.h"
+#include "MagickCore/property.h"
 #include "MagickCore/quantize.h"
 #include "MagickCore/quantum.h"
 #include "MagickCore/quantum-private.h"
@@ -241,14 +242,14 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       channel_bias[MaxPixelChannels],
       channel_sum[MaxPixelChannels];
 
-    register const Quantum
+    const Quantum
       *magick_restrict p,
       *magick_restrict pixels;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -402,7 +403,7 @@ static double KapurThreshold(const Image *image,const double *histogram,
     maximum_entropy,
     *white_entropy;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -499,7 +500,7 @@ static double OTSUThreshold(const Image *image,const double *histogram,
     *sigma,
     threshold;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -583,7 +584,7 @@ static double TriangleThreshold(const double *histogram)
     y1,
     y2;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -675,7 +676,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -698,10 +699,10 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
@@ -756,6 +757,8 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   */
   (void) FormatLocaleString(property,MagickPathExtent,"%g%%",threshold);
   (void) SetImageProperty(image,"auto-threshold:threshold",property,exception);
+  if (IsStringTrue(GetImageArtifact(image,"auto-threshold:verbose")) != MagickFalse)
+    (void) FormatLocaleFile(stdout,"%.*g%%\n",GetMagickPrecision(),threshold);
   return(BilevelImage(image,QuantumRange*threshold/100.0,exception));
 }
 
@@ -837,10 +840,10 @@ MagickExport MagickBooleanType BilevelImage(Image *image,const double threshold,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -856,7 +859,7 @@ MagickExport MagickBooleanType BilevelImage(Image *image,const double threshold,
       double
         pixel;
 
-      register ssize_t
+      ssize_t
         i;
 
       pixel=GetPixelIntensity(image,q);
@@ -998,10 +1001,10 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1017,7 +1020,7 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
       double
         pixel;
 
-      register ssize_t
+      ssize_t
         i;
 
       pixel=GetPixelIntensity(image,q);
@@ -1104,10 +1107,10 @@ MagickExport MagickBooleanType ClampImage(Image *image,ExceptionInfo *exception)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class == PseudoClass)
     {
-      register ssize_t
+      ssize_t
         i;
 
-      register PixelInfo
+      PixelInfo
         *magick_restrict q;
 
       q=image->colormap;
@@ -1133,10 +1136,10 @@ MagickExport MagickBooleanType ClampImage(Image *image,ExceptionInfo *exception)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1149,7 +1152,7 @@ MagickExport MagickBooleanType ClampImage(Image *image,ExceptionInfo *exception)
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
+      ssize_t
         i;
 
       for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
@@ -1221,12 +1224,22 @@ MagickExport MagickBooleanType ColorThresholdImage(Image *image,
   CacheView
     *image_view;
 
+  const char
+    *artifact;
+
+  IlluminantType
+    illuminant = D65Illuminant;
+
   MagickBooleanType
     status;
 
   MagickOffsetType
     progress;
 
+  PixelInfo
+    start,
+    stop;
+
   ssize_t
     y;
 
@@ -1240,6 +1253,83 @@ MagickExport MagickBooleanType ColorThresholdImage(Image *image,
   status=AcquireImageColormap(image,2,exception);
   if (status == MagickFalse)
     return(status);
+  artifact=GetImageArtifact(image,"color:illuminant");
+  if (artifact != (const char *) NULL)
+    {
+      illuminant=(IlluminantType) ParseCommandOption(MagickIlluminantOptions,
+        MagickFalse,artifact);
+      if ((ssize_t) illuminant < 0)
+        illuminant=UndefinedIlluminant;
+    }
+  start=(*start_color);
+  stop=(*stop_color);
+  switch (image->colorspace)
+  {
+    case HCLColorspace:
+    {
+      ConvertRGBToHCL(start_color->red,start_color->green,start_color->blue,
+        &start.red,&start.green,&start.blue);
+      ConvertRGBToHCL(stop_color->red,stop_color->green,stop_color->blue,
+        &stop.red,&stop.green,&stop.blue);
+      break;
+    }
+    case HSBColorspace:
+    {
+      ConvertRGBToHSB(start_color->red,start_color->green,start_color->blue,
+        &start.red,&start.green,&start.blue);
+      ConvertRGBToHSB(stop_color->red,stop_color->green,stop_color->blue,
+        &stop.red,&stop.green,&stop.blue);
+      break;
+    }
+    case HSLColorspace:
+    {
+      ConvertRGBToHSL(start_color->red,start_color->green,start_color->blue,
+        &start.red,&start.green,&start.blue);
+      ConvertRGBToHSL(stop_color->red,stop_color->green,stop_color->blue,
+        &stop.red,&stop.green,&stop.blue);
+      break;
+    }
+    case HSVColorspace:
+    {
+      ConvertRGBToHSV(start_color->red,start_color->green,start_color->blue,
+        &start.red,&start.green,&start.blue);
+      ConvertRGBToHSV(stop_color->red,stop_color->green,stop_color->blue,
+        &stop.red,&stop.green,&stop.blue);
+      break;
+    }
+    case HWBColorspace:
+    {
+      ConvertRGBToHWB(start_color->red,start_color->green,start_color->blue,
+        &start.red,&start.green,&start.blue);
+      ConvertRGBToHWB(stop_color->red,stop_color->green,stop_color->blue,
+        &stop.red,&stop.green,&stop.blue);
+      break;
+    }
+    case LabColorspace:
+    {
+      ConvertRGBToLab(start_color->red,start_color->green,start_color->blue,
+        illuminant,&start.red,&start.green,&start.blue);
+      ConvertRGBToLab(stop_color->red,stop_color->green,stop_color->blue,
+        illuminant,&stop.red,&stop.green,&stop.blue);
+      break;
+    }
+    default:
+    {
+      start.red*=QuantumScale;
+      start.green*=QuantumScale;
+      start.blue*=QuantumScale;
+      stop.red*=QuantumScale;
+      stop.green*=QuantumScale;
+      stop.blue*=QuantumScale;
+      break;
+    }
+  }
+  start.red*=QuantumRange;
+  start.green*=QuantumRange;
+  start.blue*=QuantumRange;
+  stop.red*=QuantumRange;
+  stop.green*=QuantumRange;
+  stop.blue*=QuantumRange;
   progress=0;
   image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -1248,10 +1338,10 @@ MagickExport MagickBooleanType ColorThresholdImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1267,7 +1357,7 @@ MagickExport MagickBooleanType ColorThresholdImage(Image *image,
       MagickBooleanType
         foreground = MagickTrue;
 
-      register ssize_t
+      ssize_t
         i;
 
       for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
@@ -1276,11 +1366,11 @@ MagickExport MagickBooleanType ColorThresholdImage(Image *image,
         PixelTrait traits = GetPixelChannelTraits(image,channel);
         if ((traits & UpdatePixelTrait) == 0)
           continue;
-        if ((q[i] < GetPixelInfoChannel(start_color,channel)) ||
-            (q[i] > GetPixelInfoChannel(stop_color,channel)))
+        if ((q[i] < GetPixelInfoChannel(&start,channel)) ||
+            (q[i] > GetPixelInfoChannel(&stop,channel)))
           foreground=MagickFalse;
       }
-      SetPixelIndex(image,(Quantum) foreground != MagickFalse ? 1 : 0,q);
+      SetPixelIndex(image,(Quantum) (foreground != MagickFalse ? 1 : 0),q);
       q+=GetPixelChannels(image);
     }
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
@@ -1301,6 +1391,7 @@ MagickExport MagickBooleanType ColorThresholdImage(Image *image,
       }
   }
   image_view=DestroyCacheView(image_view);
+  image->colorspace=sRGBColorspace;
   return(SyncImage(image,exception));
 }
 
@@ -1441,7 +1532,7 @@ static ThresholdMap *GetThresholdMapFile(const char *xml,const char *filename,
   double
     value;
 
-  register ssize_t
+  ssize_t
     i;
 
   ThresholdMap
@@ -1818,10 +1909,8 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image,
   MagickOffsetType
     progress;
 
-  register ssize_t
-    i;
-
   ssize_t
+    i,
     y;
 
   ThresholdMap
@@ -1886,10 +1975,10 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1902,20 +1991,18 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image,
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
-        i;
-
       ssize_t
+        j,
         n;
 
       n=0;
-      for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
+      for (j=0; j < (ssize_t) GetPixelChannels(image); j++)
       {
         ssize_t
           level,
           threshold;
 
-        PixelChannel channel = GetPixelChannelChannel(image,i);
+        PixelChannel channel = GetPixelChannelChannel(image,j);
         PixelTrait traits = GetPixelChannelTraits(image,channel);
         if ((traits & UpdatePixelTrait) == 0)
           continue;
@@ -1924,10 +2011,10 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image,
             n++;
             continue;
           }
-        threshold=(ssize_t) (QuantumScale*q[i]*(levels[n]*(map->divisor-1)+1));
+        threshold=(ssize_t) (QuantumScale*q[j]*(levels[n]*(map->divisor-1)+1));
         level=threshold/(map->divisor-1);
         threshold-=level*(map->divisor-1);
-        q[i]=ClampToQuantum((double) (level+(threshold >=
+        q[j]=ClampToQuantum((double) (level+(threshold >=
           map->levels[(x % map->width)+map->width*(y % map->height)]))*
           QuantumRange/levels[n]);
         n++;
@@ -2020,10 +2107,10 @@ MagickExport MagickBooleanType PerceptibleImage(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class == PseudoClass)
     {
-      register ssize_t
+      ssize_t
         i;
 
-      register PixelInfo
+      PixelInfo
         *magick_restrict q;
 
       q=image->colormap;
@@ -2053,10 +2140,10 @@ MagickExport MagickBooleanType PerceptibleImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2069,7 +2156,7 @@ MagickExport MagickBooleanType PerceptibleImage(Image *image,
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
+      ssize_t
         i;
 
       for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
@@ -2147,9 +2234,6 @@ MagickExport MagickBooleanType RandomThresholdImage(Image *image,
   MagickOffsetType
     progress;
 
-  PixelInfo
-    threshold;
-
   RandomInfo
     **magick_restrict random_info;
 
@@ -2169,7 +2253,6 @@ MagickExport MagickBooleanType RandomThresholdImage(Image *image,
   assert(exception->signature == MagickCoreSignature);
   if (SetImageStorageClass(image,DirectClass,exception) == MagickFalse)
     return(MagickFalse);
-  GetPixelInfo(image,&threshold);
   /*
     Random threshold image.
   */
@@ -2187,10 +2270,10 @@ MagickExport MagickBooleanType RandomThresholdImage(Image *image,
     const int
       id = GetOpenMPThreadId();
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -2203,7 +2286,7 @@ MagickExport MagickBooleanType RandomThresholdImage(Image *image,
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
+      ssize_t
         i;
 
       for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
@@ -2321,10 +2404,10 @@ MagickExport MagickBooleanType RangeThresholdImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2340,7 +2423,7 @@ MagickExport MagickBooleanType RangeThresholdImage(Image *image,
       double
         pixel;
 
-      register ssize_t
+      ssize_t
         i;
 
       pixel=GetPixelIntensity(image,q);
@@ -2353,7 +2436,7 @@ MagickExport MagickBooleanType RangeThresholdImage(Image *image,
         if (image->channel_mask != DefaultChannels)
           pixel=(double) q[i];
         if (pixel < low_black)
-          q[i]=0;
+          q[i]=(Quantum) 0;
         else
           if ((pixel >= low_black) && (pixel < low_white))
             q[i]=ClampToQuantum(QuantumRange*
@@ -2367,9 +2450,9 @@ MagickExport MagickBooleanType RangeThresholdImage(Image *image,
                   high_black-high_white)*(high_black-pixel));
               else
                 if (pixel > high_black)
-                  q[i]=0;
+                  q[i]=(Quantum) 0;
                 else
-                  q[i]=0;
+                  q[i]=(Quantum) 0;
       }
       q+=GetPixelChannels(image);
     }
@@ -2499,10 +2582,10 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2518,7 +2601,7 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
       double
         pixel;
 
-      register ssize_t
+      ssize_t
         i;
 
       pixel=GetPixelIntensity(image,q);
