@@ -17,7 +17,7 @@
 %                                July 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -304,7 +304,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
   QuantizeInfo
     *quantize_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -398,7 +398,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       nostdin=MagickTrue;
     if ((LocaleCompare("help",option+1) == 0) ||
         (LocaleCompare("-help",option+1) == 0))
-      return(DisplayUsage());
+      {
+        DestroyDisplay();
+        return(DisplayUsage());
+      }
   }
   /*
     Get user defaults from X resource database.
@@ -1497,7 +1500,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             if (i == (ssize_t) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
             if (XRemoteCommand(display,resource_info.window_id,argv[i]) != 0)
-              return(MagickFalse);
+              {
+                DestroyDisplay();
+                return(MagickFalse);
+              }
             i--;
             break;
           }
@@ -1857,7 +1863,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
                 p=fgets(answer,(int) sizeof(answer),stdin);
                 (void) p;
                 if (((*answer != 'y') && (*answer != 'Y')))
-                  return(MagickFalse);
+                  {
+                    DestroyDisplay();
+                    return(MagickFalse);
+                  }
               }
             break;
           }
