@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.  You may
@@ -16,7 +16,8 @@
 #ifndef MAGICK_BYTE_BUFFER_PRIVATE_H
 #define MAGICK_BYTE_BUFFER_PRIVATE_H
 
-#include "magick/blob.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
 
 typedef struct _MagickByteBuffer
 {
@@ -31,7 +32,7 @@ typedef struct _MagickByteBuffer
     data[MagickMinBufferExtent];
 } MagickByteBuffer;
 
-static inline int ReadMagickByteBuffer(MagickByteBuffer *buffer)
+static inline int PeekMagickByteBuffer(MagickByteBuffer *buffer)
 {
   if ((buffer->offset == buffer->count) && (buffer->offset > 0))
     {
@@ -47,7 +48,18 @@ static inline int ReadMagickByteBuffer(MagickByteBuffer *buffer)
       if (buffer->count < 1)
         return(EOF);
     }
-  return(buffer->data[buffer->offset++]);
+  return((int) buffer->data[buffer->offset]);
+}
+
+static inline int ReadMagickByteBuffer(MagickByteBuffer *buffer)
+{
+  int
+    result;
+
+  result=PeekMagickByteBuffer(buffer);
+  if (result != EOF)
+    buffer->offset++;
+  return(result);
 }
 
 static inline char *GetMagickByteBufferDatum(MagickByteBuffer *buffer)
@@ -94,8 +106,24 @@ static inline void SkipMagickByteBuffer(MagickByteBuffer *buffer,
   const size_t length)
 {
   CheckMagickByteBuffer(buffer,length);
-  if ((buffer->offset+length) < buffer->count)
+  if ((ssize_t) (buffer->offset+length) < buffer->count)
     buffer->offset+=length;
 }
 
+static inline MagickBooleanType SkipMagickByteBufferUntilNewline(
+  MagickByteBuffer *buffer)
+{
+  int
+    c;
+
+  c=ReadMagickByteBuffer(buffer);
+  while ((c != '\n') && (c != '\r'))
+  {
+    c=ReadMagickByteBuffer(buffer);
+    if (c == EOF)
+      return(MagickFalse);
+  }
+  return(MagickTrue);
+}
+
 #endif
