@@ -17,13 +17,13 @@
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/license/                                         %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,31 +39,30 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/annotate.h"
-#include "magick/artifact.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/composite-private.h"
-#include "magick/draw.h"
-#include "magick/draw-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/resource_.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/annotate.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/composite-private.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/draw-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/option.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -124,7 +123,6 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     *image;
 
   MagickBooleanType
-    left_bearing,
     split,
     status;
 
@@ -143,32 +141,34 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickCoreSignature);
-  if (IsEventLogging() != MagickFalse)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  image=AcquireImage(image_info);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   (void) ResetImagePage(image,"0x0+0+0");
   if ((image->columns != 0) && (image->rows != 0))
-    (void) SetImageBackgroundColor(image);
+    (void) SetImageBackgroundColor(image,exception);
   /*
     Format caption.
   */
   option=GetImageOption(image_info,"filename");
   if (option == (const char *) NULL)
     caption=InterpretImageProperties((ImageInfo *) image_info,image,
-      image_info->filename);
+      image_info->filename,exception);
   else
     if (LocaleNCompare(option,"caption:",8) == 0)
-      caption=InterpretImageProperties((ImageInfo *) image_info,image,option+8);
+      caption=InterpretImageProperties((ImageInfo *) image_info,image,option+8,
+        exception);
     else
-      caption=InterpretImageProperties((ImageInfo *) image_info,image,option);
+      caption=InterpretImageProperties((ImageInfo *) image_info,image,option,
+        exception);
   if (caption == (char *) NULL)
     return(DestroyImageList(image));
-  (void) SetImageProperty(image,"caption",caption);
+  (void) SetImageProperty(image,"caption",caption,exception);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  width=CastDoubleToUnsigned(0.5*draw_info->pointsize*strlen(caption)+0.5);
+  width=(size_t) floor(0.5*draw_info->pointsize*strlen(caption)+0.5);
   if (AcquireMagickResource(WidthResource,width) == MagickFalse)
     {
       caption=DestroyString(caption);
@@ -186,7 +186,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   if (image->columns == 0)
     {
       text=AcquireString(caption);
-      i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+      i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
       AdjustTypeMetricBounds(&metrics);
       (void) CloneString(&draw_info->text,text);
       text=DestroyString(text);
@@ -194,7 +194,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         metrics.bounds.x1,metrics.ascent);
       if (draw_info->gravity == UndefinedGravity)
         (void) CloneString(&draw_info->geometry,geometry);
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
       AdjustTypeMetricBounds(&metrics);
       image->columns=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
     }
@@ -202,7 +202,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     {
       split=MagickTrue;
       text=AcquireString(caption);
-      i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+      i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
       AdjustTypeMetricBounds(&metrics);
       (void) CloneString(&draw_info->text,text);
       text=DestroyString(text);
@@ -210,20 +210,20 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         metrics.bounds.x1,metrics.ascent);
       if (draw_info->gravity == UndefinedGravity)
         (void) CloneString(&draw_info->geometry,geometry);
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
       AdjustTypeMetricBounds(&metrics);
       image->rows=(size_t) ((i+1)*(metrics.ascent-metrics.descent+
         draw_info->interline_spacing+draw_info->stroke_width)+0.5);
     }
   if (status != MagickFalse)
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     { 
       caption=DestroyString(caption);
       draw_info=DestroyDrawInfo(draw_info);
       return(DestroyImageList(image));
     }
-  if (SetImageBackgroundColor(image) == MagickFalse)
+  if (SetImageBackgroundColor(image,exception) == MagickFalse)
     {
       caption=DestroyString(caption);
       draw_info=DestroyDrawInfo(draw_info);
@@ -263,7 +263,8 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
           for (n=0; n < 32; n++, draw_info->pointsize*=2.0)
           {
             text=AcquireString(caption);
-            i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+            i=FormatMagickCaption(image,draw_info,split,&metrics,&text,
+              exception);
             AdjustTypeMetricBounds(&metrics);
             (void) CloneString(&draw_info->text,text);
             text=DestroyString(text);
@@ -271,25 +272,23 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
               metrics.bounds.x1,metrics.ascent);
             if (draw_info->gravity == UndefinedGravity)
               (void) CloneString(&draw_info->geometry,geometry);
-            status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+            status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
             if (status == MagickFalse)
               break;
             AdjustTypeMetricBounds(&metrics);
-            width=CastDoubleToUnsigned(metrics.width+draw_info->stroke_width+
-              0.5);
-            height=CastDoubleToUnsigned(
-              metrics.height-metrics.underline_position+
-              draw_info->interline_spacing+draw_info->stroke_width+0.5);
+            width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+            height=(size_t) floor(metrics.height+draw_info->interline_spacing+
+              draw_info->stroke_width+0.5);
             if ((image->columns != 0) && (image->rows != 0))
               {
-                if ((width > image->columns) && (height > image->rows))
+                if ((width >= image->columns) || (height >= image->rows))
                   break;
-                if ((width <= image->columns) && (height <= image->rows))
+                if ((width < image->columns) && (height < image->rows))
                   low=draw_info->pointsize;
               }
             else
-              if (((image->columns != 0) && (width > image->columns)) ||
-                  ((image->rows != 0) && (height > image->rows)))
+              if (((image->columns != 0) && (width >= image->columns)) ||
+                  ((image->rows != 0) && (height >= image->rows)))
                 break;
           }
           high=draw_info->pointsize;
@@ -298,7 +297,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
       {
         draw_info->pointsize=(low+high)/2.0;
         text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+        i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
         AdjustTypeMetricBounds(&metrics);
         (void) CloneString(&draw_info->text,text);
         text=DestroyString(text);
@@ -306,49 +305,42 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
           metrics.bounds.x1,metrics.ascent);
         if (draw_info->gravity == UndefinedGravity)
           (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+        status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
         if (status == MagickFalse)
           break;
         AdjustTypeMetricBounds(&metrics);
-        width=CastDoubleToUnsigned(metrics.width+draw_info->stroke_width+0.5);
-        height=CastDoubleToUnsigned(metrics.height-metrics.underline_position+
-          draw_info->interline_spacing+draw_info->stroke_width+0.5);
+        width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+        height=(size_t) floor(metrics.height+draw_info->interline_spacing+
+          draw_info->stroke_width+0.5);
         if ((image->columns != 0) && (image->rows != 0))
           {
-            if ((width <= image->columns) && (height <= image->rows))
+            if ((width < image->columns) && (height < image->rows))
               low=draw_info->pointsize+0.5;
             else
               high=draw_info->pointsize-0.5;
           }
         else
-          if (((image->columns != 0) && (width <= image->columns)) ||
-              ((image->rows != 0) && (height <= image->rows)))
+          if (((image->columns != 0) && (width < image->columns)) ||
+              ((image->rows != 0) && (height < image->rows)))
             low=draw_info->pointsize+0.5;
           else
             high=draw_info->pointsize-0.5;
       }
-      if (status != MagickFalse)
-        draw_info->pointsize=floor(low-0.5);
+      draw_info->pointsize=floor((low+high)/2.0-0.5);
     }
   /*
     Draw caption.
   */
-  i=FormatMagickCaption(image,draw_info,split,&metrics,&caption);
+  i=FormatMagickCaption(image,draw_info,split,&metrics,&caption,exception);
   AdjustTypeMetricBounds(&metrics);
   (void) CloneString(&draw_info->text,caption);
   caption=DestroyString(caption);
-  left_bearing=((draw_info->gravity == UndefinedGravity) ||
-     (draw_info->gravity == NorthWestGravity) || 
-     (draw_info->gravity == WestGravity) ||
-     (draw_info->gravity == SouthWestGravity)) ? MagickTrue : MagickFalse;
   (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
     (draw_info->direction == RightToLeftDirection ? (double) image->columns-
-    (draw_info->gravity == UndefinedGravity ? metrics.bounds.x2 : 0.0) : 
-    (left_bearing != MagickFalse ? metrics.bounds.x1 : 0.0)),
-    (draw_info->gravity == UndefinedGravity ? 
-    MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0));
+    metrics.bounds.x2 : metrics.bounds.x1),(draw_info->gravity ==
+    UndefinedGravity ? MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0));
   (void) CloneString(&draw_info->geometry,geometry);
-  status=AnnotateImage(image,draw_info);
+  status=AnnotateImage(image,draw_info,exception);
   if (image_info->pointsize == 0.0)
     (void) FormatImageProperty(image,"caption:pointsize","%.*g",
       GetMagickPrecision(),draw_info->pointsize);
@@ -391,11 +383,9 @@ ModuleExport size_t RegisterCAPTIONImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("CAPTION");
+  entry=AcquireMagickInfo("CAPTION","CAPTION","Caption");
   entry->decoder=(DecodeImageHandler *) ReadCAPTIONImage;
-  entry->description=ConstantString("Caption");
-  entry->adjoin=MagickFalse;
-  entry->magick_module=ConstantString("CAPTION");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
