@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -61,12 +61,15 @@
 #if defined(MAGICKCORE_OPENEXR_DELEGATE)
   #include "coders/exr.h"
 #endif
+#include "coders/farbfeld.h"
 #include "coders/fax.h"
 #include "coders/fits.h"
+#include "coders/fl32.h"
 #if defined(MAGICKCORE_FLIF_DELEGATE)
   #include "coders/flif.h"
 #endif
 #include "coders/fpx.h"
+#include "coders/ftxt.h"
 #include "coders/gif.h"
 #include "coders/gradient.h"
 #include "coders/gray.h"
@@ -137,6 +140,7 @@
 #include "coders/ps.h"
 #include "coders/psd.h"
 #include "coders/pwp.h"
+#include "coders/qoi.h"
 #include "coders/raw.h"
 #include "coders/rgb.h"
 #include "coders/rgf.h"
@@ -191,5 +195,6 @@
 #if defined(MAGICKCORE_X11_DELEGATE)
   #include "coders/xwd.h"
 #endif
+#include "coders/yaml.h"
 #include "coders/ycbcr.h"
 #include "coders/yuv.h"
