@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -73,7 +73,8 @@
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
 #include <libraw.h>
 #endif
-
+
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -105,9 +106,6 @@
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(MAGICKCORE_OPENCL_SUPPORT)
 static void InitializeDcrawOpenCL(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    opencl_disabled;
-
   MagickCLDevice
     *devices;
 
@@ -163,12 +161,12 @@ static void SetDNGProperties(Image *image,const libraw_data_t *raw_info,
   ExceptionInfo *exception)
 {
   char
-    timestamp[MagickPathExtent];
+    timestamp[MagickTimeExtent];
 
   (void) SetImageProperty(image,"dng:make",raw_info->idata.make,exception);
   (void) SetImageProperty(image,"dng:camera.model.name",raw_info->idata.model,
     exception);
-  (void) FormatMagickTime(raw_info->other.timestamp,MagickPathExtent,timestamp);
+  (void) FormatMagickTime(raw_info->other.timestamp,sizeof(timestamp),timestamp);
   (void) SetImageProperty(image,"dng:create.date",timestamp,exception);
   (void) FormatImageProperty(image,"dng:iso.setting","%0.1f",
     raw_info->other.iso_speed);
@@ -291,7 +289,7 @@ static Image *InvokeDNGDelegate(const ImageInfo *image_info,Image *image,
                 (void) FormatLocaleString(property,MagickPathExtent,"dng:%s",
                   tag);
                 content=ConstantString(GetXMLTreeContent(next));
-                StripString(content);
+                (void) StripMagickString(content);
                 if ((LocaleCompare(tag,"log") != 0) &&
                     (LocaleCompare(tag,"InputFilename") != 0) &&
                     (LocaleCompare(tag,"OutputFilename") != 0) &&
@@ -343,7 +341,67 @@ static void SetLibRawParams(const ImageInfo *image_info,Image *image,
       if (raw_info->params.output_color == 5)
         image->colorspace=XYZColorspace;
     }
+  option=GetImageOption(image_info,"dng:interpolation-quality");
+  if (option != (const char *) NULL)
+    {
+      int
+        value;
+
+      value=StringToInteger(option);
+      if (value == -1)
+        raw_info->params.no_interpolation=1;
+      else
+        raw_info->params.user_qual=value;
+    }
 }
+
+static void LibRawDataError(void *data,const char *magick_unused(file),
+  const int offset)
+{
+  magick_unreferenced(file);
+  /* Value below zero is an EOF and an exception will be raised instead */
+  if (offset >= 0)
+    {
+      ExceptionInfo
+        *exception;
+
+      exception=(ExceptionInfo *) data;
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        CorruptImageWarning,"Data corrupted at","`%d'",offset);
+  }
+}
+
+static void ReadLibRawThumbnail(const ImageInfo *image_info,Image *image,
+  libraw_data_t *raw_info,ExceptionInfo *exception)
+{
+  const char
+    *option;
+
+  int
+    errcode;
+
+  libraw_processed_image_t
+    *thumbnail;
+
+  option=GetImageOption(image_info,"dng:read-thumbnail");
+  if (IsStringTrue(option) == MagickFalse)
+    return;
+  errcode=libraw_unpack_thumb(raw_info);
+  if (errcode != LIBRAW_SUCCESS)
+    return;
+  thumbnail=libraw_dcraw_make_mem_thumb(raw_info,&errcode);
+  if (errcode == LIBRAW_SUCCESS)
+    {
+      StringInfo
+        *profile;
+
+      profile=BlobToStringInfo(thumbnail->data,thumbnail->data_size);
+      (void) SetImageProfile(image,"dng:thumbnail",profile,exception);
+    }
+  if (thumbnail != (libraw_processed_image_t *) NULL)
+    libraw_dcraw_clear_mem(thumbnail);
+}
+
 #endif
 
 static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
@@ -385,7 +443,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     libraw_processed_image_t
       *raw_image;
 
-    register ssize_t
+    ssize_t
       y;
 
     StringInfo
@@ -395,7 +453,8 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       *p;
 
     errcode=0;
-    raw_info=libraw_init(0);
+    raw_info=libraw_init(LIBRAW_OPIONS_NO_MEMERR_CALLBACK |
+      LIBRAW_OPIONS_NO_DATAERR_CALLBACK);
     if (raw_info == (libraw_data_t *) NULL)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
@@ -403,6 +462,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         libraw_close(raw_info);
         return(DestroyImageList(image));
       }
+    libraw_set_dataerror_handler(raw_info,LibRawDataError,exception);
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_MSC_VER) && (_MSC_VER > 1310)
     {
       wchar_t
@@ -428,6 +488,7 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->page.height=raw_info->sizes.raw_height;
     image->page.x=raw_info->sizes.left_margin;
     image->page.y=raw_info->sizes.top_margin;
+    ReadLibRawThumbnail(image_info,image,raw_info,exception);
     if (image_info->ping != MagickFalse)
       {
         libraw_close(raw_info);
@@ -482,10 +543,10 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     p=(unsigned short *) raw_image->data;
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      register Quantum
+      Quantum
         *q;
 
-      register ssize_t
+      ssize_t
         x;
 
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
@@ -544,7 +605,8 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   return(InvokeDNGDelegate(image_info,image,exception));
 #endif
 }
-
+
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -727,7 +789,8 @@ ModuleExport size_t RegisterDNGImage(void)
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
-
+
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
