@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,33 +39,32 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/color.h"
-#include "magick/constitute.h"
-#include "magick/draw.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/delegate.h"
-#include "magick/geometry.h"
-#include "magick/histogram.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantize.h"
-#include "magick/quantum-private.h"
-#include "magick/resource_.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/transform.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/color.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/delegate.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/histogram.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/quantize.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/resize.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
 
 /*
   Typedef declarations.
@@ -92,7 +91,7 @@ typedef struct _EPTInfo
   Forward declarations.
 */
 static MagickBooleanType
-  WriteEPTImage(const ImageInfo *,Image *);
+  WriteEPTImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -157,11 +156,16 @@ static MagickBooleanType IsEPT(const unsigned char *magick,const size_t length)
 */
 static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  const void
+    *postscript_data,
+    *tiff_data;
+
   EPTInfo
     ept_info;
 
   Image
-    *image;
+    *image,
+    *tiff_image;
 
   ImageInfo
     *read_info;
@@ -185,7 +189,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -197,20 +201,20 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   ept_info.postscript_offset=(MagickOffsetType) ReadBlobLSBLong(image);
   ept_info.postscript_length=ReadBlobLSBLong(image);
-  if (ept_info.postscript_length > GetBlobSize(image))
+  if ((MagickSizeType) ept_info.postscript_length > GetBlobSize(image))
     ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
   (void) ReadBlobLSBLong(image);
   (void) ReadBlobLSBLong(image);
   ept_info.tiff_offset=(MagickOffsetType) ReadBlobLSBLong(image);
   ept_info.tiff_length=ReadBlobLSBLong(image);
-  if (ept_info.tiff_length > GetBlobSize(image))
+  if ((MagickSizeType) ept_info.tiff_length > GetBlobSize(image))
     ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
   (void) ReadBlobLSBShort(image);
   ept_info.postscript=(unsigned char *) AcquireQuantumMemory(
     ept_info.postscript_length+1,sizeof(*ept_info.postscript));
   if (ept_info.postscript == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(ept_info.postscript,0,(ept_info.postscript_length+1)*
+  (void) memset(ept_info.postscript,0,(ept_info.postscript_length+1)*
     sizeof(*ept_info.postscript));
   ept_info.tiff=(unsigned char *) AcquireQuantumMemory(ept_info.tiff_length+1,
     sizeof(*ept_info.tiff));
@@ -220,7 +224,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ept_info.postscript);
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  (void) ResetMagickMemory(ept_info.tiff,0,(ept_info.tiff_length+1)*
+  (void) memset(ept_info.tiff,0,(ept_info.tiff_length+1)*
     sizeof(*ept_info.tiff));
   offset=SeekBlob(image,ept_info.tiff_offset,SEEK_SET);
   if ((ept_info.tiff_length != 0) && (offset < 30))
@@ -230,7 +234,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ept_info.postscript);
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
-  count=ReadBlob(image,ept_info.tiff_length,ept_info.tiff);
+  tiff_data=ReadBlobStream(image,ept_info.tiff_length,ept_info.tiff,&count);
   if (count != (ssize_t) (ept_info.tiff_length))
     (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageWarning,
       "InsufficientImageDataInFile","`%s'",image->filename);
@@ -242,32 +246,43 @@ static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ept_info.postscript);
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
-  count=ReadBlob(image,ept_info.postscript_length,ept_info.postscript);
+  postscript_data=ReadBlobStream(image,ept_info.postscript_length,
+    ept_info.postscript,&count);
   if (count != (ssize_t) (ept_info.postscript_length))
-    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageWarning,
-      "InsufficientImageDataInFile","`%s'",image->filename);
+    {
+      ept_info.tiff=(unsigned char *) RelinquishMagickMemory(ept_info.tiff);
+      ept_info.postscript=(unsigned char *) RelinquishMagickMemory(
+        ept_info.postscript);
+      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    }
   (void) CloseBlob(image);
   image=DestroyImage(image);
   read_info=CloneImageInfo(image_info);
-  (void) CopyMagickString(read_info->magick,"EPS",MaxTextExtent);
-  image=BlobToImage(read_info,ept_info.postscript,ept_info.postscript_length,
+  read_info->number_scenes=1;
+  read_info->scene=0;
+  (void) CopyMagickString(read_info->magick,"EPS",MagickPathExtent);
+  image=BlobToImage(read_info,postscript_data,ept_info.postscript_length,
     exception);
-  if (image == (Image *) NULL)
-    {
-      (void) CopyMagickString(read_info->magick,"TIFF",MaxTextExtent);
-      image=BlobToImage(read_info,ept_info.tiff,ept_info.tiff_length,exception);
-    }
-  read_info=DestroyImageInfo(read_info);
   if (image != (Image *) NULL)
     {
       (void) CopyMagickString(image->filename,image_info->filename,
-        MaxTextExtent);
-      (void) CopyMagickString(image->magick,"EPT",MaxTextExtent);
+        MagickPathExtent);
+      (void) CopyMagickString(image->magick,"EPT",MagickPathExtent);
     }
+  (void) CopyMagickString(read_info->magick,"TIFF",MagickPathExtent);
+  tiff_image=BlobToImage(read_info,tiff_data,ept_info.tiff_length,exception);
+  if (tiff_image != (Image *) NULL)
+    {
+      if (image == (Image *) NULL)
+        image=tiff_image;
+      else
+        AppendImageToList(&image,tiff_image);
+    }
+  read_info=DestroyImageInfo(read_info);
   ept_info.tiff=(unsigned char *) RelinquishMagickMemory(ept_info.tiff);
   ept_info.postscript=(unsigned char *) RelinquishMagickMemory(
     ept_info.postscript);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -298,37 +313,31 @@ ModuleExport size_t RegisterEPTImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("EPT");
-  entry->decoder=(DecodeImageHandler *) ReadEPTImage;
-  entry->encoder=(EncodeImageHandler *) WriteEPTImage;
-  entry->magick=(IsImageFormatHandler *) IsEPT;
-  entry->seekable_stream=MagickTrue;
-  entry->adjoin=MagickFalse;
-  entry->blob_support=MagickFalse;
-  entry->description=ConstantString(
+  entry=AcquireMagickInfo("EPT","EPT",
     "Encapsulated PostScript with TIFF preview");
-  entry->module=ConstantString("EPT");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("EPT2");
   entry->decoder=(DecodeImageHandler *) ReadEPTImage;
   entry->encoder=(EncodeImageHandler *) WriteEPTImage;
   entry->magick=(IsImageFormatHandler *) IsEPT;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->description=ConstantString(
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags^=CoderBlobSupportFlag;
+  (void) RegisterMagickInfo(entry);
+  entry=AcquireMagickInfo("EPT","EPT2",
     "Encapsulated PostScript Level II with TIFF preview");
-  entry->module=ConstantString("EPT");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("EPT3");
   entry->decoder=(DecodeImageHandler *) ReadEPTImage;
   entry->encoder=(EncodeImageHandler *) WriteEPTImage;
   entry->magick=(IsImageFormatHandler *) IsEPT;
-  entry->seekable_stream=MagickTrue;
-  entry->blob_support=MagickFalse;
-  entry->description=ConstantString(
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
+  (void) RegisterMagickInfo(entry);
+  entry=AcquireMagickInfo("EPT","EPT3",
     "Encapsulated PostScript Level III with TIFF preview");
-  entry->module=ConstantString("EPT");
+  entry->decoder=(DecodeImageHandler *) ReadEPTImage;
+  entry->encoder=(EncodeImageHandler *) WriteEPTImage;
+  entry->magick=(IsImageFormatHandler *) IsEPT;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -373,7 +382,8 @@ ModuleExport void UnregisterEPTImage(void)
 %
 %  The format of the WriteEPTImage method is:
 %
-%      MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteEPTImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -381,11 +391,14 @@ ModuleExport void UnregisterEPTImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   char
-     filename[MaxTextExtent];
+    filename[MagickPathExtent];
 
   EPTInfo
     ept_info;
@@ -408,36 +421,56 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  write_image=CloneImage(image,0,0,MagickTrue,&image->exception);
+  write_image=CloneImage(image,0,0,MagickTrue,exception);
   if (write_image == (Image *) NULL)
     return(MagickFalse);
   write_info=CloneImageInfo(image_info);
-  *write_info->magick='\0';
-  (void) CopyMagickString(write_info->magick,"EPS",MaxTextExtent);
+  (void) CopyMagickString(write_info->filename,"EPS:",MagickPathExtent);
+  (void) CopyMagickString(write_info->magick,"EPS",MagickPathExtent);
   if (LocaleCompare(image_info->magick,"EPT2") == 0)
-    (void) CopyMagickString(write_info->magick,"EPS2",MaxTextExtent);
+    {
+      (void) CopyMagickString(write_info->filename,"EPS2:",MagickPathExtent);
+      (void) CopyMagickString(write_info->magick,"EPS2",MagickPathExtent);
+    }
   if (LocaleCompare(image_info->magick,"EPT3") == 0)
-    (void) CopyMagickString(write_info->magick,"EPS3",MaxTextExtent);
-  (void) ResetMagickMemory(&ept_info,0,sizeof(ept_info));
+    {
+      (void) CopyMagickString(write_info->filename,"EPS3:",MagickPathExtent);
+      (void) CopyMagickString(write_info->magick,"EPS3",MagickPathExtent);
+    }
+  (void) memset(&ept_info,0,sizeof(ept_info));
   ept_info.magick=0xc6d3d0c5ul;
   ept_info.postscript=(unsigned char *) ImageToBlob(write_info,write_image,
-    &ept_info.postscript_length,&image->exception);
+    &ept_info.postscript_length,exception);
   write_image=DestroyImage(write_image);
   write_info=DestroyImageInfo(write_info);
   if (ept_info.postscript == (void *) NULL)
     return(MagickFalse);
-  write_image=CloneImage(image,0,0,MagickTrue,&image->exception);
+  write_image=CloneImage(image,0,0,MagickTrue,exception);
   if (write_image == (Image *) NULL)
     return(MagickFalse);
   write_info=CloneImageInfo(image_info);
-  (void) CopyMagickString(write_info->magick,"TIFF",MaxTextExtent);
-  (void) FormatLocaleString(filename,MaxTextExtent,"tiff:%s",
+  (void) CopyMagickString(write_info->magick,"TIFF",MagickPathExtent);
+  (void) FormatLocaleString(filename,MagickPathExtent,"tiff:%s",
     write_info->filename);
-  (void) CopyMagickString(write_info->filename,filename,MaxTextExtent);
-  (void) TransformImage(&write_image,(char *) NULL,"512x512>");
+  (void) CopyMagickString(write_info->filename,filename,MagickPathExtent);
+  if ((write_image->columns > 512) || (write_image->rows > 512))
+    {
+      Image
+        *resize_image;
+
+      resize_image=ResizeImage(write_image,512,512,write_image->filter,
+        exception);
+      if (resize_image != (Image *) NULL)
+        {
+          write_image=DestroyImage(write_image);
+          write_image=resize_image;
+        }
+    }
   if ((write_image->storage_class == DirectClass) ||
       (write_image->colors > 256))
     {
@@ -448,13 +481,13 @@ static MagickBooleanType WriteEPTImage(const ImageInfo *image_info,Image *image)
         EPT preview requires that the image is colormapped.
       */
       GetQuantizeInfo(&quantize_info);
-      quantize_info.dither=IsPaletteImage(write_image,&image->exception) ==
-        MagickFalse ? MagickTrue : MagickFalse;
-      (void) QuantizeImage(&quantize_info,write_image);
+      quantize_info.dither_method=IdentifyPaletteImage(write_image,
+        exception) == MagickFalse ? RiemersmaDitherMethod : NoDitherMethod;
+      (void) QuantizeImage(&quantize_info,write_image,exception);
     }
   write_info->compression=NoCompression;
   ept_info.tiff=(unsigned char *) ImageToBlob(write_info,write_image,
-    &ept_info.tiff_length,&image->exception);
+    &ept_info.tiff_length,exception);
   write_image=DestroyImage(write_image);
   write_info=DestroyImageInfo(write_info);
   if (ept_info.tiff == (void *) NULL)
