@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -65,7 +65,7 @@
 #include "MagickCore/statistic.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
-#include "MagickCore/module.h"
+#include "coders/coders-private.h"
 
 /*
   Forward declarations.
@@ -179,7 +179,7 @@ static MagickOffsetType GetFITSPixelExtrema(Image *image,
   MagickSizeType
     number_pixels;
 
-  register MagickOffsetType
+  MagickOffsetType
     i;
 
   offset=TellBlob(image);
@@ -207,7 +207,7 @@ static inline double GetFITSPixelRange(const size_t depth)
 static void SetFITSUnsignedPixels(const size_t length,
   const size_t bits_per_pixel,const EndianType endian,unsigned char *pixels)
 {
-  register ssize_t
+  ssize_t
     i;
 
   if (endian != MSBEndian)
@@ -270,11 +270,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register Quantum
+  Quantum
     *q;
 
   ssize_t
@@ -308,6 +308,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
+  fits_info.number_axes=1;
   fits_info.number_planes=1;
   fits_info.min_data=0.0;
   fits_info.max_data=0.0;
@@ -321,7 +322,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   {
     for ( ; EOFBlob(image) == MagickFalse; )
     {
-      register char
+      char
         *p;
 
       count=ReadBlob(image,8,(unsigned char *) keyword);
@@ -399,6 +400,13 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           comment=DestroyString(comment);
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       }
+    if ((fits_info.columns <= 0) || (fits_info.rows <= 0) ||
+        (fits_info.number_axes <= 0) || (fits_info.number_planes <= 0))
+      {
+        if (comment != (char *) NULL)
+          comment=DestroyString(comment);
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      }
     number_pixels=(MagickSizeType) fits_info.columns*fits_info.rows;
     if ((fits_info.simple != MagickFalse) && (fits_info.number_axes >= 1) &&
         (fits_info.number_axes <= 4) && (number_pixels != 0))
@@ -625,7 +633,7 @@ static inline void CopyFitsRecord(char *buffer,const char *data,
     return;
   length=MagickMin(strlen(data),80);
   if (length > (size_t) (FITSBlocksize-offset))
-    length=FITSBlocksize-offset;
+    length=(size_t) (FITSBlocksize-offset);
   (void) strncpy(buffer+offset,data,length);
 }
 
@@ -634,18 +642,18 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
 {
   char
     *fits_info,
-    header[FITSBlocksize],
-    *url;
+    header[FITSBlocksize];
+
+  const Quantum
+    *p;
 
   MagickBooleanType
+    is_gray,
     status;
 
   QuantumInfo
     *quantum_info;
 
-  register const Quantum
-    *p;
-
   size_t
     length;
 
@@ -671,7 +679,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  (void) TransformImageColorspace(image,sRGBColorspace,exception);
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(image,sRGBColorspace,exception);
   /*
     Allocate image memory.
   */
@@ -700,8 +709,9 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
     image->depth));
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
+  is_gray=IdentifyImageCoderGray(image,exception);
   (void) FormatLocaleString(header,FITSBlocksize,"NAXIS   =           %10lu",
-    SetImageGray(image,exception) != MagickFalse ? 2UL : 3UL);
+    (is_gray != MagickFalse) ? 2UL : 3UL);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
   (void) FormatLocaleString(header,FITSBlocksize,"NAXIS1  =           %10lu",
@@ -712,7 +722,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
     (unsigned long) image->rows);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
-  if (SetImageGray(image,exception) == MagickFalse)
+  if (is_gray == MagickFalse)
     {
       (void) FormatLocaleString(header,FITSBlocksize,
         "NAXIS3  =           %10lu",3UL);
@@ -739,9 +749,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
       CopyFitsRecord(fits_info,header,offset);
       offset+=80;
     }
-  url=GetMagickHomeURL();
-  (void) FormatLocaleString(header,FITSBlocksize,"HISTORY %.72s",url);
-  url=DestroyString(url);
+  (void) FormatLocaleString(header,FITSBlocksize,"HISTORY %.72s",
+    MagickAuthoritativeURL);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
   (void) strncpy(header,"END",FITSBlocksize);
@@ -752,7 +761,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
     Convert image to fits scale PseudoColor class.
   */
   pixels=(unsigned char *) GetQuantumPixels(quantum_info);
-  if (SetImageGray(image,exception) != MagickFalse)
+  if (is_gray != MagickFalse)
     {
       length=GetQuantumExtent(image,quantum_info,GrayQuantum);
       for (y=(ssize_t) image->rows-1; y >= 0; y--)
