@@ -17,13 +17,13 @@
 %                                April 2016                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,32 +39,32 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/artifact.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/client.h"
-#include "magick/colorspace-private.h"
-#include "magick/display.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/memory_.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/module.h"
-#include "magick/utility.h"
-#include "magick/xwindow.h"
-#include "magick/xwindow-private.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/client.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/display.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
+#include "MagickCore/xwindow.h"
+#include "MagickCore/xwindow-private.h"
 #if defined(MAGICKCORE_FLIF_DELEGATE)
 #include <flif.h>
 #endif
@@ -118,13 +118,13 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register PixelPacket
+  Quantum
     *q;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned short
+  unsigned short
     *p;
 
   size_t
@@ -151,7 +151,7 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -170,7 +170,7 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
     }
   flifdec=flif_create_decoder();
   if (image_info->quality != UndefinedCompressionQuality)
-    flif_decoder_set_quality(flifdec,image_info->quality);
+    flif_decoder_set_quality(flifdec,(int32_t) image_info->quality);
   if (!flif_decoder_decode_memory(flifdec,stream,length))
     {
       flif_destroy_decoder(flifdec);
@@ -179,7 +179,7 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
   image_count=flif_decoder_num_images(flifdec);
   flifimage=flif_decoder_get_image(flifdec,0);
   length=sizeof(unsigned short)*4*flif_image_get_width(flifimage);
-  pixels=(unsigned short *) AcquireMagickMemory(length);
+  pixels=(unsigned short *) AcquireQuantumMemory(1,length);
   if (pixels == (unsigned short *) NULL)
     {
       flif_destroy_decoder(flifdec);
@@ -193,13 +193,11 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
         /*
           Allocate next image structure.
         */
-        AcquireNextImage(image_info,image);
+        AcquireNextImage(image_info,image,exception);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            image=DestroyImageList(image);
-            flif_destroy_decoder(flifdec);
-            pixels=(unsigned short *) RelinquishMagickMemory(pixels);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
       }
@@ -207,8 +205,8 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
     image->columns=(size_t) flif_image_get_width(flifimage);
     image->rows=(size_t) flif_image_get_height(flifimage);
     image->depth=flif_image_get_depth(flifimage);
-    image->matte=(flif_image_get_nb_channels(flifimage) > 3 ?
-      MagickTrue : MagickFalse);
+    image->alpha_trait=(flif_image_get_nb_channels(flifimage) > 3 ?
+      BlendPixelTrait : UndefinedPixelTrait);
     image->delay=flif_image_get_frame_delay(flifimage);
     image->ticks_per_second=1000;
     image->scene=count;
@@ -218,15 +216,15 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
       flif_image_read_row_RGBA16(flifimage,y,pixels,length);
       p=pixels;
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-      if (q == (PixelPacket *) NULL)
+      if (q == (Quantum *) NULL)
         break;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        SetPixelRed(q,ScaleShortToQuantum(*p++));
-        SetPixelGreen(q,ScaleShortToQuantum(*p++));
-        SetPixelBlue(q,ScaleShortToQuantum(*p++));
-        SetPixelAlpha(q,ScaleShortToQuantum(*p++));
-        q++;
+        SetPixelRed(image,ScaleShortToQuantum(*p++),q);
+        SetPixelGreen(image,ScaleShortToQuantum(*p++),q);
+        SetPixelBlue(image,ScaleShortToQuantum(*p++),q);
+        SetPixelAlpha(image,ScaleShortToQuantum(*p++),q);
+        q+=GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
@@ -238,6 +236,8 @@ static Image *ReadFLIFImage(const ImageInfo *image_info,
   }
   flif_destroy_decoder(flifdec);
   pixels=(unsigned short *) RelinquishMagickMemory(pixels);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(image);
 }
 #endif
@@ -304,24 +304,21 @@ static MagickBooleanType IsFLIF(const unsigned char *magick,
 ModuleExport size_t RegisterFLIFImage(void)
 {
   char
-    version[MaxTextExtent];
+    version[MagickPathExtent];
 
   MagickInfo
     *entry;
 
   *version='\0';
-  entry=SetMagickInfo("FLIF");
+  entry=AcquireMagickInfo("FLIF","FLIF","Free Lossless Image Format");
 #if defined(MAGICKCORE_FLIF_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadFLIFImage;
   entry->encoder=(EncodeImageHandler *) WriteFLIFImage;
-  (void) FormatLocaleString(version,MaxTextExtent,"libflif %d.%d.%d [%04X]",
+  (void) FormatLocaleString(version,MagickPathExtent,"libflif %d.%d.%d [%04X]",
     (FLIF_VERSION >> 16) & 0xff,
     (FLIF_VERSION  >> 8) & 0xff,
     (FLIF_VERSION  >> 0) & 0xff,FLIF_ABI_VERSION);
 #endif
-  entry->description=ConstantString("Free Lossless Image Format");
-  entry->adjoin=MagickTrue;
-  entry->module=ConstantString("FLIF");
   entry->mime_type=ConstantString("image/flif");
   entry->magick=(IsImageFormatHandler *) IsFLIF;
   if (*version != '\0')
@@ -398,16 +395,16 @@ static MagickBooleanType WriteFLIFImage(const ImageInfo *image_info,
   MagickOffsetType
     scene;
 
-  register const PixelPacket
+  const Quantum
     *magick_restrict p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *magick_restrict qc;
 
-  register unsigned short
+  unsigned short
     *magick_restrict qs;
 
   size_t
@@ -438,7 +435,7 @@ static MagickBooleanType WriteFLIFImage(const ImageInfo *image_info,
     return(status);
   flifenc=flif_create_encoder();
   if (image_info->quality != UndefinedCompressionQuality)
-    flif_encoder_set_lossy(flifenc,3*(100-image_info->quality));
+    flif_encoder_set_lossy(flifenc,3*(100-(int32_t) image_info->quality));
 
   /* relatively fast encoding */
   flif_encoder_set_learn_repeat(flifenc,1);
@@ -450,12 +447,14 @@ static MagickBooleanType WriteFLIFImage(const ImageInfo *image_info,
   /* Convert image to FLIFIMAGE */
   if (image->depth > 8)
     {
-      flifimage=flif_create_image_HDR(image->columns,image->rows);
+      flifimage=flif_create_image_HDR((uint32_t) image->columns,
+        (uint32_t) image->rows);
       length=sizeof(unsigned short)*4*image->columns;
     }
   else
     {
-      flifimage=flif_create_image(image->columns,image->rows);
+      flifimage=flif_create_image((uint32_t) image->columns,
+        (uint32_t) image->rows);
       length=sizeof(unsigned char)*4*image->columns;
     }
   if (flifimage == (FLIF_IMAGE *) NULL)
@@ -477,7 +476,7 @@ static MagickBooleanType WriteFLIFImage(const ImageInfo *image_info,
     for (y=0; y < (ssize_t) image->rows; y++)
     {
       p=GetVirtualPixels(image,0,y,image->columns,1,exception);
-      if (p == (PixelPacket *) NULL)
+      if (p == (Quantum *) NULL)
         break;
 
       if (image->depth > 8)
@@ -485,14 +484,14 @@ static MagickBooleanType WriteFLIFImage(const ImageInfo *image_info,
           qs=(unsigned short *) pixels;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
-            *qs++=ScaleQuantumToShort(GetPixelRed(p));
-            *qs++=ScaleQuantumToShort(GetPixelGreen(p));
-            *qs++=ScaleQuantumToShort(GetPixelBlue(p));
-            if (image->matte != MagickFalse)
-              *qs++=ScaleQuantumToShort(GetPixelAlpha(p));
+            *qs++=ScaleQuantumToShort(GetPixelRed(image,p));
+            *qs++=ScaleQuantumToShort(GetPixelGreen(image,p));
+            *qs++=ScaleQuantumToShort(GetPixelBlue(image,p));
+            if (image->alpha_trait != UndefinedPixelTrait)
+              *qs++=ScaleQuantumToShort(GetPixelAlpha(image,p));
             else
               *qs++=0xFFFF;
-            p++;
+            p+=GetPixelChannels(image);
           }
           flif_image_write_row_RGBA16(flifimage,y,pixels,length);
         }
@@ -501,19 +500,19 @@ static MagickBooleanType WriteFLIFImage(const ImageInfo *image_info,
           qc=pixels;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
-            *qc++=ScaleQuantumToChar(GetPixelRed(p));
-            *qc++=ScaleQuantumToChar(GetPixelGreen(p));
-            *qc++=ScaleQuantumToChar(GetPixelBlue(p));
-            if (image->matte != MagickFalse)
-              *qc++=ScaleQuantumToChar(GetPixelAlpha(p));
+            *qc++=ScaleQuantumToChar(GetPixelRed(image,p));
+            *qc++=ScaleQuantumToChar(GetPixelGreen(image,p));
+            *qc++=ScaleQuantumToChar(GetPixelBlue(image,p));
+            if (image->alpha_trait != UndefinedPixelTrait)
+              *qc++=ScaleQuantumToChar(GetPixelAlpha(image,p));
             else
               *qc++=0xFF;
-            p++;
+            p+=GetPixelChannels(image);
           }
           flif_image_write_row_RGBA8(flifimage,y,pixels,length);
         }
     }
-    flif_image_set_frame_delay(flifimage,image->delay*100/
+    flif_image_set_frame_delay(flifimage,(uint32_t) image->delay*100/
       image->ticks_per_second);
     flif_encoder_add_image(flifenc,flifimage);
     if (GetNextImageInList(image) == (Image *) NULL)
