@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,36 +39,39 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/color.h"
-#include "magick/color-private.h"
-#include "magick/colormap.h"
-#include "magick/colormap-private.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/profile.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/option.h"
-#include "magick/pixel-private.h"
-#include "magick/property.h"
-#include "magick/quantize.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colormap.h"
+#include "MagickCore/colormap-private.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/profile.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/memory-private.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantize.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/module.h"
 
 /*
   Define declarations.
@@ -134,7 +137,7 @@ static inline int
   GetNextLZWCode(LZWInfo *,const size_t);
 
 static MagickBooleanType
-  WriteGIFImage(const ImageInfo *,Image *);
+  WriteGIFImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 static ssize_t
   ReadBlobBlock(Image *,unsigned char *);
@@ -200,7 +203,7 @@ static LZWInfo *AcquireLZWInfo(Image *image,const size_t data_size)
   LZWInfo
     *lzw_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -265,7 +268,7 @@ static inline int GetNextLZWCode(LZWInfo *lzw_info,const size_t bits)
   int
     code;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -390,30 +393,26 @@ static int ReadBlobLZWByte(LZWInfo *lzw_info)
   return(PopLZWStack(lzw_info->stack));
 }
 
-static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
+static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity,
+  ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
-  IndexPacket
-    index;
-
   int
     c;
 
   LZWInfo
     *lzw_info;
 
+  size_t
+    pass;
+
   ssize_t
+    index,
     offset,
     y;
 
   unsigned char
     data_size;
 
-  size_t
-    pass;
-
   /*
     Allocate decoder tables.
   */
@@ -421,7 +420,6 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  exception=(&image->exception);
   data_size=(unsigned char) ReadBlobByte(image);
   if (data_size > MaximumLZWBits)
     ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
@@ -433,31 +431,26 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
   offset=0;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
-      *magick_restrict indexes;
-
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    Quantum
       *magick_restrict q;
 
     q=QueueAuthenticPixels(image,0,offset,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    if (q == (Quantum *) NULL)
       break;
-    indexes=GetAuthenticIndexQueue(image);
     for (x=0; x < (ssize_t) image->columns; )
     {
       c=ReadBlobLZWByte(lzw_info);
       if (c < 0)
         break;
-      index=ConstrainColormapIndex(image,(ssize_t) c);
-      SetPixelIndex(indexes+x,index);
-      SetPixelRGBO(q,image->colormap+(ssize_t) index);
-      SetPixelOpacity(q,(ssize_t) index == opacity ? TransparentOpacity :
-        OpaqueOpacity);
+      index=ConstrainColormapIndex(image,(ssize_t) c,exception);
+      SetPixelIndex(image,(Quantum) index,q);
+      SetPixelViaPixelInfo(image,image->colormap+index,q);
+      SetPixelAlpha(image,index == opacity ? TransparentAlpha : OpaqueAlpha,q);
       x++;
-      q++;
+      q+=GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
@@ -542,7 +535,7 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
 %
 */
 static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
-  const size_t data_size)
+  const size_t data_size,ExceptionInfo *exception)
 {
 #define MaxCode(number_bits)  ((one << (number_bits))-1)
 #define MaxHashTable  5003
@@ -561,10 +554,10 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   while (bits >= 8) \
   { \
     /*  \
-      Add a character to current packet. \
+      Add a character to current packet.  Maximum packet size is 255.
     */ \
     packet[length++]=(unsigned char) (datum & 0xff); \
-    if (length >= 254) \
+    if (length == 255) \
       { \
         (void) WriteBlobByte(image,(unsigned char) length); \
         (void) WriteBlob(image,length,packet); \
@@ -583,7 +576,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
     } \
 }
 
-  IndexPacket
+  Quantum
     index;
 
   short
@@ -662,23 +655,19 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   waiting_code=0;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const IndexPacket
-      *magick_restrict indexes;
-
-    register const PixelPacket
+    const Quantum
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
-    p=GetVirtualPixels(image,0,offset,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,offset,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-    indexes=GetVirtualIndexQueue(image);
     if (y == 0)
       {
-        waiting_code=(short) (*indexes);
-        p++;
+        waiting_code=(short) GetPixelIndex(image,p);
+        p+=GetPixelChannels(image);
       }
     for (x=(ssize_t) (y == 0 ? 1 : 0); x < (ssize_t) image->columns; x++)
     {
@@ -687,8 +676,8 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
       */
       next_pixel=MagickFalse;
       displacement=1;
-      index=(IndexPacket) ((size_t) GetPixelIndex(indexes+x) & 0xff);
-      p++;
+      index=(Quantum) ((size_t) GetPixelIndex(image,p) & 0xff);
+      p+=GetPixelChannels(image);
       k=(ssize_t) (((size_t) index << (MaxGIFBits-8))+waiting_code);
       if (k >= MaxHashTable)
         k-=MaxHashTable;
@@ -797,10 +786,10 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   if (bits > 0)
     {
       /*
-        Add a character to current packet.
+        Add a character to current packet.  Maximum packet size is 255.
       */
       packet[length++]=(unsigned char) (datum & 0xff);
-      if (length >= 254)
+      if (length == 255)
         {
           (void) WriteBlobByte(image,(unsigned char) length);
           (void) WriteBlob(image,length,packet);
@@ -938,7 +927,7 @@ static void *DestroyGIFProfile(void *profile)
   return((void *) DestroyStringInfo((StringInfo *) profile));
 }
 
-static MagickBooleanType PingGIFImage(Image *image)
+static MagickBooleanType PingGIFImage(Image *image,ExceptionInfo *exception)
 {
   unsigned char
     buffer[256],
@@ -950,22 +939,17 @@ static MagickBooleanType PingGIFImage(Image *image)
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (ReadBlob(image,1,&data_size) != 1)
-    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
-      image->filename);
+    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
   if (data_size > MaximumLZWBits)
-    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
-      image->filename);
+    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
   if (ReadBlob(image,1,&length) != 1)
-    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
-      image->filename);
+    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
   while (length != 0)
   {
     if (ReadBlob(image,length,buffer) != (ssize_t) length)
-      ThrowBinaryImageException(CorruptImageError,"CorruptImage",
-        image->filename);
+      ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
     if (ReadBlob(image,1,&length) != 1)
-      ThrowBinaryImageException(CorruptImageError,"CorruptImage",
-        image->filename);
+      ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
   }
   return(MagickTrue);
 }
@@ -995,10 +979,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -1017,8 +1001,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     buffer[257],
     c,
     flag,
-    *global_colormap,
-    magick[12];
+    *global_colormap;
 
   /*
     Open image file.
@@ -1030,7 +1013,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -1040,19 +1023,19 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine if this a GIF file.
   */
-  count=ReadBlob(image,6,magick);
-  if ((count != 6) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
-      (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
+  count=ReadBlob(image,6,buffer);
+  if ((count != 6) || ((LocaleNCompare((char *) buffer,"GIF87",5) != 0) &&
+      (LocaleNCompare((char *) buffer,"GIF89",5) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   (void) memset(buffer,0,sizeof(buffer));
-  meta_image=AcquireImage(image_info);  /* metadata container */
+  meta_image=AcquireImage(image_info,exception);  /* metadata container */
   meta_image->page.width=ReadBlobLSBShort(image);
   meta_image->page.height=ReadBlobLSBShort(image);
   meta_image->iterations=1;
   flag=(unsigned char) ReadBlobByte(image);
+  profiles=(LinkedListInfo *) NULL;
   background=(unsigned char) ReadBlobByte(image);
   c=(unsigned char) ReadBlobByte(image);  /* reserved */
-  profiles=(LinkedListInfo *) NULL;
   one=1;
   global_colors=one << (((size_t) flag & 0x07)+1);
   global_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
@@ -1082,10 +1065,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           GIF Extension block.
         */
+        (void) memset(buffer,0,sizeof(buffer));
         count=ReadBlob(image,1,&c);
         if (count != 1)
           ThrowGIFException(CorruptImageError,"UnableToReadExtensionBlock");
-        (void) memset(buffer,0,sizeof(buffer));
         switch (c)
         {
           case 0xf9:
@@ -1117,11 +1100,12 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (count == 0)
                 break;
               buffer[count]='\0';
-              if (((ssize_t) (count+offset+MagickPathExtent)) >= (ssize_t) extent)
+              if ((ssize_t) (count+offset+MagickPathExtent) >= (ssize_t) extent)
                 {
                   extent<<=1;
-                  comments=(char *) ResizeQuantumMemory(comments,extent+
-                    MagickPathExtent,sizeof(*comments));
+                  comments=(char *) ResizeQuantumMemory(comments,
+                    OverAllocateMemory(extent+MagickPathExtent),
+                    sizeof(*comments));
                   if (comments == (char *) NULL)
                     ThrowGIFException(ResourceLimitError,
                       "MemoryAllocationFailed");
@@ -1129,7 +1113,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               (void) CopyMagickString(&comments[offset],(char *) buffer,extent-
                 offset);
             }
-            (void) SetImageProperty(meta_image,"comment",comments);
+            (void) SetImageProperty(meta_image,"comment",comments,exception);
             comments=DestroyString(comments);
             break;
           }
@@ -1155,7 +1139,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             else
               {
                 char
-                  name[MaxTextExtent];
+                  name[MagickPathExtent];
 
                 int
                   block_length,
@@ -1193,6 +1177,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (info == (unsigned char *) NULL)
                   ThrowGIFException(ResourceLimitError,
                     "MemoryAllocationFailed");
+                (void) memset(info,0,reserved_length*sizeof(*info));
                 for (info_length=0; ; )
                 {
                   block_length=(int) ReadBlobBlock(image,info+info_length);
@@ -1234,9 +1219,9 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 else
                   (void) FormatLocaleString(name,sizeof(name),"gif:%.11s",
                     buffer);
-                info=(unsigned char *) RelinquishMagickMemory(info);
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "      profile name=%s",name);
+                info=(unsigned char *) RelinquishMagickMemory(info);
                 if (magick != MagickFalse)
                   profile=DestroyStringInfo(profile);
                 else
@@ -1258,12 +1243,13 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     if (c != (unsigned char) ',')
       continue;
-    if (image_count != 0)
+    image_count++;
+    if (image_count != 1)
       {
         /*
           Allocate next image structure.
         */
-        AcquireNextImage(image_info,image);
+        AcquireNextImage(image_info,image,exception);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             status=MagickFalse;
@@ -1271,7 +1257,6 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         image=SyncNextImageInList(image);
       }
-    image_count++;
     /*
       Read image attributes.
     */
@@ -1290,19 +1275,18 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     local_colors=BitSet((int) flag,0x80) == 0 ? global_colors : one <<
       ((size_t) (flag & 0x07)+1);
     image->colors=local_colors;
-    if (opacity >= (ssize_t) image->colors)
-      {
-        image->colors++;
-        opacity=(-1);
-      }
+    if (opacity == (ssize_t) image->colors)
+      image->colors++;
+    else if (opacity > (ssize_t) image->colors)
+      opacity=(-1);
     image->ticks_per_second=100;
-    image->matte=opacity >= 0 ? MagickTrue : MagickFalse;
+    image->alpha_trait=opacity >= 0 ? BlendPixelTrait : UndefinedPixelTrait;
     if ((image->columns == 0) || (image->rows == 0))
       ThrowGIFException(CorruptImageError,"NegativeOrZeroImageSize");
     /*
       Inititialize colormap.
     */
-    if (AcquireImageColormap(image,image->colors) == MagickFalse)
+    if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
       ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
     if (BitSet((int) flag,0x80) == 0)
       {
@@ -1312,12 +1296,12 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         p=global_colormap;
         for (i=0; i < (ssize_t) image->colors; i++)
         {
-          image->colormap[i].red=ScaleCharToQuantum(*p++);
-          image->colormap[i].green=ScaleCharToQuantum(*p++);
-          image->colormap[i].blue=ScaleCharToQuantum(*p++);
+          image->colormap[i].red=(double) ScaleCharToQuantum(*p++);
+          image->colormap[i].green=(double) ScaleCharToQuantum(*p++);
+          image->colormap[i].blue=(double) ScaleCharToQuantum(*p++);
           if (i == opacity)
             {
-              image->colormap[i].opacity=(Quantum) TransparentOpacity;
+              image->colormap[i].alpha=(double) TransparentAlpha;
               image->transparent_color=image->colormap[opacity];
             }
         }
@@ -1347,26 +1331,26 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         p=colormap;
         for (i=0; i < (ssize_t) image->colors; i++)
         {
-          image->colormap[i].red=ScaleCharToQuantum(*p++);
-          image->colormap[i].green=ScaleCharToQuantum(*p++);
-          image->colormap[i].blue=ScaleCharToQuantum(*p++);
+          image->colormap[i].red=(double) ScaleCharToQuantum(*p++);
+          image->colormap[i].green=(double) ScaleCharToQuantum(*p++);
+          image->colormap[i].blue=(double) ScaleCharToQuantum(*p++);
           if (i == opacity)
-            image->colormap[i].opacity=(Quantum) TransparentOpacity;
+            image->colormap[i].alpha=(double) TransparentAlpha;
         }
         colormap=(unsigned char *) RelinquishMagickMemory(colormap);
       }
     if (image->gamma == 1.0)
       {
         for (i=0; i < (ssize_t) image->colors; i++)
-          if (IsGrayPixel(image->colormap+i) == MagickFalse)
+          if (IsPixelInfoGray(image->colormap+i) == MagickFalse)
             break;
         (void) SetImageColorspace(image,i == (ssize_t) image->colors ?
-          LinearGRAYColorspace : RGBColorspace);
+          GRAYColorspace : RGBColorspace,exception);
       }
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       {
         if (profiles != (LinkedListInfo *) NULL)
@@ -1374,17 +1358,15 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         global_colormap=(unsigned char *) RelinquishMagickMemory(
           global_colormap);
         meta_image=DestroyImage(meta_image);
-        InheritException(exception,&image->exception);
         return(DestroyImageList(image));
       }
     /*
       Decode image.
     */
     if (image_info->ping != MagickFalse)
-      status=PingGIFImage(image);
+      status=PingGIFImage(image,exception);
     else
-      status=DecodeImage(image,opacity);
-    InheritException(exception,&image->exception);
+      status=DecodeImage(image,opacity,exception);
     if ((image_info->ping == MagickFalse) && (status == MagickFalse))
       ThrowGIFException(CorruptImageError,"CorruptImage");
     if (profiles != (LinkedListInfo *) NULL)
@@ -1399,7 +1381,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         profile=(StringInfo *) GetNextValueInLinkedList(profiles);
         while (profile != (StringInfo *) NULL)
         {
-          (void) SetImageProfile(image,GetStringInfoName(profile),profile);
+          (void) SetImageProfile(image,GetStringInfoName(profile),profile,
+            exception);
           profile=(StringInfo *) GetNextValueInLinkedList(profiles);
         }
         profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
@@ -1409,8 +1392,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     opacity=(-1);
-    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) image->scene-
-      1,image->scene);
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
+      image->scene-1,image->scene);
     if (status == MagickFalse)
       break;
   }
@@ -1455,23 +1438,21 @@ ModuleExport size_t RegisterGIFImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("GIF");
+  entry=AcquireMagickInfo("GIF","GIF",
+    "CompuServe graphics interchange format");
   entry->decoder=(DecodeImageHandler *) ReadGIFImage;
   entry->encoder=(EncodeImageHandler *) WriteGIFImage;
   entry->magick=(IsImageFormatHandler *) IsGIF;
-  entry->description=ConstantString("CompuServe graphics interchange format");
   entry->mime_type=ConstantString("image/gif");
-  entry->module=ConstantString("GIF");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("GIF87");
+  entry=AcquireMagickInfo("GIF","GIF87",
+    "CompuServe graphics interchange format");
   entry->decoder=(DecodeImageHandler *) ReadGIFImage;
   entry->encoder=(EncodeImageHandler *) WriteGIFImage;
   entry->magick=(IsImageFormatHandler *) IsGIF;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("CompuServe graphics interchange format");
+  entry->flags^=CoderAdjoinFlag;
   entry->version=ConstantString("version 87a");
   entry->mime_type=ConstantString("image/gif");
-  entry->module=ConstantString("GIF");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -1518,7 +1499,7 @@ ModuleExport void UnregisterGIFImage(void)
 %  The format of the WriteGIFImage method is:
 %
 %      MagickBooleanType WriteGIFImage(const ImageInfo *image_info,
-%        Image *image)
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -1526,8 +1507,11 @@ ModuleExport void UnregisterGIFImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   int
     c;
@@ -1544,10 +1528,10 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   RectangleInfo
     page;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -1574,7 +1558,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   /*
@@ -1622,20 +1608,18 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   /*
     Write images to file.
   */
-  if ((write_info->adjoin != MagickFalse) &&
-      (GetNextImageInList(image) != (Image *) NULL))
-    write_info->interlace=NoInterlace;
   scene=0;
   one=1;
   imageListLength=GetImageListLength(image);
   do
   {
-    (void) TransformImageColorspace(image,sRGBColorspace);
+    if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace,exception);
     opacity=(-1);
-    if (IsOpaqueImage(image,&image->exception) != MagickFalse)
+    if (IsImageOpaque(image,exception) != MagickFalse)
       {
         if ((image->storage_class == DirectClass) || (image->colors > 256))
-          (void) SetImageType(image,PaletteType);
+          (void) SetImageType(image,PaletteType,exception);
       }
     else
       {
@@ -1647,37 +1631,33 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
           Identify transparent colormap index.
         */
         if ((image->storage_class == DirectClass) || (image->colors > 256))
-          (void) SetImageType(image,PaletteBilevelMatteType);
+          (void) SetImageType(image,PaletteBilevelAlphaType,exception);
         for (i=0; i < (ssize_t) image->colors; i++)
-          if (image->colormap[i].opacity != OpaqueOpacity)
+          if (image->colormap[i].alpha != OpaqueAlpha)
             {
               if (opacity < 0)
                 {
                   opacity=i;
                   continue;
                 }
-              alpha=fabs((double) image->colormap[i].opacity-
-                TransparentOpacity);
-              beta=fabs((double) image->colormap[opacity].opacity-
-                TransparentOpacity);
+              alpha=fabs(image->colormap[i].alpha-TransparentAlpha);
+              beta=fabs(image->colormap[opacity].alpha-TransparentAlpha);
               if (alpha < beta)
                 opacity=i;
             }
         if (opacity == -1)
           {
-            (void) SetImageType(image,PaletteBilevelMatteType);
+            (void) SetImageType(image,PaletteBilevelAlphaType,exception);
             for (i=0; i < (ssize_t) image->colors; i++)
-              if (image->colormap[i].opacity != OpaqueOpacity)
+              if (image->colormap[i].alpha != OpaqueAlpha)
                 {
                   if (opacity < 0)
                     {
                       opacity=i;
                       continue;
                     }
-                  alpha=fabs((double) image->colormap[i].opacity-
-                    TransparentOpacity);
-                  beta=fabs((double) image->colormap[opacity].opacity-
-                    TransparentOpacity);
+                  alpha=fabs(image->colormap[i].alpha-TransparentAlpha);
+                  beta=fabs(image->colormap[opacity].alpha-TransparentAlpha);
                   if (alpha < beta)
                     opacity=i;
                 }
@@ -1697,9 +1677,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     q=colormap;
     for (i=0; i < (ssize_t) image->colors; i++)
     {
-      *q++=ScaleQuantumToChar(image->colormap[i].red);
-      *q++=ScaleQuantumToChar(image->colormap[i].green);
-      *q++=ScaleQuantumToChar(image->colormap[i].blue);
+      *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].red));
+      *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].green));
+      *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].blue));
     }
     for ( ; i < (ssize_t) (one << bits_per_pixel); i++)
     {
@@ -1718,7 +1698,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         c|=(bits_per_pixel-1);   /* size of global colormap */
         (void) WriteBlobByte(image,(unsigned char) c);
         for (j=0; j < (ssize_t) image->colors; j++)
-          if (IsColorEqual(&image->background_color,image->colormap+j))
+          if (IsPixelInfoEquivalent(&image->background_color,image->colormap+j))
             break;
         (void) WriteBlobByte(image,(unsigned char)
           (j == (ssize_t) image->colors ? 0 : j));  /* background color */
@@ -1749,10 +1729,33 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobByte(image,(unsigned char) (opacity >= 0 ? opacity :
           0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
-        value=GetImageProperty(image,"comment");
+        if (fabs(image->gamma - 1.0f/2.2f) > MagickEpsilon)
+          {
+            char
+              attributes[MagickPathExtent];
+
+            ssize_t
+              count;
+
+            /*
+              Write ImageMagick extension.
+            */
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+               "  Writing GIF Extension %s","ImageMagick");
+            (void) WriteBlobByte(image,(unsigned char) 0x21);
+            (void) WriteBlobByte(image,(unsigned char) 0xff);
+            (void) WriteBlobByte(image,(unsigned char) 0x0b);
+            (void) WriteBlob(image,11,(unsigned char *) "ImageMagick");
+            count=FormatLocaleString(attributes,MagickPathExtent,"gamma=%g",
+              image->gamma);
+            (void) WriteBlobByte(image,(unsigned char) count);
+            (void) WriteBlob(image,(size_t) count,(unsigned char *) attributes);
+            (void) WriteBlobByte(image,(unsigned char) 0x00);
+          }
+        value=GetImageProperty(image,"comment",exception);
         if (value != (const char *) NULL)
           {
-            register const char
+            const char
               *p;
 
             size_t
@@ -1791,29 +1794,6 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
               image->iterations-1 : 0));
             (void) WriteBlobByte(image,(unsigned char) 0x00);
           }
-        if ((image->gamma != 1.0f/2.2f))
-          {
-            char
-              attributes[MaxTextExtent];
-
-            ssize_t
-              count;
-
-            /*
-              Write ImageMagick extension.
-            */
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-               "  Writing GIF Extension %s","ImageMagick");
-            (void) WriteBlobByte(image,(unsigned char) 0x21);
-            (void) WriteBlobByte(image,(unsigned char) 0xff);
-            (void) WriteBlobByte(image,(unsigned char) 0x0b);
-            (void) WriteBlob(image,11,(unsigned char *) "ImageMagick");
-            count=FormatLocaleString(attributes,MaxTextExtent,"gamma=%g",
-              image->gamma);
-            (void) WriteBlobByte(image,(unsigned char) count);
-            (void) WriteBlob(image,(size_t) count,(unsigned char *) attributes);
-            (void) WriteBlobByte(image,(unsigned char) 0x00);
-          }
         ResetImageProfileIterator(image);
         for ( ; ; )
         {
@@ -1835,9 +1815,6 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
                 (LocaleCompare(name,"8BIM") == 0) ||
                 (LocaleNCompare(name,"gif:",4) == 0))
             {
-               size_t
-                 length;
-
                ssize_t
                  offset;
 
@@ -1883,7 +1860,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
                    else
                      {
                        char
-                         extension[MaxTextExtent];
+                         extension[MagickPathExtent];
 
                        /*
                          Write generic extension.
@@ -1946,7 +1923,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     */
     c=(int) MagickMax(bits_per_pixel,2);
     (void) WriteBlobByte(image,(unsigned char) c);
-    status=EncodeImage(write_info,image,(size_t) MagickMax(bits_per_pixel,2)+1);
+    status=EncodeImage(write_info,image,(size_t) MagickMax(bits_per_pixel,2)+1,
+      exception);
     if (status == MagickFalse)
       {
         global_colormap=(unsigned char *) RelinquishMagickMemory(
