@@ -22,7 +22,7 @@
 %                                                                             %
 %                      Copyright 2017-2018 YANDEX LLC.                        %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -126,7 +126,8 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static MagickBooleanType IsHeifSuccess(struct heif_error *error,Image *image,
+
+static MagickBooleanType IsHeifSuccess(Image *image,struct heif_error *error,
   ExceptionInfo *exception)
 {
   if (error->code == 0)
@@ -137,23 +138,23 @@ static MagickBooleanType IsHeifSuccess(struct heif_error *error,Image *image,
 static MagickBooleanType ReadHEICColorProfile(Image *image,
   struct heif_image_handle *image_handle,ExceptionInfo *exception)
 {
+#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
   size_t
     length;
 
-#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
+  /*
+    Read color profile.
+  */
   length=heif_image_handle_get_raw_color_profile_size(image_handle);
   if (length > 0)
     {
       unsigned char
         *color_buffer;
 
-      /*
-        Read color profile.
-      */
       if ((MagickSizeType) length > GetBlobSize(image))
         ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
           image->filename);
-      color_buffer=(unsigned char *) AcquireMagickMemory(length);
+      color_buffer=(unsigned char *) AcquireQuantumMemory(1,length);
       if (color_buffer != (unsigned char *) NULL)
         {
           struct heif_error
@@ -189,6 +190,9 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
   int
     count;
 
+  /*
+    Read Exif profile.
+  */
   count=heif_image_handle_get_list_of_metadata_block_IDs(image_handle,"Exif",
     &exif_id,1);
   if (count > 0)
@@ -199,14 +203,11 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
       unsigned char
         *exif_buffer;
 
-      /*
-        Read Exif profile.
-      */
       exif_size=heif_image_handle_get_metadata_size(image_handle,exif_id);
       if ((MagickSizeType) exif_size > GetBlobSize(image))
         ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
           image->filename);
-      exif_buffer=(unsigned char *) AcquireMagickMemory(exif_size);
+      exif_buffer=(unsigned char *) AcquireQuantumMemory(1,exif_size);
       if (exif_buffer != (unsigned char *) NULL)
         {
           struct heif_error
@@ -220,8 +221,7 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
                 *profile;
 
               /*
-                The first 4 byte should be skipped since they indicate the
-                offset to the start of the TIFF header of the Exif data.
+                Skip first 4 bytes, the offset to the TIFF header.
               */
               profile=(StringInfo*) NULL;
               if (exif_size > 8)
@@ -252,15 +252,15 @@ static inline MagickBooleanType HEICSkipImage(const ImageInfo *image_info,
   return(MagickFalse);
 }
 
-static MagickBooleanType ReadHEICImageByID(const ImageInfo *image_info,
-  Image *image,struct heif_context *heif_context,heif_item_id image_id,
+static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
+  Image *image,struct heif_image_handle *image_handle,
   ExceptionInfo *exception)
 {
+  const uint8_t
+    *p;
+
   int
-    stride_y,
-    stride_cb,
-    stride_cr,
-    stride_a;
+    stride = 0;
 
   MagickBooleanType
     preserve_orientation,
@@ -278,37 +278,21 @@ static MagickBooleanType ReadHEICImageByID(const ImageInfo *image_info,
   struct heif_image
     *heif_image;
 
-  struct heif_image_handle
-    *image_handle;
-
-  const uint8_t
-    *p_y,
-    *p_cb,
-    *p_cr,
-    *p_a;
-
-  error=heif_context_get_image_handle(heif_context,image_id,&image_handle);
-  if (IsHeifSuccess(&error,image,exception) == MagickFalse)
-    return(MagickFalse);
-  if (ReadHEICColorProfile(image,image_handle,exception) == MagickFalse)
-    {
-      heif_image_handle_release(image_handle);
-      return(MagickFalse);
-    }
-  if (ReadHEICExifProfile(image,image_handle,exception) == MagickFalse)
-    {
-      heif_image_handle_release(image_handle);
-      return(MagickFalse);
-    }
   /*
-    Set image size.
+    Read HEIC image from container.
   */
   image->columns=(size_t) heif_image_handle_get_width(image_handle);
   image->rows=(size_t) heif_image_handle_get_height(image_handle);
-#if LIBHEIF_NUMERIC_VERSION > 0x01040000
-  image->depth=(size_t) heif_image_handle_get_luma_bits_per_pixel(image_handle);
-#else
   image->depth=8;
+#if LIBHEIF_NUMERIC_VERSION > 0x01040000
+  {
+    int
+      bits_per_pixel;
+
+    bits_per_pixel=heif_image_handle_get_luma_bits_per_pixel(image_handle);
+    if (bits_per_pixel != -1)
+      image->depth=(size_t) bits_per_pixel;
+  }
 #endif
   if (heif_image_handle_has_alpha_channel(image_handle))
     image->alpha_trait=BlendPixelTrait;
@@ -316,66 +300,48 @@ static MagickBooleanType ReadHEICImageByID(const ImageInfo *image_info,
     "heic:preserve-orientation"));
   if (preserve_orientation == MagickFalse)
     (void) SetImageProperty(image,"exif:Orientation","1",exception);
+  if (ReadHEICColorProfile(image,image_handle,exception) == MagickFalse)
+    return(MagickFalse);
+  if (ReadHEICExifProfile(image,image_handle,exception) == MagickFalse)
+    return(MagickFalse);
   if (image_info->ping != MagickFalse)
-    {
-      image->colorspace=YCbCrColorspace;
-      heif_image_handle_release(image_handle);
-      return(MagickTrue);
-    }
+    return(MagickTrue);
   if (HEICSkipImage(image_info,image) != MagickFalse)
-    {
-      heif_image_handle_release(image_handle);
-      return(MagickTrue);
-    }
+    return(MagickTrue);
   status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      heif_image_handle_release(image_handle);
-      return(MagickFalse);
-    }
-  /*
-    Copy HEIF image into ImageMagick data structures.
-  */
-  (void) SetImageColorspace(image,YCbCrColorspace,exception);
+    return(MagickFalse);
   decode_options=heif_decoding_options_alloc();
 #if LIBHEIF_NUMERIC_VERSION > 0x01070000
   decode_options->convert_hdr_to_8bit=1;
 #endif
   if (preserve_orientation == MagickTrue)
     decode_options->ignore_transformations=1;
-  error=heif_decode_image(image_handle,&heif_image,heif_colorspace_YCbCr,
-    heif_chroma_420,decode_options);
+  error=heif_decode_image(image_handle,&heif_image,heif_colorspace_RGB,
+    image->alpha_trait != UndefinedPixelTrait ? heif_chroma_interleaved_RGBA :
+    heif_chroma_interleaved_RGB,decode_options);
   heif_decoding_options_free(decode_options);
-  if (IsHeifSuccess(&error,image,exception) == MagickFalse)
-    {
-      heif_image_handle_release(image_handle);
-      return(MagickFalse);
-    }
-  /*
-    Correct the width and height of the image.
-  */
-  image->columns=(size_t) heif_image_get_width(heif_image,heif_channel_Y);
-  image->rows=(size_t) heif_image_get_height(heif_image,heif_channel_Y);
+  if (IsHeifSuccess(image,&error,exception) == MagickFalse)
+    return(MagickFalse);
+  image->columns=(size_t) heif_image_get_width(heif_image,
+    heif_channel_interleaved);
+  image->rows=(size_t) heif_image_get_height(heif_image,
+    heif_channel_interleaved);
   status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     {
       heif_image_release(heif_image);
-      heif_image_handle_release(image_handle);
       return(MagickFalse);
     }
-  p_y=heif_image_get_plane_readonly(heif_image,heif_channel_Y,&stride_y);
-  p_cb=heif_image_get_plane_readonly(heif_image,heif_channel_Cb,&stride_cb);
-  p_cr=heif_image_get_plane_readonly(heif_image,heif_channel_Cr,&stride_cr);
-  p_a=(const uint8_t *) NULL;
-  stride_a=0;
-  if (image->alpha_trait != UndefinedPixelTrait)
-    p_a=heif_image_get_plane_readonly(heif_image,heif_channel_Alpha,&stride_a);
+  p=heif_image_get_plane_readonly(heif_image,heif_channel_interleaved,&stride);
+  stride-=(int) (image->columns * (image->alpha_trait != UndefinedPixelTrait ?
+    4 : 3));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     Quantum
       *q;
 
-    register ssize_t
+    ssize_t
       x;
 
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
@@ -383,33 +349,65 @@ static MagickBooleanType ReadHEICImageByID(const ImageInfo *image_info,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      SetPixelRed(image,ScaleCharToQuantum((unsigned char) p_y[y*
-        stride_y+x]),q);
-      SetPixelGreen(image,ScaleCharToQuantum((unsigned char) p_cb[(y/2)*
-        stride_cb+x/2]),q);
-      SetPixelBlue(image,ScaleCharToQuantum((unsigned char) p_cr[(y/2)*
-        stride_cr+x/2]),q);
-      if (p_a != (const uint8_t *) NULL)
-        SetPixelAlpha(image,ScaleCharToQuantum((unsigned char) p_a[y*
-          stride_a+x]),q);
+      SetPixelRed(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+      SetPixelGreen(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+      SetPixelBlue(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
+      if (image->alpha_trait != UndefinedPixelTrait)
+        SetPixelAlpha(image,ScaleCharToQuantum((unsigned char) *(p++)),q);
       q+=GetPixelChannels(image);
     }
+    p+=stride;
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   heif_image_release(heif_image);
-  heif_image_handle_release(image_handle);
   return(MagickTrue);
 }
 
+static void ReadHEICDepthImage(const ImageInfo *image_info,Image *image,
+  struct heif_image_handle *image_handle,ExceptionInfo *exception)
+{
+  const char
+    *option;
+
+  heif_item_id
+    depth_id;
+
+  int
+    number_images;
+
+  struct heif_error
+    error;
+
+  struct heif_image_handle
+    *depth_handle;
+
+  option=GetImageOption(image_info,"heic:depth-image");
+  if (IsStringTrue(option) == MagickFalse)
+    return;
+  if (heif_image_handle_has_depth_image(image_handle) == 0)
+    return;
+  number_images=heif_image_handle_get_list_of_depth_image_IDs(image_handle,
+    &depth_id,1);
+  if (number_images != 1)
+    return;
+  error=heif_image_handle_get_depth_image_handle(image_handle,depth_id,
+    &depth_handle);
+  if (IsHeifSuccess(image,&error,exception) == MagickFalse)
+    return;
+  AcquireNextImage(image_info,image,exception);
+  if (GetNextImageInList(image) != (Image *) NULL)
+    {
+      image=SyncNextImageInList(image);
+      (void) ReadHEICImageHandle(image_info,image,depth_handle,exception);
+    }
+  heif_image_handle_release(depth_handle);
+}
+
 static Image *ReadHEICImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-  const StringInfo
-    *profile;
-
   heif_item_id
-    *image_ids,
     primary_image_id;
 
   Image
@@ -428,6 +426,9 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   struct heif_error
     error;
 
+  struct heif_image_handle
+    *image_handle;
+
   void
     *file_data;
 
@@ -445,7 +446,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(DestroyImageList(image));
-  if (GetBlobSize(image) > (MagickSizeType) SSIZE_MAX)
+  if (GetBlobSize(image) > (MagickSizeType) MAGICK_SSIZE_MAX)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   length=(size_t) GetBlobSize(image);
   file_data=AcquireMagickMemory(length);
@@ -456,32 +457,46 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
       file_data=RelinquishMagickMemory(file_data);
       ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
     }
+#if LIBHEIF_NUMERIC_VERSION >= 0x010b0000
+  if (heif_has_compatible_brand((unsigned char *) file_data,(int) length,"avif") != MagickFalse)
+    (void) CopyMagickString(image->magick,"AVIF",MagickPathExtent);
+#endif
   /*
-    Decode HEIF file
+    Decode HEIF image.
   */
   heif_context=heif_context_alloc();
   error=heif_context_read_from_memory_without_copy(heif_context,file_data,
     length,NULL);
-  if (IsHeifSuccess(&error,image,exception) == MagickFalse)
+  if (IsHeifSuccess(image,&error,exception) == MagickFalse)
     {
       heif_context_free(heif_context);
       file_data=RelinquishMagickMemory(file_data);
       return(DestroyImageList(image));
     }
   error=heif_context_get_primary_image_ID(heif_context,&primary_image_id);
-  if (IsHeifSuccess(&error,image,exception) == MagickFalse)
+  if (IsHeifSuccess(image,&error,exception) == MagickFalse)
     {
       heif_context_free(heif_context);
       file_data=RelinquishMagickMemory(file_data);
       return(DestroyImageList(image));
     }
-  status=ReadHEICImageByID(image_info,image,heif_context,primary_image_id,
-    exception);
-  image_ids=(heif_item_id *) NULL;
+  error=heif_context_get_image_handle(heif_context,primary_image_id,
+    &image_handle);
+  if (IsHeifSuccess(image,&error,exception) == MagickFalse)
+    {
+      heif_context_free(heif_context);
+      file_data=RelinquishMagickMemory(file_data);
+      return(DestroyImageList(image));
+    }
+  status=ReadHEICImageHandle(image_info,image,image_handle,exception);
+  heif_image_handle_release(image_handle);
   count=(size_t) heif_context_get_number_of_top_level_images(heif_context);
   if ((status != MagickFalse) && (count > 1))
     {
-      register size_t
+      heif_item_id
+        *image_ids;
+
+      size_t
         i;
 
       image_ids=(heif_item_id *) AcquireQuantumMemory((size_t) count,
@@ -489,7 +504,8 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
       if (image_ids == (heif_item_id *) NULL)
         {
           heif_context_free(heif_context);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          file_data=RelinquishMagickMemory(file_data);
+          return(DestroyImageList(image));
         }
       (void) heif_context_get_list_of_top_level_image_IDs(heif_context,
         image_ids,(int) count);
@@ -507,45 +523,39 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
             break;
           }
         image=SyncNextImageInList(image);
-        status=ReadHEICImageByID(image_info,image,heif_context,image_ids[i],
-          exception);
+        error=heif_context_get_image_handle(heif_context,image_ids[i],
+          &image_handle);
+        if (IsHeifSuccess(image,&error,exception) == MagickFalse)
+          {
+            status=MagickFalse;
+            break;
+          }
+        status=ReadHEICImageHandle(image_info,image,image_handle,exception);
+        heif_image_handle_release(image_handle);
         if (status == MagickFalse)
           break;
         if (image_info->number_scenes != 0)
           if (image->scene >= (image_info->scene+image_info->number_scenes-1))
             break;
       }
+      image_ids=(heif_item_id *) RelinquishMagickMemory(image_ids);
     }
-  if (image_ids != (heif_item_id *) NULL)
-    (void) RelinquishMagickMemory(image_ids);
+  error=heif_context_get_image_handle(heif_context,primary_image_id,
+    &image_handle);
+  if (IsHeifSuccess(image,&error,exception) == MagickFalse)
+    {
+      heif_context_free(heif_context);
+      file_data=RelinquishMagickMemory(file_data);
+      return(DestroyImageList(image));
+    }
+  ReadHEICDepthImage(image_info,image,image_handle,exception);
+  heif_image_handle_release(image_handle);
   heif_context_free(heif_context);
   file_data=RelinquishMagickMemory(file_data);
   if (status == MagickFalse)
     return(DestroyImageList(image));
-  /*
-    Change image colorspace if it contains a color profile.
-  */
-  image=GetFirstImageInList(image);
-  profile=GetImageProfile(image,"icc");
-  if (profile != (const StringInfo *) NULL)
-    {
-      Image
-        *next;
-
-      next=image;
-      while (next != (Image *) NULL)
-      {
-        if (HEICSkipImage(image_info,next) != MagickFalse)
-          {
-            if (image_info->ping == MagickFalse)
-              (void) TransformImageColorspace(next,sRGBColorspace,exception);
-            else
-              next->colorspace=sRGBColorspace;
-          }
-        next=GetNextImageInList(next);
-      }
-    }
-  return(image);
+  (void) CloseBlob(image);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -576,18 +586,18 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
 */
 static MagickBooleanType IsHEIC(const unsigned char *magick,const size_t length)
 {
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
+  enum heif_filetype_result
+    heif_filetype;
+
   if (length < 12)
     return(MagickFalse);
-  if (LocaleNCompare((const char *) magick+4,"ftyp",4) != 0)
-    return(MagickFalse);
-  if (LocaleNCompare((const char *) magick+8,"avif",4) == 0)
-    return(MagickTrue);
-  if (LocaleNCompare((const char *) magick+8,"heic",4) == 0)
-    return(MagickTrue);
-  if (LocaleNCompare((const char *) magick+8,"heix",4) == 0)
-    return(MagickTrue);
-  if (LocaleNCompare((const char *) magick+8,"mif1",4) == 0)
+  heif_filetype=heif_check_filetype(magick,(int) length);
+  if (heif_filetype == heif_filetype_yes_supported)
     return(MagickTrue);
+#endif
+#endif
   return(MagickFalse);
 }
 
@@ -621,10 +631,32 @@ ModuleExport size_t RegisterHEICImage(void)
   entry=AcquireMagickInfo("HEIC","HEIC","High Efficiency Image Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadHEICImage;
+#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
+  if (heif_have_encoder_for_format(heif_compression_HEVC))
+    entry->encoder=(EncodeImageHandler *) WriteHEICImage;
+#else
   entry->encoder=(EncodeImageHandler *) WriteHEICImage;
+#endif
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
-  entry->mime_type=ConstantString("image/x-heic");
+  entry->mime_type=ConstantString("image/heic");
+#if defined(LIBHEIF_VERSION)
+  entry->version=ConstantString(LIBHEIF_VERSION);
+#endif
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  (void) RegisterMagickInfo(entry);
+  entry=AcquireMagickInfo("HEIC","HEIF","High Efficiency Image Format");
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadHEICImage;
+#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
+  if (heif_have_encoder_for_format(heif_compression_HEVC))
+    entry->encoder=(EncodeImageHandler *) WriteHEICImage;
+#else
+  entry->encoder=(EncodeImageHandler *) WriteHEICImage;
+#endif
+#endif
+  entry->magick=(IsImageFormatHandler *) IsHEIC;
+  entry->mime_type=ConstantString("image/heif");
 #if defined(LIBHEIF_VERSION)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
@@ -639,7 +671,7 @@ ModuleExport size_t RegisterHEICImage(void)
     entry->encoder=(EncodeImageHandler *) WriteHEICImage;
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
-  entry->mime_type=ConstantString("image/x-heic");
+  entry->mime_type=ConstantString("image/avif");
 #if defined(LIBHEIF_VERSION)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
@@ -670,7 +702,11 @@ ModuleExport size_t RegisterHEICImage(void)
 */
 ModuleExport void UnregisterHEICImage(void)
 {
+#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+  (void) UnregisterMagickInfo("AVIF");
+#endif
   (void) UnregisterMagickInfo("HEIC");
+  (void) UnregisterMagickInfo("HEIF");
 }
 
 /*
@@ -712,7 +748,7 @@ static void WriteProfile(struct heif_context *context,Image *image,
   const StringInfo
     *profile;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -749,8 +785,8 @@ static void WriteProfile(struct heif_context *context,Image *image,
               image->filename);
             length=65533L;
           }
-          (void) heif_context_add_exif_metadata(context,image_handle,
-            (void*) GetStringInfoDatum(profile),(int) length);
+        (void) heif_context_add_exif_metadata(context,image_handle,
+          (void*) GetStringInfoDatum(profile),(int) length);
       }
     if (LocaleCompare(name,"XMP") == 0)
       {
@@ -804,20 +840,18 @@ static struct heif_error heif_write_func(struct heif_context *context,
 static MagickBooleanType WriteHEICImageYCbCr(Image *image,
   struct heif_image *heif_image,ExceptionInfo *exception)
 {
+  int
+    bit_depth,
+    p_y,
+    p_cb,
+    p_cr;
+
   MagickBooleanType
     status;
 
   ssize_t
     y;
 
-  const Quantum
-    *p;
-
-  int
-    p_y,
-    p_cb,
-    p_cr;
-
   struct heif_error
     error;
 
@@ -827,22 +861,22 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
     *q_cr;
 
   status=MagickTrue;
-
+  bit_depth=8;
   error=heif_image_add_plane(heif_image,heif_channel_Y,(int) image->columns,
-    (int) image->rows,8);
-  status=IsHeifSuccess(&error,image,exception);
+    (int) image->rows,bit_depth);
+  status=IsHeifSuccess(image,&error,exception);
   if (status == MagickFalse)
-    return status;
+    return(status);
   error=heif_image_add_plane(heif_image,heif_channel_Cb,
-    ((int) image->columns+1)/2,((int) image->rows+1)/2,8);
-  status=IsHeifSuccess(&error,image,exception);
+    ((int) image->columns+1)/2,((int) image->rows+1)/2,bit_depth);
+  status=IsHeifSuccess(image,&error,exception);
   if (status == MagickFalse)
-    return status;
+    return(status);
   error=heif_image_add_plane(heif_image,heif_channel_Cr,
-    ((int) image->columns+1)/2,((int) image->rows+1)/2,8);
-  status=IsHeifSuccess(&error,image,exception);
+    ((int) image->columns+1)/2,((int) image->rows+1)/2,bit_depth);
+  status=IsHeifSuccess(image,&error,exception);
   if (status == MagickFalse)
-    return status;
+    return(status);
   q_y=heif_image_get_plane(heif_image,heif_channel_Y,&p_y);
   q_cb=heif_image_get_plane(heif_image,heif_channel_Cb,&p_cb);
   q_cr=heif_image_get_plane(heif_image,heif_channel_Cr,&p_cr);
@@ -851,7 +885,10 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
   */
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    const Quantum
+      *p;
+
+    ssize_t
       x;
 
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -860,7 +897,13 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
         status=MagickFalse;
         break;
       }
-    if ((y & 0x01) == 0)
+    if ((y & 0x01) != 0)
+      for (x=0; x < (ssize_t) image->columns; x++)
+      {
+        q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(image,p));
+        p+=GetPixelChannels(image);
+      }
+    else
       for (x=0; x < (ssize_t) image->columns; x+=2)
       {
         q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(image,p));
@@ -873,12 +916,6 @@ static MagickBooleanType WriteHEICImageYCbCr(Image *image,
             p+=GetPixelChannels(image);
           }
       }
-    else
-      for (x=0; x < (ssize_t) image->columns; x++)
-      {
-        q_y[y*p_y+x]=ScaleQuantumToChar(GetPixelRed(image,p));
-        p+=GetPixelChannels(image);
-      }
     if (image->previous == (Image *) NULL)
       {
         status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -915,7 +952,7 @@ static MagickBooleanType WriteHEICImageRGBA(Image *image,
   status=MagickTrue;
   error=heif_image_add_plane(heif_image,heif_channel_interleaved,
     (int) image->columns,(int) image->rows,8);
-  status=IsHeifSuccess(&error,image,exception);
+  status=IsHeifSuccess(image,&error,exception);
   if (status == MagickFalse)
     return status;
   plane=heif_image_get_plane(heif_image,heif_channel_interleaved,&stride);
@@ -924,7 +961,7 @@ static MagickBooleanType WriteHEICImageRGBA(Image *image,
   */
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -970,9 +1007,20 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   struct heif_encoder
     *heif_encoder;
 
+  struct heif_error
+    error;
+
   struct heif_image
     *heif_image;
 
+  struct heif_writer
+    writer;
+
+#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+  MagickBooleanType
+    encode_avif;
+#endif
+
   /*
     Open output image file.
   */
@@ -989,7 +1037,10 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   heif_context=heif_context_alloc();
   heif_image=(struct heif_image*) NULL;
   heif_encoder=(struct heif_encoder*) NULL;
-
+#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+  encode_avif=(LocaleCompare(image_info->magick,"AVIF") == 0) ?
+    MagickTrue : MagickFalse;
+#endif
   do
   {
 #if LIBHEIF_NUMERIC_VERSION >= 0x01040000
@@ -997,64 +1048,54 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
       *profile;
 #endif
 
-    struct heif_error
-      error;
-
-    struct heif_writer
-      writer;
-
-    enum heif_colorspace 
+    enum heif_colorspace
       colorspace;
 
-    enum heif_chroma 
+    enum heif_chroma
       chroma;
 
-    MagickBooleanType 
-      is_avif;
-
+    MagickBooleanType
+      lossless;
 
     colorspace=heif_colorspace_YCbCr;
-    chroma=heif_chroma_420;
-    is_avif=MagickFalse;
+    lossless=image_info->quality == 100 ? MagickTrue : MagickFalse;
+    chroma=lossless ? heif_chroma_444 : heif_chroma_420;
     /*
       Get encoder for the specified format.
     */
 #if LIBHEIF_NUMERIC_VERSION > 0x01060200
-    if (LocaleCompare(image_info->magick,"AVIF") == 0)
-      {
-        error=heif_context_get_encoder_for_format(heif_context,
-          heif_compression_AV1,&heif_encoder);
-        is_avif=MagickTrue;
-      }
+    if (encode_avif != MagickFalse)
+      error=heif_context_get_encoder_for_format(heif_context,
+        heif_compression_AV1,&heif_encoder);
     else
 #endif
       error=heif_context_get_encoder_for_format(heif_context,
         heif_compression_HEVC,&heif_encoder);
-    status=IsHeifSuccess(&error,image,exception);
+    status=IsHeifSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
-    if ((is_avif != MagickFalse) &&
-        (IssRGBCompatibleColorspace(image->colorspace) != MagickFalse))
+    if (image->alpha_trait == BlendPixelTrait)
       {
         colorspace=heif_colorspace_RGB;
-        chroma=(image->alpha_trait == UndefinedPixelTrait) ?
-          heif_chroma_interleaved_RGB : heif_chroma_interleaved_RGBA;
-      } 
-    else if (image->colorspace != YCbCrColorspace)
-      {
-        /*
-        Transform colorspace to YCbCr.
-        */
-        status=TransformImageColorspace(image,YCbCrColorspace,exception);
-        if (status == MagickFalse)
-          break;
+        chroma=heif_chroma_interleaved_RGBA;
+        if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+          status=TransformImageColorspace(image,sRGBColorspace,exception);
       }
+    else if (IssRGBCompatibleColorspace(image->colorspace) != MagickFalse)
+      {
+        colorspace=heif_colorspace_RGB;
+        chroma=heif_chroma_interleaved_RGB;
+      }
+    else if (image->colorspace != YCbCrColorspace)
+      status=TransformImageColorspace(image,YCbCrColorspace,exception);
+    if (status == MagickFalse)
+      break;
     /*
       Initialize HEIF encoder context.
     */
-    error=heif_image_create((int) image->columns,(int) image->rows,
-      colorspace,chroma,&heif_image);
-    status=IsHeifSuccess(&error,image,exception);
+    error=heif_image_create((int) image->columns,(int) image->rows,colorspace,
+      chroma,&heif_image);
+    status=IsHeifSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
 #if LIBHEIF_NUMERIC_VERSION >= 0x01040000
@@ -1072,30 +1113,48 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     /*
       Code and actually write the HEIC image
     */
-    if (image_info->quality != UndefinedCompressionQuality)
+    if (lossless != MagickFalse)
+      error=heif_encoder_set_lossless(heif_encoder,1);
+    else if (image_info->quality != UndefinedCompressionQuality)
+      error=heif_encoder_set_lossy_quality(heif_encoder,(int)
+        image_info->quality);
+    status=IsHeifSuccess(image,&error,exception);
+    if (status == MagickFalse)
+      break;
+#if LIBHEIF_NUMERIC_VERSION > 0x01060200
+    if (encode_avif != MagickFalse)
       {
-        error=heif_encoder_set_lossy_quality(heif_encoder,(int)
-          image_info->quality);
-        status=IsHeifSuccess(&error,image,exception);
-        if (status == MagickFalse)
-          break;
+        const char
+          *option;
+
+        option=GetImageOption(image_info,"heic:speed");
+        if (option != (char *) NULL)
+          {
+            error=heif_encoder_set_parameter(heif_encoder,"speed",option);
+            status=IsHeifSuccess(image,&error,exception);
+            if (status == MagickFalse)
+              break;
+          }
+        option=GetImageOption(image_info,"heic:chroma");
+        if (option != (char *) NULL)
+          {
+            error=heif_encoder_set_parameter(heif_encoder,"chroma",option);
+            status=IsHeifSuccess(image,&error,exception);
+            if (status == MagickFalse)
+              break;
+          }
       }
+#endif
     error=heif_context_encode_image(heif_context,heif_image,heif_encoder,
       (const struct heif_encoding_options *) NULL,
       (struct heif_image_handle **) NULL);
-    status=IsHeifSuccess(&error,image,exception);
+    status=IsHeifSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
-    writer.writer_api_version=1;
-    writer.write=heif_write_func;
 #if LIBHEIF_NUMERIC_VERSION >= 0x01030000
     if (image->profiles != (void *) NULL)
-      WriteProfile(heif_context, image, exception);
+      WriteProfile(heif_context,image,exception);
 #endif
-    error=heif_context_write(heif_context,&writer,image);
-    status=IsHeifSuccess(&error,image,exception);
-    if (status == MagickFalse)
-      break;
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
@@ -1109,6 +1168,17 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     heif_image=(struct heif_image*) NULL;
     scene++;
   } while (image_info->adjoin != MagickFalse);
+  if (status != MagickFalse)
+    {
+      writer.writer_api_version=1;
+      writer.write=heif_write_func;
+#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
+      if (image->profiles != (void *) NULL)
+        WriteProfile(heif_context,image,exception);
+#endif
+      error=heif_context_write(heif_context,&writer,image);
+      status=IsHeifSuccess(image,&error,exception);
+    }
   if (heif_encoder != (struct heif_encoder*) NULL)
     heif_encoder_release(heif_encoder);
   if (heif_image != (struct heif_image*) NULL)
