@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.  You may
@@ -17,14 +17,15 @@
 #include "coders/coders-private.h"
 
 #define MagickHEICHeaders \
-  MagickCoderHeader("HEIC", 4, "ftypavif") \
+  MagickCoderHeader("AVIF", 4, "ftypavif") \
   MagickCoderHeader("HEIC", 4, "ftypheic") \
   MagickCoderHeader("HEIC", 4, "ftypheix") \
-  MagickCoderHeader("HEIC", 4, "ftypmif1")
+  MagickCoderHeader("HEIC", 4, "ftypmif1") \
+  MagickCoderHeader("HEIC", 4, "ftypmsf1")
 
 #define MagickHEICAliases \
-  MagickCoderAlias("HEIC", "HEIC") \
-  MagickCoderAlias("HEIC", "AVIF")
+  MagickCoderAlias("HEIC", "AVIF") \
+  MagickCoderAlias("HEIC", "HEIF")
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -34,4 +35,4 @@ MagickCoderExports(HEIC)
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
-#endif
\ No newline at end of file
+#endif
