@@ -17,7 +17,7 @@
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,37 +39,37 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/artifact.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/color.h"
-#include "magick/color-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/profile.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/resource_.h"
-#include "magick/semaphore.h"
-#include "magick/static.h"
-#include "magick/statistic.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/color.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/profile.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/semaphore.h"
+#include "MagickCore/static.h"
+#include "MagickCore/statistic.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/module.h"
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
 #include <openjpeg.h>
 #endif
@@ -79,7 +79,7 @@
 */
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
 static MagickBooleanType
-  WriteJP2Image(const ImageInfo *,Image *);
+  WriteJP2Image(const ImageInfo *,Image *,ExceptionInfo *);
 #endif
 
 /*
@@ -277,7 +277,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   opj_stream_t
     *jp2_stream;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -296,7 +296,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -326,8 +326,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   if (option != (const char *) NULL)
     parameters.cp_reduce=StringToInteger(option);
   option=GetImageOption(image_info,"jp2:quality-layers");
-  if (option == (const char *) NULL)
-    option=GetImageOption(image_info,"jp2:layer-number");
   if (option != (const char *) NULL)
     parameters.cp_layer=StringToInteger(option);
   if (opj_setup_decoder(jp2_codec,&parameters) == 0)
@@ -335,7 +333,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       opj_destroy_codec(jp2_codec);
       ThrowReaderException(DelegateError,"UnableToManageJP2Stream");
     }
-  jp2_stream=opj_stream_create(OPJ_J2K_STREAM_CHUNK_SIZE,OPJ_TRUE);
+  jp2_stream=opj_stream_create(OPJ_J2K_STREAM_CHUNK_SIZE,1);
   opj_stream_set_read_function(jp2_stream,JP2ReadHandler);
   opj_stream_set_write_function(jp2_stream,JP2WriteHandler);
   opj_stream_set_seek_function(jp2_stream,JP2SeekHandler);
@@ -349,6 +347,14 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
     }
   jp2_status=OPJ_TRUE;
+  if ((AcquireMagickResource(WidthResource,(size_t) jp2_image->comps[0].w) == MagickFalse) ||
+      (AcquireMagickResource(HeightResource,(size_t) jp2_image->comps[0].h) == MagickFalse))
+    {
+      opj_stream_destroy(jp2_stream);
+      opj_destroy_codec(jp2_codec);
+      opj_image_destroy(jp2_image);
+      ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
+    }
   if (image->ping == MagickFalse)
     {
       if ((image->columns != 0) && (image->rows != 0))
@@ -357,8 +363,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         jp2_status=opj_set_decode_area(jp2_codec,jp2_image,
           (OPJ_INT32) image->extract_info.x,(OPJ_INT32) image->extract_info.y,
-          (OPJ_INT32) image->extract_info.x+(ssize_t) image->columns,
-          (OPJ_INT32) image->extract_info.y+(ssize_t) image->rows);
+          (OPJ_INT32) (image->extract_info.x+(ssize_t) image->columns),
+          (OPJ_INT32) (image->extract_info.y+(ssize_t) image->rows));
       else
         jp2_status=opj_set_decode_area(jp2_codec,jp2_image,0,0,
           jp2_image->comps[0].w,jp2_image->comps[0].h);
@@ -370,14 +376,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
         }
     }
-  if ((AcquireMagickResource(WidthResource,(size_t) jp2_image->comps[0].w) == MagickFalse) ||
-      (AcquireMagickResource(HeightResource,(size_t) jp2_image->comps[0].h) == MagickFalse))
-    {
-      opj_stream_destroy(jp2_stream);
-      opj_destroy_codec(jp2_codec);
-      opj_image_destroy(jp2_image);
-      ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
-    }
   if ((image_info->number_scenes != 0) && (image_info->scene != 0))
     jp2_status=opj_get_decoded_tile(jp2_codec,jp2_stream,jp2_image,
       (unsigned int) image_info->scene-1);
@@ -398,8 +396,9 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   opj_stream_destroy(jp2_stream);
   for (i=0; i < (ssize_t) jp2_image->numcomps; i++)
   {
-    if ((jp2_image->comps[0].dx == 0) || (jp2_image->comps[0].dy == 0) ||
+    if ((jp2_image->comps[i].dx == 0) || (jp2_image->comps[i].dy == 0) ||
         (jp2_image->comps[0].prec != jp2_image->comps[i].prec) ||
+        (jp2_image->comps[0].prec > 64) ||
         (jp2_image->comps[0].sgnd != jp2_image->comps[i].sgnd) ||
         ((image->ping == MagickFalse) && (jp2_image->comps[i].data == NULL)))
       {
@@ -416,19 +415,19 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   image->depth=jp2_image->comps[0].prec;
   image->compression=JPEG2000Compression;
   if (jp2_image->numcomps == 1)
-    SetImageColorspace(image,GRAYColorspace);
+    SetImageColorspace(image,GRAYColorspace,exception);
   else
     if (jp2_image->color_space == 2)
       {
-        SetImageColorspace(image,GRAYColorspace);
+        SetImageColorspace(image,GRAYColorspace,exception);
         if (jp2_image->numcomps > 1)
-          image->matte=MagickTrue;
+          image->alpha_trait=BlendPixelTrait;
       }
     else
       if (jp2_image->color_space == 3)
-        SetImageColorspace(image,Rec601YCbCrColorspace);
+        SetImageColorspace(image,Rec601YCbCrColorspace,exception);
   if (jp2_image->numcomps > 3)
-    image->matte=MagickTrue;
+    image->alpha_trait=BlendPixelTrait;
   if (jp2_image->icc_profile_buf != (unsigned char *) NULL)
     {
       StringInfo
@@ -438,7 +437,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         jp2_image->icc_profile_len);
       if (profile != (StringInfo *) NULL)
         {
-          SetImageProfile(image,"icc",profile);
+          SetImageProfile(image,"icc",profile,exception);
           profile=DestroyStringInfo(profile);
         }
     }
@@ -448,39 +447,49 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       opj_image_destroy(jp2_image);
       return(GetFirstImageInList(image));
     }
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     {
       opj_destroy_codec(jp2_codec);
       opj_image_destroy(jp2_image);
-      InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register PixelPacket
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    if (q == (Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
-        i;
-
       for (i=0; i < (ssize_t) jp2_image->numcomps; i++)
       {
         double
           pixel,
           scale;
 
+        ssize_t
+          index,
+          pad;
+
+        pad=image->columns % jp2_image->comps[i].dx;
+        index=y/jp2_image->comps[i].dy*(image->columns+pad)/
+          jp2_image->comps[i].dx+x/jp2_image->comps[i].dx;
+        if ((index < 0) ||
+            (index >= (jp2_image->comps[i].h*jp2_image->comps[i].w)))
+          {
+            opj_destroy_codec(jp2_codec);
+            opj_image_destroy(jp2_image);
+            ThrowReaderException(CoderError,
+              "IrregularChannelGeometryNotSupported")
+          }
         scale=QuantumRange/(double) ((1UL << jp2_image->comps[i].prec)-1);
-        pixel=scale*(jp2_image->comps[i].data[y/jp2_image->comps[i].dy*
-          image->columns/jp2_image->comps[i].dx+x/jp2_image->comps[i].dx]+
+        pixel=scale*(jp2_image->comps[i].data[index]+
           (jp2_image->comps[i].sgnd ? 1UL << (jp2_image->comps[i].prec-1) : 0));
         switch (i)
         {
@@ -488,39 +497,39 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
            {
              if (jp2_image->numcomps == 1)
                {
-                 SetPixelGray(q,ClampToQuantum(pixel));
-                 SetPixelOpacity(q,OpaqueOpacity);
+                 SetPixelGray(image,ClampToQuantum(pixel),q);
+                 SetPixelAlpha(image,OpaqueAlpha,q);
                  break;
                }
-             q->red=ClampToQuantum(pixel);
-             q->green=q->red;
-             q->blue=q->red;
-             q->opacity=OpaqueOpacity;
+             SetPixelRed(image,ClampToQuantum(pixel),q);
+             SetPixelGreen(image,ClampToQuantum(pixel),q);
+             SetPixelBlue(image,ClampToQuantum(pixel),q);
+             SetPixelAlpha(image,OpaqueAlpha,q);
              break;
            }
            case 1:
            {
              if (jp2_image->numcomps == 2)
                {
-                 q->opacity=ClampToQuantum(QuantumRange-pixel);
+                 SetPixelAlpha(image,ClampToQuantum(pixel),q);
                  break;
                }
-             q->green=ClampToQuantum(pixel);
+             SetPixelGreen(image,ClampToQuantum(pixel),q);
              break;
            }
            case 2:
            {
-             q->blue=ClampToQuantum(pixel);
+             SetPixelBlue(image,ClampToQuantum(pixel),q);
              break;
            }
            case 3:
            {
-             q->opacity=ClampToQuantum(QuantumRange-pixel);
+             SetPixelAlpha(image,ClampToQuantum(pixel),q);
              break;
            }
         }
       }
-      q++;
+      q+=GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
@@ -566,94 +575,88 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
 ModuleExport size_t RegisterJP2Image(void)
 {
   char
-    version[MaxTextExtent];
+    version[MagickPathExtent];
 
   MagickInfo
     *entry;
 
   *version='\0';
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
-  (void) FormatLocaleString(version,MaxTextExtent,"%s",opj_version());
+  (void) FormatLocaleString(version,MagickPathExtent,"%s",opj_version());
 #endif
-  entry=SetMagickInfo("JP2");
-  entry->description=ConstantString("JPEG-2000 File Format Syntax");
+  entry=AcquireMagickInfo("JP2","JP2","JPEG-2000 File Format Syntax");
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jp2");
-  entry->magick_module=ConstantString("JP2");
   entry->magick=(IsImageFormatHandler *) IsJP2;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEncoderSeekableStreamFlag;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("J2C");
-  entry->description=ConstantString("JPEG-2000 Code Stream Syntax");
+  entry=AcquireMagickInfo("JP2","J2C","JPEG-2000 Code Stream Syntax");
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jp2");
-  entry->magick_module=ConstantString("JP2");
   entry->magick=(IsImageFormatHandler *) IsJ2K;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEncoderSeekableStreamFlag;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("J2K");
-  entry->description=ConstantString("JPEG-2000 Code Stream Syntax");
+  entry=AcquireMagickInfo("JP2","J2K","JPEG-2000 Code Stream Syntax");
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jp2");
-  entry->magick_module=ConstantString("JP2");
   entry->magick=(IsImageFormatHandler *) IsJ2K;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEncoderSeekableStreamFlag;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("JPM");
-  entry->description=ConstantString("JPEG-2000 Code Stream Syntax");
+  entry=AcquireMagickInfo("JP2","JPM","JPEG-2000 File Format Syntax");
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jp2");
-  entry->magick_module=ConstantString("JP2");
   entry->magick=(IsImageFormatHandler *) IsJP2;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEncoderSeekableStreamFlag;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("JPT");
-  entry->description=ConstantString("JPEG-2000 File Format Syntax");
+  entry=AcquireMagickInfo("JP2","JPT","JPEG-2000 File Format Syntax");
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jp2");
-  entry->magick_module=ConstantString("JP2");
   entry->magick=(IsImageFormatHandler *) IsJP2;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEncoderSeekableStreamFlag;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("JPC");
-  entry->description=ConstantString("JPEG-2000 Code Stream Syntax");
+  entry=AcquireMagickInfo("JP2","JPC","JPEG-2000 Code Stream Syntax");
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jp2");
-  entry->magick_module=ConstantString("JP2");
   entry->magick=(IsImageFormatHandler *) IsJP2;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags|=CoderEncoderSeekableStreamFlag;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -708,7 +711,8 @@ ModuleExport void UnregisterJP2Image(void)
 %
 %  The format of the WriteJP2Image method is:
 %
-%      MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -799,7 +803,8 @@ static inline int CalculateNumResolutions(size_t width,size_t height)
   return(i);
 }
 
-static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   const char
     *option,
@@ -829,7 +834,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   opj_stream_t
     *jp2_stream;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -847,13 +852,15 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   /*
-    Initialize JPEG 2000 encoder parameters.
+    Initialize JPEG 2000 API.
   */
-  parameters=AcquireMagickMemory(sizeof(*parameters));
+  parameters=(opj_cparameters_t *) AcquireMagickMemory(sizeof(*parameters));
   if (parameters == (opj_cparameters_t *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   opj_set_default_encoder_parameters(parameters);
@@ -870,6 +877,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     {
       parameters->tcp_distoratio[0]=(double) image_info->quality;
       parameters->cp_fixed_quality=OPJ_TRUE;
+      parameters->cp_disto_alloc=0;
     }
   if (image_info->extract != (char *) NULL)
     {
@@ -899,7 +907,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   option=GetImageOption(image_info,"jp2:quality");
   if (option != (const char *) NULL)
     {
-      register const char
+      const char
         *p;
 
       /*
@@ -908,7 +916,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       p=option;
       for (i=0; sscanf(p,"%f",&parameters->tcp_distoratio[i]) == 1; i++)
       {
-        if (i >= 100)
+        if (i > 100)
           break;
         while ((*p != '\0') && (*p != ','))
           p++;
@@ -918,6 +926,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       }
       parameters->tcp_numlayers=i+1;
       parameters->cp_fixed_quality=OPJ_TRUE;
+      parameters->cp_disto_alloc=0;
     }
   option=GetImageOption(image_info,"jp2:progression-order");
   if (option != (const char *) NULL)
@@ -936,7 +945,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   option=GetImageOption(image_info,"jp2:rate");
   if (option != (const char *) NULL)
     {
-      register const char
+      const char
         *p;
 
       /*
@@ -945,7 +954,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       p=option;
       for (i=0; sscanf(p,"%f",&parameters->tcp_rates[i]) == 1; i++)
       {
-        if (i > 100)
+        if (i >= 100)
           break;
         while ((*p != '\0') && (*p != ','))
           p++;
@@ -959,7 +968,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   if (image_info->sampling_factor != (const char *) NULL)
     (void) sscanf(image_info->sampling_factor,"%d,%d",
       &parameters->subsampling_dx,&parameters->subsampling_dy);
-  property=GetImageProperty(image,"comment");
+  property=GetImageProperty(image,"comment",exception);
   if (property != (const char *) NULL)
     parameters->cp_comment=(char *) property;
   channels=3;
@@ -977,16 +986,17 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
           jp2_colorspace=OPJ_CLRSPC_GRAY;
         }
       else
-        (void) TransformImageColorspace(image,sRGBColorspace);
-      if (image->matte != MagickFalse)
+        if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+          (void) TransformImageColorspace(image,sRGBColorspace,exception);
+      if (image->alpha_trait != UndefinedPixelTrait)
         channels++;
     }
   parameters->tcp_mct=channels == 3 ? 1 : 0;
   memset(jp2_info,0,sizeof(jp2_info));
   for (i=0; i < (ssize_t) channels; i++)
   {
-    jp2_info[i].prec=(unsigned int) image->depth;
-    jp2_info[i].bpp=(unsigned int) image->depth;
+    jp2_info[i].prec=(OPJ_UINT32) image->depth;
+    jp2_info[i].bpp=(OPJ_UINT32) image->depth;
     if ((image->depth == 1) &&
         ((LocaleCompare(image_info->magick,"JPT") == 0) ||
          (LocaleCompare(image_info->magick,"JP2") == 0)))
@@ -997,10 +1007,10 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     jp2_info[i].sgnd=0;
     jp2_info[i].dx=parameters->subsampling_dx;
     jp2_info[i].dy=parameters->subsampling_dy;
-    jp2_info[i].w=(unsigned int) image->columns;
-    jp2_info[i].h=(unsigned int) image->rows;
+    jp2_info[i].w=(OPJ_UINT32) image->columns;
+    jp2_info[i].h=(OPJ_UINT32) image->rows;
   }
-  jp2_image=opj_image_create(channels,jp2_info,jp2_colorspace);
+  jp2_image=opj_image_create((OPJ_UINT32) channels,jp2_info,jp2_colorspace);
   if (jp2_image == (opj_image_t *) NULL)
     {
       parameters=(opj_cparameters_t *) RelinquishMagickMemory(parameters);
@@ -1026,14 +1036,14 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   */
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const Quantum
       *p;
 
     ssize_t
       x;
 
-    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
@@ -1042,48 +1052,50 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         double
           scale;
 
-        register int
+        int
           *q;
 
         scale=(double) (((size_t) 1UL << jp2_image->comps[i].prec)-1)/
           QuantumRange;
-        q=jp2_image->comps[i].data+(y/jp2_image->comps[i].dy*
-          image->columns/jp2_image->comps[i].dx+x/jp2_image->comps[i].dx);
+        q=jp2_image->comps[i].data+(ssize_t) (y*PerceptibleReciprocal(
+          jp2_image->comps[i].dy)*image->columns*PerceptibleReciprocal(
+          jp2_image->comps[i].dx)+x*PerceptibleReciprocal(
+          jp2_image->comps[i].dx));
         switch (i)
         {
           case 0:
           {
             if (jp2_colorspace == OPJ_CLRSPC_GRAY)
               {
-                *q=(int) (scale*GetPixelGray(p));
+                *q=(int) (scale*GetPixelGray(image,p));
                 break;
               }
-            *q=(int) (scale*p->red);
+            *q=(int) (scale*GetPixelRed(image,p));
             break;
           }
           case 1:
           {
             if (jp2_colorspace == OPJ_CLRSPC_GRAY)
               {
-                *q=(int) (scale*(QuantumRange-p->opacity));
+                *q=(int) (scale*GetPixelAlpha(image,p));
                 break;
               }
-            *q=(int) (scale*p->green);
+            *q=(int) (scale*GetPixelGreen(image,p));
             break;
           }
           case 2:
           {
-            *q=(int) (scale*p->blue);
+            *q=(int) (scale*GetPixelBlue(image,p));
             break;
           }
           case 3:
           {
-            *q=(int) (scale*(QuantumRange-p->opacity));
+            *q=(int) (scale*GetPixelAlpha(image,p));
             break;
           }
         }
       }
-      p++;
+      p+=GetPixelChannels(image);
     }
     status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
       image->rows);
@@ -1097,8 +1109,8 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       jp2_codec=opj_create_compress(OPJ_CODEC_J2K);
     else
       jp2_codec=opj_create_compress(OPJ_CODEC_JP2);
-  opj_set_warning_handler(jp2_codec,JP2WarningHandler,&image->exception);
-  opj_set_error_handler(jp2_codec,JP2ErrorHandler,&image->exception);
+  opj_set_warning_handler(jp2_codec,JP2WarningHandler,exception);
+  opj_set_error_handler(jp2_codec,JP2ErrorHandler,exception);
   opj_setup_encoder(jp2_codec,parameters,jp2_image);
   jp2_stream=opj_stream_create(OPJ_J2K_STREAM_CHUNK_SIZE,OPJ_FALSE);
   if (jp2_stream == (opj_stream_t *) NULL)
