@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -85,6 +85,7 @@
 #include "MagickCore/utility.h"
 #include "MagickCore/xml-tree.h"
 #include "MagickCore/xml-tree-private.h"
+#include "coders/coders-private.h"
 #include <setjmp.h>
 #if defined(MAGICKCORE_JPEG_DELEGATE)
 #define JPEG_INTERNAL_OPTIONS
@@ -125,7 +126,7 @@ typedef struct _DestinationManager
     *buffer;
 } DestinationManager;
 
-typedef struct _ErrorManager
+typedef struct _JPEGClientInfo
 {
   jmp_buf
     error_recovery;
@@ -141,7 +142,7 @@ typedef struct _ErrorManager
 
   ExceptionInfo
     *exception;
-} ErrorManager;
+} JPEGClientInfo;
 
 typedef struct _SourceManager
 {
@@ -254,19 +255,19 @@ static MagickBooleanType IsJPEG(const unsigned char *magick,const size_t length)
 %
 */
 
-static boolean FillInputBuffer(j_decompress_ptr cinfo)
+static boolean FillInputBuffer(j_decompress_ptr compress_info)
 {
   SourceManager
     *source;
 
-  source=(SourceManager *) cinfo->src;
+  source=(SourceManager *) compress_info->src;
   source->manager.bytes_in_buffer=(size_t) ReadBlob(source->image,
     MagickMinBufferExtent,source->buffer);
   if (source->manager.bytes_in_buffer == 0)
     {
       if (source->start_of_blob != FALSE)
-        ERREXIT(cinfo,JERR_INPUT_EMPTY);
-      WARNMS(cinfo,JWRN_JPEG_EOF);
+        ERREXIT(compress_info,JERR_INPUT_EMPTY);
+      WARNMS(compress_info,JWRN_JPEG_EOF);
       source->buffer[0]=(JOCTET) 0xff;
       source->buffer[1]=(JOCTET) JPEG_EOI;
       source->manager.bytes_in_buffer=2;
@@ -288,12 +289,12 @@ static int GetCharacter(j_decompress_ptr jpeg_info)
   return((int) GETJOCTET(*jpeg_info->src->next_input_byte++));
 }
 
-static void InitializeSource(j_decompress_ptr cinfo)
+static void InitializeSource(j_decompress_ptr compress_info)
 {
   SourceManager
     *source;
 
-  source=(SourceManager *) cinfo->src;
+  source=(SourceManager *) compress_info->src;
   source->start_of_blob=TRUE;
 }
 
@@ -322,75 +323,75 @@ static void JPEGErrorHandler(j_common_ptr jpeg_info)
   char
     message[JMSG_LENGTH_MAX];
 
-  ErrorManager
-    *error_manager;
-
   ExceptionInfo
     *exception;
 
   Image
     *image;
 
+  JPEGClientInfo
+    *client_info;
+
   *message='\0';
-  error_manager=(ErrorManager *) jpeg_info->client_data;
-  image=error_manager->image;
-  exception=error_manager->exception;
+  client_info=(JPEGClientInfo *) jpeg_info->client_data;
+  image=client_info->image;
+  exception=client_info->exception;
   (jpeg_info->err->format_message)(jpeg_info,message);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "[%s] JPEG Trace: \"%s\"",image->filename,message);
-  if (error_manager->finished != MagickFalse)
-    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageWarning,
-      (char *) message,"`%s'",image->filename);
+  if (client_info->finished != MagickFalse)
+    (void) ThrowMagickException(exception,GetMagickModule(),
+      CorruptImageWarning,(char *) message,"`%s'",image->filename);
   else
     (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
       (char *) message,"`%s'",image->filename);
-  longjmp(error_manager->error_recovery,1);
+  longjmp(client_info->error_recovery,1);
 }
 
 static void JPEGProgressHandler(j_common_ptr jpeg_info)
 {
-  ErrorManager
-    *error_manager;
-
   ExceptionInfo
     *exception;
 
   Image
     *image;
 
-  error_manager=(ErrorManager *) jpeg_info->client_data;
-  image=error_manager->image;
-  exception=error_manager->exception;
+  JPEGClientInfo
+    *client_info;
+
+  client_info=(JPEGClientInfo *) jpeg_info->client_data;
+  image=client_info->image;
+  exception=client_info->exception;
   if (jpeg_info->is_decompressor == 0)
     return;
   if (((j_decompress_ptr) jpeg_info)->input_scan_number < MaxJPEGScans)
     return;
   (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
     "too many scans","`%s'",image->filename);
-  longjmp(error_manager->error_recovery,1);
+  longjmp(client_info->error_recovery,1);
 }
 
-static MagickBooleanType JPEGWarningHandler(j_common_ptr jpeg_info,int level)
+static void JPEGWarningHandler(j_common_ptr jpeg_info,int level)
 {
 #define JPEGExcessiveWarnings  1000
 
   char
     message[JMSG_LENGTH_MAX];
 
-  ErrorManager
-    *error_manager;
-
   ExceptionInfo
     *exception;
 
   Image
     *image;
 
+  JPEGClientInfo
+    *client_info;
+
   *message='\0';
-  error_manager=(ErrorManager *) jpeg_info->client_data;
-  exception=error_manager->exception;
-  image=error_manager->image;
+  client_info=(JPEGClientInfo *) jpeg_info->client_data;
+  exception=client_info->exception;
+  image=client_info->image;
   if (level < 0)
     {
       /*
@@ -398,66 +399,65 @@ static MagickBooleanType JPEGWarningHandler(j_common_ptr jpeg_info,int level)
       */
       (jpeg_info->err->format_message)(jpeg_info,message);
       if (jpeg_info->err->num_warnings++ < JPEGExcessiveWarnings)
-        ThrowBinaryException(CorruptImageWarning,(char *) message,
-          image->filename);
+        (void) ThrowMagickException(exception,GetMagickModule(),
+          CorruptImageWarning,message,"`%s'",image->filename);
     }
   else
-    if ((image->debug != MagickFalse) &&
-        (level >= jpeg_info->err->trace_level))
+    if (level >= jpeg_info->err->trace_level)
       {
         /*
           Process trace message.
         */
         (jpeg_info->err->format_message)(jpeg_info,message);
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "[%s] JPEG Trace: \"%s\"",image->filename,message);
+        if ((image != (Image *) NULL) && (image->debug != MagickFalse))
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "[%s] JPEG Trace: \"%s\"",image->filename,message);
       }
-  return(MagickTrue);
 }
 
 static boolean ReadProfileData(j_decompress_ptr jpeg_info,const size_t index,
   const size_t length)
 {
-  ErrorManager
-    *error_manager;
-
   ExceptionInfo
     *exception;
 
   Image
     *image;
 
-  register unsigned char
+  JPEGClientInfo
+    *client_info;
+
+  unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  error_manager=(ErrorManager *) jpeg_info->client_data;
-  exception=error_manager->exception;
-  image=error_manager->image;
-  if (error_manager->profiles[index] == (StringInfo *) NULL)
+  client_info=(JPEGClientInfo *) jpeg_info->client_data;
+  exception=client_info->exception;
+  image=client_info->image;
+  if (client_info->profiles[index] == (StringInfo *) NULL)
     {
-      error_manager->profiles[index]=BlobToStringInfo((const void *) NULL,
+      client_info->profiles[index]=BlobToStringInfo((const void *) NULL,
         length);
-      if (error_manager->profiles[index] == (StringInfo *) NULL)
+      if (client_info->profiles[index] == (StringInfo *) NULL)
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",
             image->filename);
           return(FALSE);
         }
-      p=GetStringInfoDatum(error_manager->profiles[index]);
+      p=GetStringInfoDatum(client_info->profiles[index]);
     }
   else
     {
       size_t
         current_length;
 
-      current_length=GetStringInfoLength(error_manager->profiles[index]);
-      SetStringInfoLength(error_manager->profiles[index],current_length+
+      current_length=GetStringInfoLength(client_info->profiles[index]);
+      SetStringInfoLength(client_info->profiles[index],current_length+
         length);
-      p=GetStringInfoDatum(error_manager->profiles[index])+current_length;
+      p=GetStringInfoDatum(client_info->profiles[index])+current_length;
     }
   for (i=0; i < (ssize_t) length; i++)
   {
@@ -515,7 +515,7 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   char
     magick[13];
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -559,7 +559,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   char
     magick[MagickPathExtent];
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -631,19 +631,19 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
 
 static boolean ReadXmlProfile(j_decompress_ptr jpeg_info)
 {
-  ErrorManager
-    *error_manager;
-
   ExceptionInfo
     *exception;
 
   Image
     *image;
 
+  JPEGClientInfo
+    *client_info;
+
   MagickBooleanType
     status;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -658,10 +658,10 @@ static boolean ReadXmlProfile(j_decompress_ptr jpeg_info)
   length-=2;
   if (ReadProfileData(jpeg_info,XML_INDEX,length) == FALSE)
     return(FALSE);
-  error_manager=(ErrorManager *) jpeg_info->client_data;
-  exception=error_manager->exception;
-  image=error_manager->image;
-  profile=error_manager->profiles[XML_INDEX];
+  client_info=(JPEGClientInfo *) jpeg_info->client_data;
+  exception=client_info->exception;
+  image=client_info->image;
+  profile=client_info->profiles[XML_INDEX];
   p=GetStringInfoDatum(profile);
   length=GetStringInfoLength(profile);
   status=MagickTrue;
@@ -695,44 +695,45 @@ static boolean ReadXmlProfile(j_decompress_ptr jpeg_info)
       }
     else
       status=SetImageProfile(image,"app1",profile,exception);
-  error_manager->profiles[XML_INDEX]=DestroyStringInfo(
-    error_manager->profiles[XML_INDEX]);
+  client_info->profiles[XML_INDEX]=DestroyStringInfo(
+    client_info->profiles[XML_INDEX]);
   return(status != MagickFalse ? TRUE : FALSE);
 }
 
-static void SkipInputData(j_decompress_ptr cinfo,long number_bytes)
+static void SkipInputData(j_decompress_ptr compress_info,long number_bytes)
 {
   SourceManager
     *source;
 
   if (number_bytes <= 0)
     return;
-  source=(SourceManager *) cinfo->src;
+  source=(SourceManager *) compress_info->src;
   while (number_bytes > (long) source->manager.bytes_in_buffer)
   {
     number_bytes-=(long) source->manager.bytes_in_buffer;
-    (void) FillInputBuffer(cinfo);
+    (void) FillInputBuffer(compress_info);
   }
   source->manager.next_input_byte+=number_bytes;
   source->manager.bytes_in_buffer-=number_bytes;
 }
 
-static void TerminateSource(j_decompress_ptr cinfo)
+static void TerminateSource(j_decompress_ptr compress_info)
 {
-  (void) cinfo;
+  (void) compress_info;
 }
 
-static void JPEGSourceManager(j_decompress_ptr cinfo,Image *image)
+static void JPEGSourceManager(j_decompress_ptr compress_info,Image *image)
 {
   SourceManager
     *source;
 
-  cinfo->src=(struct jpeg_source_mgr *) (*cinfo->mem->alloc_small)
-    ((j_common_ptr) cinfo,JPOOL_IMAGE,sizeof(SourceManager));
-  source=(SourceManager *) cinfo->src;
-  source->buffer=(JOCTET *) (*cinfo->mem->alloc_small)
-    ((j_common_ptr) cinfo,JPOOL_IMAGE,MagickMinBufferExtent*sizeof(JOCTET));
-  source=(SourceManager *) cinfo->src;
+  compress_info->src=(struct jpeg_source_mgr *) (*
+    compress_info->mem->alloc_small) ((j_common_ptr) compress_info,JPOOL_IMAGE,
+    sizeof(SourceManager));
+  source=(SourceManager *) compress_info->src;
+  source->buffer=(JOCTET *) (*compress_info->mem->alloc_small) ((j_common_ptr)
+    compress_info,JPOOL_IMAGE,MagickMinBufferExtent*sizeof(JOCTET));
+  source=(SourceManager *) compress_info->src;
   source->manager.init_source=InitializeSource;
   source->manager.fill_input_buffer=FillInputBuffer;
   source->manager.skip_input_data=SkipInputData;
@@ -762,7 +763,7 @@ static void JPEGSetImageQuality(struct jpeg_decompress_struct *jpeg_info,
       qvalue,
       sum;
 
-    register ssize_t
+    ssize_t
       i;
 
     /*
@@ -944,22 +945,22 @@ static void JPEGSetImageSamplingFactor(struct jpeg_decompress_struct *jpeg_info,
 
 static void JPEGDestroyDecompress(j_decompress_ptr jpeg_info)
 {
-  ErrorManager
-    *error_manager;
+  JPEGClientInfo
+    *client_info;
 
-  register size_t
+  size_t
     i;
 
-  error_manager=(ErrorManager *) jpeg_info->client_data;
+  client_info=(JPEGClientInfo *) jpeg_info->client_data;
   for (i=0; i < 16; i++)
   {
-    if (error_manager->profiles[i] != (StringInfo *) NULL)
-      error_manager->profiles[i]=DestroyStringInfo(error_manager->profiles[i]);
+    if (client_info->profiles[i] != (StringInfo *) NULL)
+      client_info->profiles[i]=DestroyStringInfo(client_info->profiles[i]);
   }
   jpeg_destroy_decompress(jpeg_info);
 }
 
-static MagickBooleanType JPEGSetImageProfiles(ErrorManager *error_manager)
+static MagickBooleanType JPEGSetImageProfiles(JPEGClientInfo *client_info)
 {
   ExceptionInfo
     *exception;
@@ -970,24 +971,21 @@ static MagickBooleanType JPEGSetImageProfiles(ErrorManager *error_manager)
   MagickBooleanType
     status;
 
-  register unsigned char
+  unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  size_t
-    length;
-
   StringInfo
     *profile;
 
-  exception=error_manager->exception;
-  image=error_manager->image;
+  exception=client_info->exception;
+  image=client_info->image;
   status=MagickTrue;
   for (i=0; i < 16; i++)
   {
-    profile=error_manager->profiles[i];
+    profile=client_info->profiles[i];
     if (profile == (StringInfo *) NULL)
       continue;
     switch (i)
@@ -1021,7 +1019,7 @@ static MagickBooleanType JPEGSetImageProfiles(ErrorManager *error_manager)
         break;
       }
     }
-    error_manager->profiles[i]=DestroyStringInfo(error_manager->profiles[i]);
+    client_info->profiles[i]=DestroyStringInfo(client_info->profiles[i]);
     if (status == MagickFalse)
       break;
   }
@@ -1031,6 +1029,13 @@ static MagickBooleanType JPEGSetImageProfiles(ErrorManager *error_manager)
 static Image *ReadJPEGImage_(const ImageInfo *image_info,
   struct jpeg_decompress_struct *jpeg_info,ExceptionInfo *exception)
 {
+#define ThrowJPEGReaderException(exception,message) \
+{ \
+  if (client_info != (JPEGClientInfo *) NULL) \
+    client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info); \
+  ThrowReaderException((exception),(message)); \
+}
+
   char
     value[MagickPathExtent];
 
@@ -1038,12 +1043,12 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
     *dct_method,
     *option;
 
-  ErrorManager
-    error_manager;
-
   Image
     *image;
 
+  JPEGClientInfo
+    *client_info = (JPEGClientInfo *) NULL;
+
   JSAMPLE
     *volatile jpeg_pixels;
 
@@ -1051,7 +1056,6 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
     scanline[1];
 
   MagickBooleanType
-    debug,
     status;
 
   MagickSizeType
@@ -1063,7 +1067,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   Quantum
     index;
 
-  register ssize_t
+  ssize_t
     i;
 
   struct jpeg_error_mgr
@@ -1072,7 +1076,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   struct jpeg_progress_mgr
     jpeg_progress;
 
-  register JSAMPLE
+  JSAMPLE
     *p;
 
   size_t
@@ -1091,8 +1095,6 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  debug=IsEventLogging();
-  (void) debug;
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1104,11 +1106,14 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
     Verify that file size large enough to contain a JPEG datastream.
   */
   if (GetBlobSize(image) < 107)
-    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    ThrowJPEGReaderException(CorruptImageError,"InsufficientImageDataInFile");
   /*
     Initialize JPEG parameters.
   */
-  (void) memset(&error_manager,0,sizeof(error_manager));
+  client_info=(JPEGClientInfo *) AcquireMagickMemory(sizeof(*client_info));
+  if (client_info == (JPEGClientInfo *) NULL)
+    ThrowJPEGReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(client_info,0,sizeof(*client_info));
   (void) memset(jpeg_info,0,sizeof(*jpeg_info));
   (void) memset(&jpeg_error,0,sizeof(jpeg_error));
   (void) memset(&jpeg_progress,0,sizeof(jpeg_progress));
@@ -1116,18 +1121,18 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   jpeg_info->err->emit_message=(void (*)(j_common_ptr,int)) JPEGWarningHandler;
   jpeg_info->err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
   memory_info=(MemoryInfo *) NULL;
-  error_manager.exception=exception;
-  error_manager.image=image;
-  if (setjmp(error_manager.error_recovery) != 0)
+  client_info->exception=exception;
+  client_info->image=image;
+  if (setjmp(client_info->error_recovery) != 0)
     {
       JPEGDestroyDecompress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       (void) CloseBlob(image);
-      number_pixels=(MagickSizeType) image->columns*image->rows;
-      if (number_pixels != 0)
+      if (exception->severity < ErrorException)
         return(GetFirstImageInList(image));
       return(DestroyImage(image));
     }
-  jpeg_info->client_data=(void *) &error_manager;
+  jpeg_info->client_data=(void *) client_info;
   jpeg_create_decompress(jpeg_info);
   if (GetMaxMemoryRequest() != ~0UL)
     jpeg_info->mem->max_memory_to_use=(long) GetMaxMemoryRequest();
@@ -1143,13 +1148,12 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   if (IsOptionMember("APP",option) == MagickFalse)
     jpeg_set_marker_processor(jpeg_info,XML_MARKER,ReadXmlProfile);
   for (i=3; i < 16; i++)
-    if (i != IPTC_INDEX)
+    /* APP14 is ignored because this will change the colors of the image */
+    if (i != IPTC_INDEX && i != 14)
       if (IsOptionMember("APP",option) == MagickFalse)
         jpeg_set_marker_processor(jpeg_info,(int) (JPEG_APP0+i),ReadProfile);
   i=(ssize_t) jpeg_read_header(jpeg_info,TRUE);
-  if ((image_info->colorspace == YCbCrColorspace) ||
-      (image_info->colorspace == Rec601YCbCrColorspace) ||
-      (image_info->colorspace == Rec709YCbCrColorspace))
+  if (IsYCbCrCompatibleColorspace(image_info->colorspace) != MagickFalse)
     jpeg_info->out_color_space=JCS_YCbCr;
   /*
     Set image resolution.
@@ -1230,7 +1234,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
         Let the JPEG library quantize the image.
       */
       jpeg_info->quantize_colors=TRUE;
-      jpeg_info->desired_number_of_colors=(int) StringToUnsignedLong(option);
+      jpeg_info->desired_number_of_colors=StringToInteger(option);
     }
   option=GetImageOption(image_info,"jpeg:block-smoothing");
   if (option != (const char *) NULL)
@@ -1308,7 +1312,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
     if (AcquireImageColormap(image,StringToUnsignedLong(option),exception) == MagickFalse)
       {
         JPEGDestroyDecompress(jpeg_info);
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        ThrowJPEGReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
   if ((jpeg_info->output_components == 1) && (jpeg_info->quantize_colors == 0))
     {
@@ -1319,7 +1323,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
       if (AcquireImageColormap(image,colors,exception) == MagickFalse)
         {
           JPEGDestroyDecompress(jpeg_info);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          ThrowJPEGReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
   if (image->debug != MagickFalse)
@@ -1344,14 +1348,16 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   if (jpeg_info->arith_code == TRUE)
     (void) SetImageProperty(image,"jpeg:coding","arithmetic",exception);
 #endif
-  if (JPEGSetImageProfiles(&error_manager) == MagickFalse)
+  if (JPEGSetImageProfiles(client_info) == MagickFalse)
     {
       JPEGDestroyDecompress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       return(DestroyImageList(image));
     }
   if (image_info->ping != MagickFalse)
     {
       JPEGDestroyDecompress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
@@ -1359,6 +1365,7 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   if (status == MagickFalse)
     {
       JPEGDestroyDecompress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       return(DestroyImageList(image));
     }
   (void) jpeg_start_decompress(jpeg_info);
@@ -1366,14 +1373,14 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
       (jpeg_info->output_components != 3) && (jpeg_info->output_components != 4))
     {
       JPEGDestroyDecompress(jpeg_info);
-      ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
+      ThrowJPEGReaderException(CorruptImageError,"ImageTypeNotSupported");
     }
   memory_info=AcquireVirtualMemory((size_t) image->columns,
     jpeg_info->output_components*sizeof(*jpeg_pixels));
   if (memory_info == (MemoryInfo *) NULL)
     {
       JPEGDestroyDecompress(jpeg_info);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      ThrowJPEGReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   jpeg_pixels=(JSAMPLE *) GetVirtualMemoryBlob(memory_info);
   (void) memset(jpeg_pixels,0,image->columns*
@@ -1381,11 +1388,12 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   /*
     Convert JPEG pixels to pixel packets.
   */
-  if (setjmp(error_manager.error_recovery) != 0)
+  if (setjmp(client_info->error_recovery) != 0)
     {
       if (memory_info != (MemoryInfo *) NULL)
         memory_info=RelinquishVirtualMemory(memory_info);
       JPEGDestroyDecompress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       (void) CloseBlob(image);
       number_pixels=(MagickSizeType) image->columns*image->rows;
       if (number_pixels != 0)
@@ -1419,10 +1427,10 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   scanline[0]=(JSAMPROW) jpeg_pixels;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
     if (jpeg_read_scanlines(jpeg_info,scanline,1) != 1)
@@ -1536,14 +1544,15 @@ static Image *ReadJPEGImage_(const ImageInfo *image_info,
   }
   if (status != MagickFalse)
     {
-      error_manager.finished=MagickTrue;
-      if (setjmp(error_manager.error_recovery) == 0)
+      client_info->finished=MagickTrue;
+      if (setjmp(client_info->error_recovery) == 0)
         (void) jpeg_finish_decompress(jpeg_info);
     }
   /*
     Free jpeg resources.
   */
   JPEGDestroyDecompress(jpeg_info);
+  client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
   memory_info=RelinquishVirtualMemory(memory_info);
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -1753,16 +1762,16 @@ static QuantizationTable *DestroyQuantizationTable(QuantizationTable *table)
   return(table);
 }
 
-static boolean EmptyOutputBuffer(j_compress_ptr cinfo)
+static boolean EmptyOutputBuffer(j_compress_ptr compress_info)
 {
   DestinationManager
     *destination;
 
-  destination=(DestinationManager *) cinfo->dest;
+  destination=(DestinationManager *) compress_info->dest;
   destination->manager.free_in_buffer=(size_t) WriteBlob(destination->image,
     MagickMinBufferExtent,destination->buffer);
   if (destination->manager.free_in_buffer != MagickMinBufferExtent)
-    ERREXIT(cinfo,JERR_FILE_WRITE);
+    ERREXIT(compress_info,JERR_FILE_WRITE);
   destination->manager.next_output_byte=destination->buffer;
   return(TRUE);
 }
@@ -1781,7 +1790,7 @@ static QuantizationTable *GetQuantizationTable(const char *filename,
   double
     value;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1961,24 +1970,25 @@ static QuantizationTable *GetQuantizationTable(const char *filename,
   return(table);
 }
 
-static void InitializeDestination(j_compress_ptr cinfo)
+static void InitializeDestination(j_compress_ptr compress_info)
 {
   DestinationManager
     *destination;
 
-  destination=(DestinationManager *) cinfo->dest;
-  destination->buffer=(JOCTET *) (*cinfo->mem->alloc_small)
-    ((j_common_ptr) cinfo,JPOOL_IMAGE,MagickMinBufferExtent*sizeof(JOCTET));
+  destination=(DestinationManager *) compress_info->dest;
+  destination->buffer=(JOCTET *) (*compress_info->mem->alloc_small) (
+    (j_common_ptr) compress_info,JPOOL_IMAGE,MagickMinBufferExtent*
+    sizeof(JOCTET));
   destination->manager.next_output_byte=destination->buffer;
   destination->manager.free_in_buffer=MagickMinBufferExtent;
 }
 
-static void TerminateDestination(j_compress_ptr cinfo)
+static void TerminateDestination(j_compress_ptr compress_info)
 {
   DestinationManager
     *destination;
 
-  destination=(DestinationManager *) cinfo->dest;
+  destination=(DestinationManager *) compress_info->dest;
   if ((MagickMinBufferExtent-(int) destination->manager.free_in_buffer) > 0)
     {
       ssize_t
@@ -1988,11 +1998,11 @@ static void TerminateDestination(j_compress_ptr cinfo)
         destination->manager.free_in_buffer,destination->buffer);
       if (count != (ssize_t)
           (MagickMinBufferExtent-destination->manager.free_in_buffer))
-        ERREXIT(cinfo,JERR_FILE_WRITE);
+        ERREXIT(compress_info,JERR_FILE_WRITE);
     }
 }
 
-static void WriteProfile(j_compress_ptr jpeg_info,Image *image,
+static void WriteProfiles(j_compress_ptr jpeg_info,Image *image,
   ExceptionInfo *exception)
 {
   const char
@@ -2004,7 +2014,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image,
   MagickBooleanType
     iptc;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2049,7 +2059,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image,
       }
     if (LocaleCompare(name,"ICC") == 0)
       {
-        register unsigned char
+        unsigned char
           *p;
 
         tag_length=strlen(ICC_PROFILE);
@@ -2070,7 +2080,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image,
     if (((LocaleCompare(name,"IPTC") == 0) ||
         (LocaleCompare(name,"8BIM") == 0)) && (iptc == MagickFalse))
       {
-        register unsigned char
+        unsigned char
           *p;
 
         size_t
@@ -2130,14 +2140,15 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image,
   custom_profile=DestroyStringInfo(custom_profile);
 }
 
-static void JPEGDestinationManager(j_compress_ptr cinfo,Image * image)
+static void JPEGDestinationManager(j_compress_ptr compress_info,Image * image)
 {
   DestinationManager
     *destination;
 
-  cinfo->dest=(struct jpeg_destination_mgr *) (*cinfo->mem->alloc_small)
-    ((j_common_ptr) cinfo,JPOOL_IMAGE,sizeof(DestinationManager));
-  destination=(DestinationManager *) cinfo->dest;
+  compress_info->dest=(struct jpeg_destination_mgr *) (*
+    compress_info->mem->alloc_small) ((j_common_ptr) compress_info,JPOOL_IMAGE,
+    sizeof(DestinationManager));
+  destination=(DestinationManager *) compress_info->dest;
   destination->manager.init_destination=InitializeDestination;
   destination->manager.empty_output_buffer=EmptyOutputBuffer;
   destination->manager.term_destination=TerminateDestination;
@@ -2149,13 +2160,13 @@ static char **SamplingFactorToList(const char *text)
   char
     **textlist;
 
-  register char
+  char
     *q;
 
-  register const char
+  const char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   if (text == (char *) NULL)
@@ -2192,15 +2203,19 @@ static char **SamplingFactorToList(const char *text)
 static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
   Image *image,struct jpeg_compress_struct *jpeg_info,ExceptionInfo *exception)
 {
+#define ThrowJPEGWriterException(exception,message) \
+{ \
+  if (client_info != (JPEGClientInfo *) NULL) \
+    client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info); \
+  ThrowWriterException((exception),(message)); \
+}
+
   const char
     *dct_method,
     *option,
     *sampling_factor,
     *value;
 
-  ErrorManager
-    error_manager;
-
   Image
     *volatile volatile_image;
 
@@ -2208,6 +2223,9 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     colorspace,
     quality;
 
+  JPEGClientInfo
+    *client_info = (JPEGClientInfo *) NULL;
+
   JSAMPLE
     *volatile jpeg_pixels;
 
@@ -2220,10 +2238,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
   MemoryInfo
     *memory_info;
 
-  register JSAMPLE
+  JSAMPLE
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -2255,7 +2273,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
   /*
     Initialize JPEG parameters.
   */
-  (void) memset(&error_manager,0,sizeof(error_manager));
+  client_info=(JPEGClientInfo *) AcquireMagickMemory(sizeof(*client_info));
+  if (client_info == (JPEGClientInfo *) NULL)
+    ThrowJPEGWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(client_info,0,sizeof(*client_info));
   (void) memset(jpeg_info,0,sizeof(*jpeg_info));
   (void) memset(&jpeg_error,0,sizeof(jpeg_error));
   volatile_image=image;
@@ -2263,21 +2284,22 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
   jpeg_info->err=jpeg_std_error(&jpeg_error);
   jpeg_info->err->emit_message=(void (*)(j_common_ptr,int)) JPEGWarningHandler;
   jpeg_info->err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
-  error_manager.exception=exception;
-  error_manager.image=volatile_image;
+  client_info->exception=exception;
+  client_info->image=volatile_image;
   memory_info=(MemoryInfo *) NULL;
-  if (setjmp(error_manager.error_recovery) != 0)
+  if (setjmp(client_info->error_recovery) != 0)
     {
       jpeg_destroy_compress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       (void) CloseBlob(volatile_image);
       return(MagickFalse);
     }
-  jpeg_info->client_data=(void *) &error_manager;
+  jpeg_info->client_data=(void *) client_info;
   jpeg_create_compress(jpeg_info);
   JPEGDestinationManager(jpeg_info,image);
   if ((image->columns != (unsigned int) image->columns) ||
       (image->rows != (unsigned int) image->rows))
-    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+    ThrowJPEGWriterException(ImageError,"WidthOrHeightExceedsLimit");
   jpeg_info->image_width=(unsigned int) image->columns;
   jpeg_info->image_height=(unsigned int) image->rows;
   jpeg_info->input_components=3;
@@ -2309,10 +2331,11 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     }
     default:
     {
-      (void) TransformImageColorspace(image,sRGBColorspace,exception);
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+        (void) TransformImageColorspace(image,sRGBColorspace,exception);
       if (image_info->type == TrueColorType)
         break;
-      if (SetImageGray(image,exception) != MagickFalse)
+      if (IdentifyImageCoderGray(image,exception) != MagickFalse)
         {
           jpeg_info->input_components=1;
           jpeg_info->in_color_space=JCS_GRAYSCALE;
@@ -2673,30 +2696,9 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
           "Number of colors: unspecified");
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "JPEG data precision: %d",(int) jpeg_info->data_precision);
-      switch (image->colorspace)
+      switch (jpeg_info->in_color_space)
       {
-        case CMYKColorspace:
-        {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "Storage class: DirectClass");
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "Colorspace: CMYK");
-          break;
-        }
-        case YCbCrColorspace:
-        case Rec601YCbCrColorspace:
-        case Rec709YCbCrColorspace:
-        {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "Colorspace: YCbCr");
-          break;
-        }
-        default:
-          break;
-      }
-      switch (image->colorspace)
-      {
-        case CMYKColorspace:
+        case JCS_CMYK:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Colorspace: CMYK");
@@ -2712,7 +2714,7 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
             jpeg_info->comp_info[3].v_samp_factor);
           break;
         }
-        case GRAYColorspace:
+        case JCS_GRAYSCALE:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Colorspace: GRAY");
@@ -2721,11 +2723,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
             jpeg_info->comp_info[0].v_samp_factor);
           break;
         }
-        case sRGBColorspace:
-        case RGBColorspace:
+        case JCS_UNKNOWN:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "Image colorspace is RGB");
+            "Colorspace: RGB");
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Sampling factors: %dx%d,%dx%d,%dx%d",
             jpeg_info->comp_info[0].h_samp_factor,
@@ -2736,9 +2737,7 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
             jpeg_info->comp_info[2].v_samp_factor);
           break;
         }
-        case YCbCrColorspace:
-        case Rec601YCbCrColorspace:
-        case Rec709YCbCrColorspace:
+        case JCS_YCbCr:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Colorspace: YCbCr");
@@ -2752,22 +2751,7 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
             jpeg_info->comp_info[2].v_samp_factor);
           break;
         }
-        default:
-        {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: %d",
-            image->colorspace);
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "Sampling factors: %dx%d,%dx%d,%dx%d,%dx%d",
-            jpeg_info->comp_info[0].h_samp_factor,
-            jpeg_info->comp_info[0].v_samp_factor,
-            jpeg_info->comp_info[1].h_samp_factor,
-            jpeg_info->comp_info[1].v_samp_factor,
-            jpeg_info->comp_info[2].h_samp_factor,
-            jpeg_info->comp_info[2].v_samp_factor,
-            jpeg_info->comp_info[3].h_samp_factor,
-            jpeg_info->comp_info[3].v_samp_factor);
-          break;
-        }
+        default: break;
       }
     }
   /*
@@ -2785,18 +2769,19 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
           (unsigned int) MagickMin((size_t) strlen(value+i),65533L));
     }
   if (image->profiles != (void *) NULL)
-    WriteProfile(jpeg_info,image,exception);
+    WriteProfiles(jpeg_info,image,exception);
   /*
     Convert MIFF to JPEG raster pixels.
   */
   memory_info=AcquireVirtualMemory((size_t) image->columns,
     jpeg_info->input_components*sizeof(*jpeg_pixels));
   if (memory_info == (MemoryInfo *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    ThrowJPEGWriterException(ResourceLimitError,"MemoryAllocationFailed");
   jpeg_pixels=(JSAMPLE *) GetVirtualMemoryBlob(memory_info);
-  if (setjmp(error_manager.error_recovery) != 0)
+  if (setjmp(client_info->error_recovery) != 0)
     {
       jpeg_destroy_compress(jpeg_info);
+      client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
       if (memory_info != (MemoryInfo *) NULL)
         memory_info=RelinquishVirtualMemory(memory_info);
       (void) CloseBlob(image);
@@ -2813,10 +2798,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
           (jpeg_info->in_color_space == JCS_YCbCr))
         for (y=0; y < (ssize_t) image->rows; y++)
         {
-          register const Quantum
+          const Quantum
             *p;
 
-          register ssize_t
+          ssize_t
             x;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -2840,10 +2825,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
         if (jpeg_info->in_color_space == JCS_GRAYSCALE)
           for (y=0; y < (ssize_t) image->rows; y++)
           {
-            register const Quantum
+            const Quantum
               *p;
 
-            register ssize_t
+            ssize_t
               x;
 
             p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -2865,10 +2850,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
         else
           for (y=0; y < (ssize_t) image->rows; y++)
           {
-            register const Quantum
+            const Quantum
               *p;
 
-            register ssize_t
+            ssize_t
               x;
 
             p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -2901,10 +2886,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     if (jpeg_info->in_color_space == JCS_GRAYSCALE)
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register const Quantum
+        const Quantum
           *p;
 
-        register ssize_t
+        ssize_t
           x;
 
         p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -2928,10 +2913,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
           (jpeg_info->in_color_space == JCS_YCbCr))
         for (y=0; y < (ssize_t) image->rows; y++)
         {
-          register const Quantum
+          const Quantum
             *p;
 
-          register ssize_t
+          ssize_t
             x;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -2954,10 +2939,10 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
       else
         for (y=0; y < (ssize_t) image->rows; y++)
         {
-          register const Quantum
+          const Quantum
             *p;
 
-          register ssize_t
+          ssize_t
             x;
 
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
@@ -2991,6 +2976,7 @@ static MagickBooleanType WriteJPEGImage_(const ImageInfo *image_info,
     Relinquish resources.
   */
   jpeg_destroy_compress(jpeg_info);
+  client_info=(JPEGClientInfo *) RelinquishMagickMemory(client_info);
   memory_info=RelinquishVirtualMemory(memory_info);
   (void) CloseBlob(image);
   return(MagickTrue);
