@@ -10,14 +10,14 @@
 %                            M   M  P       CCCC                              %
 %                                                                             %
 %                                                                             %
-%              Read/Write Magick Persistant Cache Image Format                %
+%                 Read/Write Magick Pixel Cache Image Format                  %
 %                                                                             %
 %                              Software Design                                %
 %                                   Cristy                                    %
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -63,6 +63,7 @@
 #include "MagickCore/monitor.h"
 #include "MagickCore/monitor-private.h"
 #include "MagickCore/option.h"
+#include "MagickCore/pixel-private.h"
 #include "MagickCore/profile.h"
 #include "MagickCore/property.h"
 #include "MagickCore/quantum-private.h"
@@ -74,6 +75,11 @@
 #include "MagickCore/utility.h"
 #include "MagickCore/version-private.h"
 
+/*
+  Define declarations.
+*/
+#define MagickPixelCacheNonce  "MagickPixelCache"
+
 /*
   Forward declarations.
 */
@@ -92,7 +98,7 @@ static MagickBooleanType
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  IsMPC() returns MagickTrue if the image format type, identified by the
-%  magick string, is an Magick Persistent Cache image.
+%  magick string, is an Magick Pixel Cache image.
 %
 %  The format of the IsMPC method is:
 %
@@ -107,9 +113,9 @@ static MagickBooleanType
 */
 static MagickBooleanType IsMPC(const unsigned char *magick,const size_t length)
 {
-  if (length < 14)
+  if (length < 19)
     return(MagickFalse);
-  if (LocaleNCompare((const char *) magick,"id=MagickPixelCache",14) == 0)
+  if (LocaleNCompare((const char *) magick,"id=MagickPixelCache",19) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -125,7 +131,7 @@ static MagickBooleanType IsMPC(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadMPCImage() reads an Magick Persistent Cache image file and returns
+%  ReadMPCImage() reads an Magick Pixel Cache image file and returns
 %  it.  It allocates the memory necessary for the new Image structure and
 %  returns a pointer to the new image.
 %
@@ -150,9 +156,6 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     keyword[MagickPathExtent],
     *options;
 
-  const unsigned char
-    *p;
-
   GeometryInfo
     geometry_info;
 
@@ -174,7 +177,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickStatusType
     flags;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -185,6 +188,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ssize_t
     count;
 
+  StringInfo
+    *nonce;
+
   unsigned int
     signature;
 
@@ -225,12 +231,14 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     profiles=(LinkedListInfo *) NULL;
     length=MagickPathExtent;
     options=AcquireString((char *) NULL);
-    signature=GetMagickSignature((const StringInfo *) NULL);
+    nonce=StringToStringInfo(MagickPixelCacheNonce);
+    signature=GetMagickSignature(nonce);
+    nonce=DestroyStringInfo(nonce);
     image->depth=8;
     image->compression=NoCompression;
-    while ((isgraph(c) != MagickFalse) && (c != (int) ':'))
+    while ((isgraph((int) ((unsigned char) c)) != 0) && (c != (int) ':'))
     {
-      register char
+      char
         *p;
 
       if (c == (int) '{')
@@ -266,8 +274,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (comment == (char *) NULL)
             {
               options=DestroyString(options);
-              ThrowReaderException(ResourceLimitError,
-                "MemoryAllocationFailed");
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
           *p='\0';
           (void) SetImageProperty(image,"comment",comment,exception);
@@ -275,7 +282,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           c=ReadBlobByte(image);
         }
       else
-        if (isalnum(c) != MagickFalse)
+        if (isalnum((int) ((unsigned char) c)) != MagickFalse)
           {
             /*
               Get the keyword.
@@ -292,7 +299,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             } while (c != EOF);
             *p='\0';
             p=options;
-            while (isspace(c) != 0)
+            while (isspace((int) ((unsigned char) c)) != 0)
               c=ReadBlobByte(image);
             if (c == (int) '=')
               {
@@ -324,7 +331,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         }
                     }
                   if (*options != '{')
-                    if (isspace(c) != 0)
+                    if (isspace((int) ((unsigned char) c)) != 0)
                       break;
                 }
                 if (options == (char *) NULL)
@@ -571,15 +578,6 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (LocaleCompare(keyword,"number-meta-channels") == 0)
                   {
                     image->number_meta_channels=StringToUnsignedLong(options);
-                    if (image->number_meta_channels > MaxPixelChannels)
-                      {
-                        if (profiles != (LinkedListInfo *) NULL)
-                          profiles=DestroyLinkedList(profiles,
-                            RelinquishMagickMemory);
-                        options=DestroyString(options);
-                        ThrowReaderException(CorruptImageError,
-                          "ImproperImageHeader");
-                      }
                     break;
                   }
                 break;
@@ -775,7 +773,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         else
           c=ReadBlobByte(image);
-      while (isspace(c) != 0)
+      while (isspace((int) ((unsigned char) c)) != 0)
         c=ReadBlobByte(image);
     }
     options=DestroyString(options);
@@ -788,7 +786,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (image->compression == UndefinedCompression) ||
         (image->columns == 0) || (image->rows == 0) ||
         (image->number_channels > MaxPixelChannels) ||
-        (image->number_meta_channels > (MaxPixelChannels-8)) ||
+        (image->number_meta_channels > (MaxPixelChannels-StartMetaPixelChannel)) ||
         ((image->number_channels+image->number_meta_channels) >= MaxPixelChannels) ||
         (image->depth == 0) || (image->depth > 64))
       {
@@ -796,15 +794,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       }
-    if (signature != GetMagickSignature((const StringInfo *) NULL))
+    nonce=StringToStringInfo(MagickPixelCacheNonce);
+    if (signature != GetMagickSignature(nonce))
       {
+        nonce=DestroyStringInfo(nonce);
         if (profiles != (LinkedListInfo *) NULL)
           profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
         ThrowReaderException(CacheError,"IncompatibleAPI");
       }
+    nonce=DestroyStringInfo(nonce);
     if (image->montage != (char *) NULL)
       {
-        register char
+        char
           *p;
 
         /*
@@ -879,6 +880,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     depth=GetImageQuantumDepth(image,MagickFalse);
     if (image->storage_class == PseudoClass)
       {
+        const unsigned char
+          *p;
+
         size_t
           packet_size;
 
@@ -1006,8 +1010,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     do
     {
       c=ReadBlobByte(image);
-    } while ((isgraph(c) == MagickFalse) && (c != EOF));
-    if (c != EOF)
+    } while ((isgraph((int) ((unsigned char) c)) == 0) && (c != EOF));
+    if ((c != EOF) && ((c == 'i') || (c == 'I')))
       {
         /*
           Allocate next image structure.
@@ -1024,7 +1028,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (status == MagickFalse)
           break;
       }
-  } while (c != EOF);
+  } while ((c != EOF) && ((c == 'i') || (c == 'I')));
   (void) CloseBlob(image);
   if (status == MagickFalse)
     return(DestroyImageList(image));
@@ -1059,11 +1063,10 @@ ModuleExport size_t RegisterMPCImage(void)
   MagickInfo
     *entry;
 
-  entry=AcquireMagickInfo("MPC","CACHE",
-    "Magick Persistent Cache image format");
+  entry=AcquireMagickInfo("MPC","CACHE","Magick Pixel Cache image format");
   entry->flags|=CoderStealthFlag;
   (void) RegisterMagickInfo(entry);
-  entry=AcquireMagickInfo("MPC","MPC","Magick Persistent Cache image format");
+  entry=AcquireMagickInfo("MPC","MPC","Magick Pixel Cache image format");
   entry->decoder=(DecodeImageHandler *) ReadMPCImage;
   entry->encoder=(EncodeImageHandler *) WriteMPCImage;
   entry->magick=(IsImageFormatHandler *) IsMPC;
@@ -1108,7 +1111,7 @@ ModuleExport void UnregisterMPCImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  WriteMPCImage() writes an Magick Persistent Cache image to a file.
+%  WriteMPCImage() writes an Magick Pixel Cache image to a file.
 %
 %  The format of the WriteMPCImage method is:
 %
@@ -1142,7 +1145,7 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image,
     offset,
     scene;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1170,17 +1173,24 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image,
   imageListLength=GetImageListLength(image);
   do
   {
+    StringInfo
+      *nonce;
+
     /*
-      Write persistent cache meta-information.
+      Write cache meta-information.
+
+      SetImageStorageClass() required to sync pixel cache.
     */
-    depth=GetImageQuantumDepth(image,MagickTrue);
     (void) SetImageStorageClass(image,image->storage_class,exception);
+    depth=GetImageQuantumDepth(image,MagickTrue);
     if ((image->storage_class == PseudoClass) &&
         (image->colors > (size_t) (GetQuantumRange(image->depth)+1)))
       (void) SetImageStorageClass(image,DirectClass,exception);
     (void) WriteBlobString(image,"id=MagickPixelCache\n");
+    nonce=StringToStringInfo(MagickPixelCacheNonce);
     (void) FormatLocaleString(buffer,MagickPathExtent,"magick-signature=%u\n",
-      GetMagickSignature((const StringInfo *) NULL));
+      GetMagickSignature(nonce));
+    nonce=DestroyStringInfo(nonce);
     (void) WriteBlobString(image,buffer);
     (void) FormatLocaleString(buffer,MagickPathExtent,
       "class=%s  colors=%.20g  alpha-trait=%s\n",CommandOptionToMnemonic(
