@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,43 +39,43 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/property.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/client.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/constitute.h"
-#include "magick/decorate.h"
-#include "magick/distort.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/gem.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/montage.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/resize.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
-#include "magick/transform.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/property.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/client.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/decorate.h"
+#include "MagickCore/distort.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/gem.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/montage.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/resize.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/utility.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WritePCDImage(const ImageInfo *,Image *);
+  WritePCDImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -111,7 +111,7 @@ static MagickBooleanType
 %
 */
 static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
-  unsigned char *chroma1,unsigned char *chroma2)
+  unsigned char *chroma1,unsigned char *chroma2,ExceptionInfo *exception)
 {
 #define IsSync(sum)  ((sum & 0xffffff00UL) == 0xfffffe00UL)
 #define PCDGetBits(n) \
@@ -125,7 +125,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
         count=ReadBlob(image,0x800,buffer); \
         p=buffer; \
       } \
-    sum|=((unsigned int) (*p) << (24-bits)); \
+    sum|=(((unsigned int) (*p)) << (24-bits)); \
     bits+=8; \
     p++; \
   } \
@@ -147,14 +147,14 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   PCDTable
     *pcd_table[3];
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
-  register PCDTable
+  PCDTable
     *r;
 
-  register unsigned char
+  unsigned char
     *p,
     *q;
 
@@ -195,7 +195,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     pcd_table[i]=(PCDTable *) NULL;
     pcd_length[i]=0;
   }
-  for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
+  for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
   {
     PCDGetBits(8);
     length=(sum & 0xff)+1;
@@ -204,6 +204,8 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+        for (j=0; j < i; j++)
+          pcd_table[j]=(PCDTable *) RelinquishMagickMemory(pcd_table[j]);
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
       }
@@ -215,6 +217,8 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       if (r->length > 16)
         {
           buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+          for (j=0; j <= i; j++)
+            pcd_table[j]=(PCDTable *) RelinquishMagickMemory(pcd_table[j]);
           return(MagickFalse);
         }
       PCDGetBits(16);
@@ -226,17 +230,20 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     }
     pcd_length[i]=(size_t) length;
   }
-  /*
-    Search for Sync byte.
-  */
-  for (i=0; i < 1; i++)
-    PCDGetBits(16);
-  for (i=0; i < 1; i++)
-    PCDGetBits(16);
-  while ((sum & 0x00fff000UL) != 0x00fff000UL)
-    PCDGetBits(8);
-  while (IsSync(sum) == 0)
-    PCDGetBits(1);
+  if (EOFBlob(image) == MagickFalse)
+    {
+      /*
+        Search for Sync byte.
+      */
+      for (i=0; i < 1; i++)
+        PCDGetBits(16);
+      for (i=0; i < 1; i++)
+        PCDGetBits(16);
+      while ((sum & 0x00fff000UL) != 0x00fff000UL)
+        PCDGetBits(8);
+      while (IsSync(sum) == 0)
+        PCDGetBits(1);
+    }
   /*
     Recover the Huffman encoded luminance and chrominance deltas.
   */
@@ -244,8 +251,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   length=0;
   plane=0;
   row=0;
-  q=luma;
-  for ( ; ; )
+  for (q=luma; EOFBlob(image) == MagickFalse; )
   {
     if (IsSync(sum) != 0)
       {
@@ -283,6 +289,9 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
           default:
           {
+            for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
+              pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
+            buffer=(unsigned char *) RelinquishMagickMemory(buffer);
             ThrowBinaryException(CorruptImageError,"CorruptImage",
               image->filename);
           }
@@ -298,7 +307,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       r++;
     if ((row > image->rows) || (r == (PCDTable *) NULL))
       {
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        (void) ThrowMagickException(exception,GetMagickModule(),
           CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
         while ((sum & 0x00fff000) != 0x00fff000)
           PCDGetBits(8);
@@ -318,7 +327,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   /*
     Relinquish resources.
   */
-  for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
+  for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
     pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   return(MagickTrue);
@@ -385,6 +394,7 @@ static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
 static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
@@ -394,7 +404,7 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   MontageInfo
     *montage_info;
 
-  register Image
+  Image
     *p;
 
   /*
@@ -403,27 +413,27 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   for (p=image; p != (Image *) NULL; p=p->next)
   {
     (void) DeleteImageProperty(p,"label");
-    (void) SetImageProperty(p,"label",DefaultTileLabel);
+    (void) SetImageProperty(p,"label",DefaultTileLabel,exception);
   }
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
   (void) CopyMagickString(montage_info->filename,image_info->filename,
-    MaxTextExtent);
+    MagickPathExtent);
   montage_image=MontageImageList(image_info,montage_info,image,exception);
   montage_info=DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  image=DestroyImage(image);
+  image=DestroyImageList(image);
   return(montage_image);
 }
 
 static void Upsample(const size_t width,const size_t height,
   const size_t scaled_width,unsigned char *pixels)
 {
-  register ssize_t
+  ssize_t
     x,
     y;
 
-  register unsigned char
+  unsigned char
     *p,
     *q,
     *r;
@@ -465,11 +475,20 @@ static void Upsample(const size_t width,const size_t height,
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
-  (void) CopyMagickMemory(q,p,(size_t) (2*width));
+  (void) memcpy(q,p,(size_t) (2*width));
 }
 
 static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define ThrowPCDException(exception,message) \
+{ \
+  if (header != (unsigned char *) NULL) \
+    header=(unsigned char *) RelinquishMagickMemory(header); \
+  if (pixel_info != (MemoryInfo *) NULL) \
+    pixel_info=RelinquishVirtualMemory(pixel_info); \
+  ThrowReaderException((exception),(message)); \
+}
+
   Image
     *image;
 
@@ -479,17 +498,17 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickOffsetType
     offset;
 
-  MagickSizeType
-    number_pixels;
+  MemoryInfo
+    *pixel_info;
 
-  register ssize_t
+  ssize_t
     i,
     y;
 
-  register PixelPacket
+  Quantum
     *q;
 
-  register unsigned char
+  unsigned char
     *c1,
     *c2,
     *yy;
@@ -497,6 +516,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     height,
     number_images,
+    number_pixels,
     rotate,
     scene,
     width;
@@ -524,7 +544,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -537,19 +557,19 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   header=(unsigned char *) AcquireQuantumMemory(0x800,3UL*sizeof(*header));
   if (header == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  pixel_info=(MemoryInfo *) NULL;
   count=ReadBlob(image,3*0x800,header);
+  if (count != (3*0x800))
+    ThrowPCDException(CorruptImageError,"ImproperImageHeader");
   overview=LocaleNCompare((char *) header,"PCD_OPA",7) == 0;
-  if ((count != (3*0x800)) ||
-      ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && (overview == 0)))
-    {
-      header=(unsigned char *) RelinquishMagickMemory(header);
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    }
+  if ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && (overview == 0))
+    ThrowPCDException(CorruptImageError,"ImproperImageHeader");
   rotate=header[0x0e02] & 0x03;
-  number_images=(header[10] << 8) | header[11];
+  number_images=((header[10] << 8) | header[11]) & 0xffff;
   header=(unsigned char *) RelinquishMagickMemory(header);
-  if (number_images > 65535)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if ((overview != 0) &&
+      (AcquireMagickResource(ListLengthResource,number_images) == MagickFalse))
+    ThrowPCDException(ResourceLimitError,"ListLengthExceedsLimit");
   /*
     Determine resolution by scene specification.
   */
@@ -589,35 +609,24 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->columns<<=1;
     image->rows<<=1;
   }
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+    return(DestroyImageList(image));
+  status=ResetImagePixels(image,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   /*
     Allocate luma and chroma memory.
   */
-  number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (number_pixels != (size_t) number_pixels)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  chroma1=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    sizeof(*chroma1));
-  chroma2=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    sizeof(*chroma2));
-  luma=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
+  pixel_info=AcquireVirtualMemory(image->columns+1UL,30*image->rows*
     sizeof(*luma));
-  if ((chroma1 == (unsigned char *) NULL) ||
-      (chroma2 == (unsigned char *) NULL) || (luma == (unsigned char *) NULL))
-    {
-      if (chroma1 != (unsigned char *) NULL)
-        chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
-      if (chroma2 != (unsigned char *) NULL)
-        chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
-      if (luma != (unsigned char *) NULL)
-        luma=(unsigned char *) RelinquishMagickMemory(luma);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
+  if (pixel_info == (MemoryInfo *) NULL)
+    ThrowPCDException(ResourceLimitError,"MemoryAllocationFailed");
+  number_pixels=(image->columns+1UL)*10*image->rows*sizeof(*luma);
+  luma=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
+  chroma1=(unsigned char *) GetVirtualMemoryBlob(pixel_info)+number_pixels;
+  chroma2=(unsigned char *) GetVirtualMemoryBlob(pixel_info)+2*number_pixels;
+  (void) memset(luma,0,3*number_pixels);
   /*
     Advance to image data.
   */
@@ -631,16 +640,14 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (scene <= 1)
         offset=1;
   for (i=0; i < (ssize_t) (offset*0x800); i++)
-    (void) ReadBlobByte(image);
+    if (ReadBlobByte(image) == EOF)
+      ThrowPCDException(CorruptImageError,"UnexpectedEndOfFile");
   if (overview != 0)
     {
-      Image
-        *overview_image;
-
       MagickProgressMonitor
         progress_monitor;
 
-      register ssize_t
+      ssize_t
         j;
 
       /*
@@ -650,9 +657,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         progress_monitor=SetImageProgressMonitor(image,
           (MagickProgressMonitor) NULL,image->client_data);
-        (void) FormatLocaleString(image->filename,MaxTextExtent,
+        (void) FormatLocaleString(image->filename,MagickPathExtent,
           "images/img%04ld.pcd",(long) j);
-        (void) FormatLocaleString(image->magick_filename,MaxTextExtent,
+        (void) FormatLocaleString(image->magick_filename,MagickPathExtent,
           "images/img%04ld.pcd",(long) j);
         image->scene=(size_t) j;
         image->columns=width;
@@ -671,6 +678,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           c1+=image->columns;
           count=ReadBlob(image,width >> 1,c2);
           c2+=image->columns;
+          if (EOFBlob(image) != MagickFalse)
+            ThrowPCDException(CorruptImageError,"UnexpectedEndOfFile");
         }
         Upsample(image->columns >> 1,image->rows >> 1,image->columns,chroma1);
         Upsample(image->columns >> 1,image->rows >> 1,image->columns,chroma2);
@@ -683,31 +692,33 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (y=0; y < (ssize_t) image->rows; y++)
         {
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-          if (q == (PixelPacket *) NULL)
+          if (q == (Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
-            SetPixelRed(q,ScaleCharToQuantum(*yy++));
-            SetPixelGreen(q,ScaleCharToQuantum(*c1++));
-            SetPixelBlue(q,ScaleCharToQuantum(*c2++));
-            q++;
+            SetPixelRed(image,ScaleCharToQuantum(*yy++),q);
+            SetPixelGreen(image,ScaleCharToQuantum(*c1++),q);
+            SetPixelBlue(image,ScaleCharToQuantum(*c2++),q);
+            q+=GetPixelChannels(image);
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
         image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
-          (void) SetImageColorspace(image,sRGBColorspace);
+          (void) SetImageColorspace(image,sRGBColorspace,exception);
+        if (EOFBlob(image) != MagickFalse)
+          break;
         if (j < (ssize_t) number_images)
           {
             /*
               Allocate next image structure.
             */
-            AcquireNextImage(image_info,image);
+            AcquireNextImage(image_info,image,exception);
             if (GetNextImageInList(image) == (Image *) NULL)
               {
-                image=DestroyImageList(image);
-                return((Image *) NULL);
+                status=MagickFalse;
+                break;
               }
             image=SyncNextImageInList(image);
           }
@@ -720,12 +731,10 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
           }
       }
-      chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
-      chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
-      luma=(unsigned char *) RelinquishMagickMemory(luma);
-      image=GetFirstImageInList(image);
-      overview_image=OverviewImage(image_info,image,exception);
-      return(overview_image);
+      pixel_info=RelinquishVirtualMemory(pixel_info);
+      if (status == MagickFalse)
+        return(DestroyImageList(image));
+      return(OverviewImage(image_info,GetFirstImageInList(image),exception));
     }
   /*
     Read interleaved image.
@@ -743,6 +752,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     c1+=image->columns;
     count=ReadBlob(image,width >> 1,c2);
     c2+=image->columns;
+    if (EOFBlob(image) != MagickFalse)
+      ThrowPCDException(CorruptImageError,"UnexpectedEndOfFile");
   }
   if (scene >= 4)
     {
@@ -755,7 +766,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image->rows=1024;
       for (i=0; i < (4*0x800); i++)
         (void) ReadBlobByte(image);
-      status=DecodeImage(image,luma,chroma1,chroma2);
+      status=DecodeImage(image,luma,chroma1,chroma2,exception);
       if ((scene >= 5) && status)
         {
           /*
@@ -767,7 +778,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->rows=2048;
           offset=TellBlob(image)/0x800+12;
           offset=SeekBlob(image,offset*0x800,SEEK_SET);
-          status=DecodeImage(image,luma,chroma1,chroma2);
+          status=DecodeImage(image,luma,chroma1,chroma2,exception);
           if ((scene >= 6) && (status != MagickFalse))
             {
               /*
@@ -791,28 +802,26 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    if (q == (Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      SetPixelRed(q,ScaleCharToQuantum(*yy++));
-      SetPixelGreen(q,ScaleCharToQuantum(*c1++));
-      SetPixelBlue(q,ScaleCharToQuantum(*c2++));
-      q++;
+      SetPixelRed(image,ScaleCharToQuantum(*yy++),q);
+      SetPixelGreen(image,ScaleCharToQuantum(*c1++),q);
+      SetPixelBlue(image,ScaleCharToQuantum(*c2++),q);
+      q+=GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     if (image->previous == (Image *) NULL)
       {
         status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
-                image->rows);
+          image->rows);
         if (status == MagickFalse)
           break;
       }
   }
-  chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
-  chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
-  luma=(unsigned char *) RelinquishMagickMemory(luma);
+  pixel_info=RelinquishVirtualMemory(pixel_info);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
@@ -851,7 +860,10 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->gamma=1.000f/2.200f;
   image->colorspace=YCCColorspace;
   if (LocaleCompare(image_info->magick,"PCDS") == 0)
-    (void) SetImageColorspace(image,sRGBColorspace);
+    (void) SetImageColorspace(image,sRGBColorspace,exception);
+  if (image_info->scene != 0)
+    for (i=0; i < (ssize_t) image_info->scene; i++)
+      AppendImageToList(&image,CloneImage(image,0,0,MagickTrue,exception));
   return(GetFirstImageInList(image));
 }
 
@@ -883,22 +895,18 @@ ModuleExport size_t RegisterPCDImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("PCD");
+  entry=AcquireMagickInfo("PCD","PCD","Photo CD");
   entry->decoder=(DecodeImageHandler *) ReadPCDImage;
   entry->encoder=(EncodeImageHandler *) WritePCDImage;
   entry->magick=(IsImageFormatHandler *) IsPCD;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString("Photo CD");
-  entry->module=ConstantString("PCD");
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("PCDS");
+  entry=AcquireMagickInfo("PCD","PCDS","Photo CD");
   entry->decoder=(DecodeImageHandler *) ReadPCDImage;
   entry->encoder=(EncodeImageHandler *) WritePCDImage;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString("Photo CD");
-  entry->module=ConstantString("PCD");
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -943,7 +951,8 @@ ModuleExport void UnregisterPCDImage(void)
 %
 %  The format of the WritePCDImage method is:
 %
-%      MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePCDImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -951,10 +960,12 @@ ModuleExport void UnregisterPCDImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 
 static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
-  const char *tile_geometry)
+  const size_t tile_columns,const size_t tile_rows,ExceptionInfo *exception)
 {
   GeometryInfo
     geometry_info;
@@ -972,11 +983,11 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   RectangleInfo
     geometry;
 
-  register const PixelPacket
+  const Quantum
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -994,7 +1005,7 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   if ((geometry.height % 2) != 0)
     geometry.height--;
   tile_image=ResizeImage(image,geometry.width,geometry.height,TriangleFilter,
-    1.0,&image->exception);
+    exception);
   if (tile_image == (Image *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(page_geometry,&geometry_info);
@@ -1016,16 +1027,29 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
       */
       border_info.width=(geometry.width-tile_image->columns+1) >> 1;
       border_info.height=(geometry.height-tile_image->rows+1) >> 1;
-      bordered_image=BorderImage(tile_image,&border_info,&image->exception);
+      bordered_image=BorderImage(tile_image,&border_info,image->compose,
+        exception);
       if (bordered_image == (Image *) NULL)
         return(MagickFalse);
       tile_image=DestroyImage(tile_image);
       tile_image=bordered_image;
     }
-  (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
-  (void) TransformImageColorspace(tile_image,YCCColorspace);
+  if ((tile_image->columns != tile_columns) || (tile_image->rows != tile_rows))
+    {
+      Image
+        *resize_image;
+
+      resize_image=ResizeImage(tile_image,tile_columns,tile_rows,
+        tile_image->filter,exception);
+      if (resize_image != (Image *) NULL)
+        {
+          tile_image=DestroyImage(tile_image);
+          tile_image=resize_image;
+        }
+    }
+  (void) TransformImageColorspace(tile_image,YCCColorspace,exception);
   downsample_image=ResizeImage(tile_image,tile_image->columns/2,
-    tile_image->rows/2,TriangleFilter,1.0,&image->exception);
+    tile_image->rows/2,TriangleFilter,exception);
   if (downsample_image == (Image *) NULL)
     return(MagickFalse);
   /*
@@ -1033,34 +1057,36 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   */
   for (y=0; y < (ssize_t) tile_image->rows; y+=2)
   {
-    p=GetVirtualPixels(tile_image,0,y,tile_image->columns,2,
-      &tile_image->exception);
-    if (p == (const PixelPacket *) NULL)
+    p=GetVirtualPixels(tile_image,0,y,tile_image->columns,2,exception);
+    if (p == (const Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) (tile_image->columns << 1); x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
-      p++;
+      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(tile_image,p)));
+      p+=GetPixelChannels(tile_image);
     }
-    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,
-      1,&downsample_image->exception);
-    if (q == (const PixelPacket *) NULL)
+    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,1,
+      exception);
+    if (q == (Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(q)));
-      q++;
+      (void) WriteBlobByte(image,ScaleQuantumToChar(
+        GetPixelGreen(tile_image,q)));
+      q+=GetPixelChannels(tile_image);
     }
-    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,
-      1,&downsample_image->exception);
-    if (q == (const PixelPacket *) NULL)
+    q=GetVirtualPixels(downsample_image,0,y >> 1,downsample_image->columns,1,
+      exception);
+    if (q == (Quantum *) NULL)
       break;
     for (x=0; x < (ssize_t) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(q)));
-      q++;
+      (void) WriteBlobByte(image,ScaleQuantumToChar(
+        GetPixelBlue(tile_image,q)));
+      q+=GetPixelChannels(tile_image);
     }
-    status=SetImageProgress(image,SaveImageTag,y,tile_image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      tile_image->rows);
     if (status == MagickFalse)
       break;
   }
@@ -1071,7 +1097,8 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   return(MagickTrue);
 }
 
-static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   Image
     *pcd_image;
@@ -1079,7 +1106,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(image_info != (const ImageInfo *) NULL);
@@ -1097,23 +1124,25 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
       /*
         Rotate portrait to landscape.
       */
-      rotate_image=RotateImage(image,90.0,&image->exception);
+      rotate_image=RotateImage(image,90.0,exception);
       if (rotate_image == (Image *) NULL)
         return(MagickFalse);
       pcd_image=rotate_image;
+      DestroyBlob(rotate_image);
+      pcd_image->blob=ReferenceBlob(image->blob);
     }
   /*
     Open output image file.
   */
-  status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,&image->exception);
+  status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       if (pcd_image != image)
         pcd_image=DestroyImage(pcd_image);
       return(status);
     }
-  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-    (void) TransformImageColorspace(pcd_image,sRGBColorspace);
+  if (IssRGBCompatibleColorspace(pcd_image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(pcd_image,sRGBColorspace,exception);
   /*
     Write PCD image header.
   */
@@ -1150,9 +1179,9 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   /*
     Write PCD tiles.
   */
-  status=WritePCDTile(pcd_image,"768x512>","192x128");
-  status=WritePCDTile(pcd_image,"768x512>","384x256");
-  status=WritePCDTile(pcd_image,"768x512>","768x512");
+  status=WritePCDTile(pcd_image,"768x512>",192,128,exception);
+  status=WritePCDTile(pcd_image,"768x512>",384,256,exception);
+  status=WritePCDTile(pcd_image,"768x512>",768,512,exception);
   (void) CloseBlob(pcd_image);
   if (pcd_image != image)
     pcd_image=DestroyImage(pcd_image);
