@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -329,13 +329,13 @@ static const unsigned char *UnpackScanline(
   const unsigned char *magick_restrict pixels,const unsigned int bits_per_pixel,
   unsigned char *scanline,MagickSizeType *bytes_per_line)
 {
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   p=pixels;
@@ -402,13 +402,13 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   MagickSizeType
     number_pixels;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -586,13 +586,13 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 #define MaxCount  128
 #define MaxPackbitsRunlength  128
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -750,7 +750,6 @@ static MagickBooleanType IsPICT(const unsigned char *magick,const size_t length)
   return(MagickFalse);
 }
 
-#if !defined(macintosh)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -857,10 +856,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   Quantum
     index;
 
-  register Quantum
+  Quantum
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1007,6 +1006,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               break;
             image->columns=(size_t) (frame.right-frame.left);
             image->rows=(size_t) (frame.bottom-frame.top);
+            if (image_info->ping != MagickFalse)
+              break;
             status=SetImageExtent(image,image->columns,image->rows,exception);
             if (status != MagickFalse)
               status=ResetImagePixels(image,exception);
@@ -1141,11 +1142,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               source,
               destination;
 
-            register unsigned char
+            unsigned char
               *p;
 
             size_t
-              j;
+              k;
 
             ssize_t
               bytes_per_line;
@@ -1210,14 +1211,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     for (i=0; i < (ssize_t) tile_image->colors; i++)
                     {
-                      j=ReadBlobMSBShort(image) % tile_image->colors;
+                      k=ReadBlobMSBShort(image) % tile_image->colors;
                       if ((flags & 0x8000) != 0)
-                        j=(size_t) i;
-                      tile_image->colormap[j].red=(Quantum)
+                        k=(size_t) i;
+                      tile_image->colormap[k].red=(Quantum)
                         ScaleShortToQuantum(ReadBlobMSBShort(image));
-                      tile_image->colormap[j].green=(Quantum)
+                      tile_image->colormap[k].green=(Quantum)
                         ScaleShortToQuantum(ReadBlobMSBShort(image));
-                      tile_image->colormap[j].blue=(Quantum)
+                      tile_image->colormap[k].blue=(Quantum)
                         ScaleShortToQuantum(ReadBlobMSBShort(image));
                     }
                   }
@@ -1298,14 +1299,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     if (pixmap.bits_per_pixel == 16)
                       {
                         i=(ssize_t) (*p++);
-                        j=(size_t) (*p);
+                        k=(size_t) (*p);
                         SetPixelRed(tile_image,ScaleCharToQuantum(
                           (unsigned char) ((i & 0x7c) << 1)),q);
                         SetPixelGreen(tile_image,ScaleCharToQuantum(
                           (unsigned char) (((i & 0x03) << 6) |
-                          ((j & 0xe0) >> 2))),q);
+                          ((k & 0xe0) >> 2))),q);
                         SetPixelBlue(tile_image,ScaleCharToQuantum(
-                          (unsigned char) ((j & 0x1f) << 3)),q);
+                          (unsigned char) ((k & 0x1f) << 3)),q);
                       }
                     else
                       if (tile_image->alpha_trait == UndefinedPixelTrait)
@@ -1468,9 +1469,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             const void
               *stream;
 
-            ssize_t
-              count;
-
             unsigned char
               *pixels;
 
@@ -1493,7 +1491,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 ThrowPICTException(CorruptImageError,"ImproperImageHeader");
               }
             read_info=AcquireImageInfo();
-            (void) FormatLocaleString(read_info->filename,MaxTextExtent,
+            (void) FormatLocaleString(read_info->filename,MagickPathExtent,
               "jpeg:%s",image_info->filename);
             tile_image=BlobToImage(read_info,stream,count,exception);
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
@@ -1549,7 +1547,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
-#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1686,10 +1683,10 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     size_rectangle,
     source_rectangle;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1727,7 +1724,8 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  (void) TransformImageColorspace(image,sRGBColorspace,exception);
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(image,sRGBColorspace,exception);
   /*
     Initialize image info.
   */
@@ -2031,7 +2029,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         break;
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        scanline[x]=(unsigned char) GetPixelIndex(image,p);
+        scanline[x]=(unsigned char) ((ssize_t) GetPixelIndex(image,p));
         p+=GetPixelChannels(image);
       }
       count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
@@ -2054,7 +2052,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       }
     else
       {
-        register unsigned char
+        unsigned char
           *blue,
           *green,
           *opacity,
