@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -79,6 +79,7 @@
 #include "MagickCore/transform.h"
 #include "MagickCore/utility.h"
 #include "coders/bytebuffer-private.h"
+#include "coders/coders-private.h"
 #include "coders/ghostscript-private.h"
 
 /*
@@ -179,7 +180,7 @@ static inline int ProfileInteger(MagickByteBuffer *buffer,short int *hex_digits)
     l,
     value;
 
-  register ssize_t
+  ssize_t
     i;
 
   l=0;
@@ -202,8 +203,8 @@ static inline int ProfileInteger(MagickByteBuffer *buffer,short int *hex_digits)
   return(value);
 }
 
-static void ReadPSInfo(const ImageInfo *image_info,Image *image,
-  PSInfo *ps_info,ExceptionInfo *exception)
+static void ReadPSInfo(const ImageInfo *image_info,Image *image,PSInfo *ps_info,
+  ExceptionInfo *exception)
 {
 #define BeginDocument  "BeginDocument:"
 #define EndDocument  "EndDocument:"
@@ -230,15 +231,16 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
 
   MagickBooleanType
     new_line,
-    skip;
+    skip,
+    spot_color;
 
   MagickByteBuffer
     buffer;
 
-  register char
+  char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   SegmentInfo
@@ -340,7 +342,8 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
         i=0;
         for (c=ReadMagickByteBuffer(&buffer); c != EOF; c=ReadMagickByteBuffer(&buffer))
         {
-          if ((c == '\r') || (c == '\n') || ((i+1) == sizeof(version)))
+          if ((c == '\r') || (c == '\n') ||
+              ((i+1) == (ssize_t) sizeof(version)))
             break;
           version[i++]=(char) c;
         }
@@ -367,10 +370,32 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
       ps_info->cmyk=MagickTrue;
     if (CompareMagickByteBuffer(&buffer,CMYKProcessColor,strlen(CMYKProcessColor)) != MagickFalse)
       ps_info->cmyk=MagickTrue;
+    spot_color=MagickFalse;
     length=strlen(DocumentCustomColors);
-    if ((CompareMagickByteBuffer(&buffer,DocumentCustomColors,length) != MagickFalse) ||
-        (CompareMagickByteBuffer(&buffer,CMYKCustomColor,strlen(CMYKCustomColor)) != MagickFalse) ||
-        (CompareMagickByteBuffer(&buffer,SpotColor,strlen(SpotColor)) != MagickFalse))
+    if (CompareMagickByteBuffer(&buffer,DocumentCustomColors,length) != MagickFalse)
+      {
+        spot_color=MagickTrue;
+        SkipMagickByteBuffer(&buffer,length+1);
+      }
+    if (spot_color == MagickFalse)
+      {
+        length=strlen(CMYKCustomColor);
+        if (CompareMagickByteBuffer(&buffer,CMYKCustomColor,length) != MagickFalse)
+          {
+            spot_color=MagickTrue;
+            SkipMagickByteBuffer(&buffer,length+1);
+          }
+      }
+    if (spot_color == MagickFalse)
+      {
+        length=strlen(SpotColor);
+        if (CompareMagickByteBuffer(&buffer,SpotColor,length) != MagickFalse)
+          {
+            spot_color=MagickTrue;
+            SkipMagickByteBuffer(&buffer,length+1);
+          }
+      }
+    if (spot_color != MagickFalse)
       {
         char
           name[MagickPathExtent],
@@ -383,17 +408,15 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
         (void) FormatLocaleString(property,MagickPathExtent,
           "pdf:SpotColor-%.20g",(double) spotcolor++);
         i=0;
-        for (c=ReadMagickByteBuffer(&buffer); c != EOF; c=ReadMagickByteBuffer(&buffer))
+        for (c=PeekMagickByteBuffer(&buffer); c != EOF; c=PeekMagickByteBuffer(&buffer))
         {
-          if ((isspace(c) != 0) || ((i+1) == sizeof(name)))
+          if ((c == '\r') || (c == '\n') || ((i+1) == MagickPathExtent))
             break;
-          name[i++]=(char) c;
+          name[i++]=(char) ReadMagickByteBuffer(&buffer);
         }
         name[i]='\0';
         value=ConstantString(name);
-        (void) SubstituteString(&value,"(","");
-        (void) SubstituteString(&value,")","");
-        (void) StripString(value);
+        (void) StripMagickString(value);
         if (*value != '\0')
           (void) SetImageProperty(image,property,value,exception);
         value=DestroyString(value);
@@ -408,7 +431,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
         /*
           Read ICC profile.
         */
-        if (SkipMagickByteBufferUntil(&buffer,'\n') != MagickFalse)
+        if (SkipMagickByteBufferUntilNewline(&buffer) != MagickFalse)
           {
             ps_info->icc_profile=AcquireStringInfo(MagickPathExtent);
             datum=GetStringInfoDatum(ps_info->icc_profile);
@@ -428,12 +451,12 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
     if ((ps_info->photoshop_profile == (StringInfo *) NULL) &&
         (CompareMagickByteBuffer(&buffer,PhotoshopProfile,strlen(PhotoshopProfile)) != MagickFalse))
       {
-        unsigned char
-          *q;
-
         unsigned long
           extent;
 
+        unsigned char
+          *q;
+
         /*
           Read Photoshop profile.
         */
@@ -445,7 +468,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
         if ((MagickSizeType) extent > GetBlobSize(image))
           continue;
         length=(size_t) extent;
-        if (SkipMagickByteBufferUntil(&buffer,'\n') != MagickFalse)
+        if (SkipMagickByteBufferUntilNewline(&buffer) != MagickFalse)
           {
             ps_info->photoshop_profile=AcquireStringInfo(length+1U);
             q=GetStringInfoDatum(ps_info->photoshop_profile);
@@ -455,7 +478,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
               if (c == EOF)
                 break;
               *q++=(unsigned char) c;
-              extent-=MagickMin(extent,2);
+              extent-=MagickMin(extent,1);
             }
             SetStringInfoLength(ps_info->photoshop_profile,length);
           }
@@ -558,6 +581,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     file;
 
   MagickBooleanType
+    crop,
     fitPage,
     status;
 
@@ -574,7 +598,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   RectangleInfo
     page;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -616,18 +640,20 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((image->resolution.x == 0.0) || (image->resolution.y == 0.0))
     {
       flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-      image->resolution.x=geometry_info.rho;
-      image->resolution.y=geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        image->resolution.y=image->resolution.x;
+      if ((flags & RhoValue) != 0)
+        image->resolution.x=geometry_info.rho;
+      image->resolution.y=image->resolution.x;
+      if ((flags & SigmaValue) != 0)
+        image->resolution.y=geometry_info.sigma;
     }
   if (image_info->density != (char *) NULL)
     {
       flags=ParseGeometry(image_info->density,&geometry_info);
-      image->resolution.x=geometry_info.rho;
-      image->resolution.y=geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        image->resolution.y=image->resolution.x;
+      if ((flags & RhoValue) != 0)
+        image->resolution.x=geometry_info.rho;
+      image->resolution.y=image->resolution.x;
+      if ((flags & SigmaValue) != 0)
+        image->resolution.y=geometry_info.sigma;
     }
   (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
@@ -658,7 +684,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   fitPage=MagickFalse;
   option=GetImageOption(image_info,"eps:fit-page");
-  if (option != (char *) NULL)
+  if (option != (const char *) NULL)
     {
       char
         *page_geometry;
@@ -681,6 +707,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       page_geometry=DestroyString(page_geometry);
       fitPage=MagickTrue;
     }
+  crop=MagickFalse;
+  if (*image_info->magick == 'E')
+    {
+      option=GetImageOption(image_info,"eps:use-cropbox");
+      if ((option == (const char *) NULL) ||
+          (IsStringTrue(option) != MagickFalse))
+        crop=MagickTrue;
+    }
   if (IssRGBCompatibleColorspace(image_info->colorspace) != MagickFalse)
     info.cmyk=MagickFalse;
   /*
@@ -696,8 +730,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   (void) CopyMagickString(command,"/setpagedevice {pop} bind 1 index where {"
-    "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n"
-    "<</UseCIEColor true>>setpagedevice\n",MagickPathExtent);
+    "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n",
+    MagickPathExtent);
   count=write(file,command,(unsigned int) strlen(command));
   if (image_info->page == (char *) NULL)
     {
@@ -732,8 +766,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   options=AcquireString("");
   (void) FormatLocaleString(density,MagickPathExtent,"%gx%g",resolution.x,
     resolution.y);
-  (void) FormatLocaleString(options,MagickPathExtent,"-g%.20gx%.20g ",(double)
-    page.width,(double) page.height);
+  if (crop == MagickFalse)
+    {
+      if (image_info->ping != MagickFalse)
+        (void) FormatLocaleString(density,MagickPathExtent,"2.0x2.0");
+      else
+        (void) FormatLocaleString(options,MagickPathExtent,"-g%.20gx%.20g ",
+          (double) page.width,(double) page.height);
+    }
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   if (read_info->number_scenes != 0)
@@ -751,9 +791,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   if (*image_info->magick == 'E')
     {
-      option=GetImageOption(image_info,"eps:use-cropbox");
-      if ((option == (const char *) NULL) ||
-          (IsStringTrue(option) != MagickFalse))
+      if (crop != MagickFalse)
         (void) ConcatenateMagickString(options,"-dEPSCrop ",MagickPathExtent);
       if (fitPage != MagickFalse)
         (void) ConcatenateMagickString(options,"-dEPSFitPage ",
@@ -865,6 +903,13 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (info.rows != 0)
       postscript_image->magick_rows=info.rows;
     postscript_image->page=page;
+    if (image_info->ping != MagickFalse)
+      {
+        postscript_image->magick_columns=page.width;
+        postscript_image->magick_rows=page.height;
+        postscript_image->columns=page.width;
+        postscript_image->rows=page.height;
+      }
     (void) CloneImageProfiles(postscript_image,image);
     (void) CloneImageProperties(postscript_image,image);
     next=SyncNextImageInList(postscript_image);
@@ -1020,7 +1065,7 @@ ModuleExport void UnregisterPSImage(void)
 static inline unsigned char *PopHexPixel(const char hex_digits[][3],
   const size_t pixel,unsigned char *pixels)
 {
-  register const char
+  const char
     *hex;
 
   hex=hex_digits[pixel];
@@ -1328,7 +1373,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
 
   char
     buffer[MagickPathExtent],
-    date[MagickPathExtent],
+    date[MagickTimeExtent],
     **labels,
     page_geometry[MagickPathExtent];
 
@@ -1338,6 +1383,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
   const char
     *value;
 
+  const Quantum
+    *p;
+
   const StringInfo
     *profile;
 
@@ -1372,14 +1420,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
     media_info,
     page_info;
 
-  register const Quantum
-    *p;
-
-  register ssize_t
-    i,
-    x;
-
-  register unsigned char
+  unsigned char
     *q;
 
   SegmentInfo
@@ -1394,7 +1435,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
     text_size;
 
   ssize_t
+    i,
     j,
+    x,
     y;
 
   time_t
@@ -1426,10 +1469,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
   imageListLength=GetImageListLength(image);
   do
   {
+    ImageType
+      type = UndefinedType;
+
     /*
       Scale relative to dots-per-inch.
     */
-    (void) TransformImageColorspace(image,sRGBColorspace,exception);
+    if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+      (void) TransformImageColorspace(image,sRGBColorspace,exception);
     delta.x=DefaultResolution;
     delta.y=DefaultResolution;
     resolution.x=image->resolution.x;
@@ -1437,18 +1484,20 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
     if ((resolution.x == 0.0) || (resolution.y == 0.0))
       {
         flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-        resolution.x=geometry_info.rho;
-        resolution.y=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          resolution.y=resolution.x;
+        if ((flags & RhoValue) != 0)
+          resolution.x=geometry_info.rho;
+        resolution.y=resolution.x;
+        if ((flags & SigmaValue) != 0)
+          resolution.y=geometry_info.sigma;
       }
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
-        resolution.x=geometry_info.rho;
-        resolution.y=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          resolution.y=resolution.x;
+        if ((flags & RhoValue) != 0)
+          resolution.x=geometry_info.rho;
+        resolution.y=resolution.x;
+        if ((flags & SigmaValue) != 0)
+          resolution.y=geometry_info.sigma;
       }
     if (image->units == PixelsPerCentimeterResolution)
       {
@@ -1507,7 +1556,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
           image->filename);
         (void) WriteBlobString(image,buffer);
         timer=GetMagickTime();
-        (void) FormatMagickTime(timer,MagickPathExtent,date);
+        (void) FormatMagickTime(timer,sizeof(date),date);
         (void) FormatLocaleString(buffer,MagickPathExtent,
           "%%%%CreationDate: (%s)\n",date);
         (void) WriteBlobString(image,buffer);
@@ -1550,22 +1599,6 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image,
             (void) WriteBlobString(image,"\n%EndPhotoshop\n");
           }
         profile=GetImageProfile(image,"xmp");
-DisableMSCWarning(4127)
-        if (0 && (profile != (StringInfo *) NULL))
-RestoreMSCWarning
-          {
-            /*
-              Embed XML profile.
-            */
-            (void) WriteBlobString(image,"\n%begin_xml_code\n");
-            (void) FormatLocaleString(buffer,MagickPathExtent,
-               "\n%%begin_xml_packet: %.20g\n",(double)
-               GetStringInfoLength(profile));
-            (void) WriteBlobString(image,buffer);
-            for (i=0; i < (ssize_t) GetStringInfoLength(profile); i++)
-              (void) WriteBlobByte(image,GetStringInfoDatum(profile)[i]);
-            (void) WriteBlobString(image,"\n%end_xml_packet\n%end_xml_code\n");
-          }
         value=GetImageProperty(image,"label",exception);
         if (value != (const char *) NULL)
           (void) WriteBlobString(image,
@@ -1596,15 +1629,6 @@ RestoreMSCWarning
             Image
               *preview_image;
 
-            Quantum
-              pixel;
-
-            register ssize_t
-              x;
-
-            ssize_t
-              y;
-
             /*
               Create preview image.
             */
@@ -1631,9 +1655,12 @@ RestoreMSCWarning
               byte=0;
               for (x=0; x < (ssize_t) preview_image->columns; x++)
               {
+                Quantum
+                  luma;
+
                 byte<<=1;
-                pixel=ClampToQuantum(GetPixelLuma(preview_image,p));
-                if (pixel >= (Quantum) (QuantumRange/2))
+                luma=ClampToQuantum(GetPixelLuma(preview_image,p));
+                if (luma >= (Quantum) (QuantumRange/2))
                   byte|=0x01;
                 bit++;
                 if (bit == 8)
@@ -1723,8 +1750,9 @@ RestoreMSCWarning
     /*
       Output image data.
     */
-    (void) FormatLocaleString(buffer,MagickPathExtent,"%.20g %.20g\n%g %g\n%g\n",
-      (double) geometry.x,(double) geometry.y,scale.x,scale.y,pointsize);
+    (void) FormatLocaleString(buffer,MagickPathExtent,
+      "%.20g %.20g\n%g %g\n%g\n",(double) geometry.x,(double) geometry.y,
+      scale.x,scale.y,pointsize);
     (void) WriteBlobString(image,buffer);
     labels=(char **) NULL;
     value=GetImageProperty(image,"label",exception);
@@ -1745,14 +1773,12 @@ RestoreMSCWarning
     pixel.alpha=(MagickRealType) TransparentAlpha;
     index=(Quantum) 0;
     x=0;
-    if ((image_info->type != TrueColorType) &&
-        (SetImageGray(image,exception) != MagickFalse))
+    if (image_info->type != TrueColorType)
+      type=IdentifyImageCoderGrayType(image,exception);
+    if (IsGrayImageType(type) != MagickFalse)
       {
-        if (SetImageMonochrome(image,exception) == MagickFalse)
+        if (type != BilevelType)
           {
-            Quantum
-              pixel;
-
             /*
               Dump image as grayscale.
             */
@@ -1768,9 +1794,12 @@ RestoreMSCWarning
                 break;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                pixel=(Quantum) ScaleQuantumToChar(ClampToQuantum(GetPixelLuma(
+                Quantum
+                  luma;
+
+                luma=(Quantum) ScaleQuantumToChar(ClampToQuantum(GetPixelLuma(
                   image,p)));
-                q=PopHexPixel(hex_digits,(size_t) pixel,q);
+                q=PopHexPixel(hex_digits,(size_t) luma,q);
                 if ((q-pixels+8) >= 80)
                   {
                     *q++='\n';
@@ -1795,12 +1824,6 @@ RestoreMSCWarning
           }
         else
           {
-            ssize_t
-              y;
-
-            Quantum
-              pixel;
-
             /*
               Dump image as bitmap.
             */
@@ -1818,9 +1841,12 @@ RestoreMSCWarning
               byte=0;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
+                Quantum
+                  luma;
+
                 byte<<=1;
-                pixel=ClampToQuantum(GetPixelLuma(image,p));
-                if (pixel >= (Quantum) (QuantumRange/2))
+                luma=ClampToQuantum(GetPixelLuma(image,p));
+                if (luma >= (Quantum) (QuantumRange/2))
                   byte|=0x01;
                 bit++;
                 if (bit == 8)
