@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,33 +39,34 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/artifact.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/color-private.h"
-#include "magick/colormap.h"
-#include "magick/colormap-private.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colormap.h"
+#include "MagickCore/colormap-private.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "coders/coders-private.h"
 
 /*
   Enumerated declaractions.
@@ -114,7 +115,7 @@ typedef struct _TGAInfo
   Forward declarations.
 */
 static MagickBooleanType
-  WriteTGAImage(const ImageInfo *,Image *);
+  WriteTGAImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -147,34 +148,30 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  IndexPacket
-    index;
-
   MagickBooleanType
     status;
 
-  PixelPacket
+  PixelInfo
     pixel;
 
-  register IndexPacket
-    *indexes;
+  Quantum
+    index;
 
-  register PixelPacket
+  Quantum
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
   size_t
     base,
     flag,
-    offset,
-    real,
     skip;
 
   ssize_t
     count,
+    offset,
     y;
 
   TGAInfo
@@ -199,7 +196,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -246,8 +243,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (tga_info.image_type != TGARLEMonochrome))
     {
       alpha_bits=(tga_info.attributes & 0x0FU);
-      image->matte=(alpha_bits > 0) || (tga_info.bits_per_pixel == 32) ||
-        (tga_info.colormap_size == 32) ?  MagickTrue : MagickFalse;
+      image->alpha_trait=(alpha_bits > 0) || (tga_info.bits_per_pixel == 32) ||
+        (tga_info.colormap_size == 32) ?  BlendPixelTrait : UndefinedPixelTrait;
     }
   if ((tga_info.image_type != TGAColormap) &&
       (tga_info.image_type != TGARLEColormap))
@@ -261,10 +258,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->storage_class=PseudoClass;
   if ((tga_info.image_type == TGAMonochrome) ||
       (tga_info.image_type == TGARLEMonochrome))
-    {
-      image->type=GrayscaleType;
-      image->colorspace=GRAYColorspace;
-    }
+    (void) SetImageColorspace(image,GRAYColorspace,exception);
   image->compression=NoCompression;
   if ((tga_info.image_type == TGARLEColormap) ||
       (tga_info.image_type == TGARLEMonochrome) ||
@@ -284,7 +278,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if ((MagickSizeType) image->colors > GetBlobSize(image))
             ThrowReaderException(CorruptImageError,
               "InsufficientImageDataInFile");
-          if (AcquireImageColormap(image,image->colors) == MagickFalse)
+          if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
@@ -301,8 +295,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       length=(size_t) tga_info.id_length;
       comment=(char *) NULL;
-      if (~length >= (MaxTextExtent-1))
-        comment=(char *) AcquireQuantumMemory(length+MaxTextExtent,
+      if (~length >= (MagickPathExtent-1))
+        comment=(char *) AcquireQuantumMemory(length+MagickPathExtent,
           sizeof(*comment));
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -310,37 +304,35 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (count == (ssize_t) length)
         {
           comment[length]='\0';
-          (void) SetImageProperty(image,"comment",comment);
+          (void) SetImageProperty(image,"comment",comment,exception);
         }
       comment=DestroyString(comment);
     }
-  if (tga_info.attributes & (1UL << 4))
+  image->orientation=BottomLeftOrientation;
+  if ((tga_info.attributes & (1UL << 4)) == 0)
     {
-      if (tga_info.attributes & (1UL << 5))
-        image->orientation=TopRightOrientation;
+      if ((tga_info.attributes & (1UL << 5)) == 0)
+        image->orientation=BottomLeftOrientation;
       else
-        image->orientation=BottomRightOrientation;
+        image->orientation=TopLeftOrientation;
     }
   else
     {
-      if (tga_info.attributes & (1UL << 5))
-        image->orientation=TopLeftOrientation;
+      if ((tga_info.attributes & (1UL << 5)) == 0)
+        image->orientation=BottomRightOrientation;
       else
-        image->orientation=BottomLeftOrientation;
+        image->orientation=TopRightOrientation;
     }
   if (image_info->ping != MagickFalse)
     {
       (void) CloseBlob(image);
       return(image);
     }
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+    return(DestroyImageList(image));
   (void) memset(&pixel,0,sizeof(pixel));
-  pixel.opacity=(Quantum) OpaqueOpacity;
+  pixel.alpha=(MagickRealType) OpaqueAlpha;
   if (tga_info.colormap_type != 0)
     {
       /*
@@ -348,7 +340,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       if (image->colors < tga_info.colormap_index)
         image->colors=tga_info.colormap_index;
-      if (AcquireImageColormap(image,image->colors) == MagickFalse)
+      if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (ssize_t) tga_info.colormap_index; i++)
         image->colormap[i]=pixel;
@@ -362,7 +354,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Gray scale.
             */
-            pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+            pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
             pixel.green=pixel.red;
             pixel.blue=pixel.red;
             break;
@@ -379,10 +372,11 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             j=(unsigned char) ReadBlobByte(image);
             k=(unsigned char) ReadBlobByte(image);
             range=GetQuantumRange(5UL);
-            pixel.red=ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,range);
-            pixel.green=ScaleAnyToQuantum((1UL*(k & 0x03) << 3)+
-              (1UL*(j & 0xe0) >> 5),range);
-            pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
+            pixel.red=(MagickRealType) ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,
+              range);
+            pixel.green=(MagickRealType) ScaleAnyToQuantum((1UL*(k & 0x03)
+              << 3)+(1UL*(j & 0xe0) >> 5),range);
+            pixel.blue=(MagickRealType) ScaleAnyToQuantum(1UL*(j & 0x1f),range);
             break;
           }
           case 24:
@@ -390,9 +384,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               8 bits each of blue, green and red.
             */
-            pixel.blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+            pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
             break;
           }
           case 32:
@@ -400,11 +397,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               8 bits each of blue, green, red, and alpha.
             */
-            pixel.blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
-              (unsigned char) ReadBlobByte(image)));
+            pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.alpha=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
             break;
           }
         }
@@ -417,19 +417,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   base=0;
   flag=0;
   skip=MagickFalse;
-  real=0;
-  index=(IndexPacket) 0;
+  index=0;
   runlength=0;
   offset=0;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    real=offset;
-    if (((unsigned char) (tga_info.attributes & 0x20) >> 5) == 0)
-      real=image->rows-real-1;
-    q=QueueAuthenticPixels(image,0,(ssize_t) real,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    q=QueueAuthenticPixels(image,0,offset,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
       break;
-    indexes=GetAuthenticIndexQueue(image);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
       if ((tga_info.image_type == TGARLEColormap) ||
@@ -466,12 +461,15 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             index=(Quantum) pixels[0];
             if (tga_info.colormap_type != 0)
               pixel=image->colormap[(ssize_t) ConstrainColormapIndex(image,
-                (ssize_t) index)];
+                (ssize_t) index,exception)];
             else
               {
-                pixel.red=ScaleCharToQuantum((unsigned char) index);
-                pixel.green=ScaleCharToQuantum((unsigned char) index);
-                pixel.blue=ScaleCharToQuantum((unsigned char) index);
+                pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+                  index);
+                pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+                  index);
+                pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+                  index);
               }
             break;
           }
@@ -482,22 +480,24 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               range;
 
             /*
-              5 bits each of RGB;
+              5 bits each of RGB.
             */
             if (ReadBlob(image,2,pixels) != 2)
               ThrowReaderException(CorruptImageError,"UnableToReadImageData");
             j=pixels[0];
             k=pixels[1];
             range=GetQuantumRange(5UL);
-            pixel.red=ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,range);
-            pixel.green=ScaleAnyToQuantum((1UL*(k & 0x03) << 3)+
-              (1UL*(j & 0xe0) >> 5),range);
-            pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
-            if (image->matte != MagickFalse)
-              pixel.opacity=(k & 0x80) == 0 ? (Quantum) TransparentOpacity :
-                (Quantum) OpaqueOpacity;
+            pixel.red=(MagickRealType) ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,
+              range);
+            pixel.green=(MagickRealType) ScaleAnyToQuantum((1UL*
+              (k & 0x03) << 3)+(1UL*(j & 0xe0) >> 5),range);
+            pixel.blue=(MagickRealType) ScaleAnyToQuantum(1UL*(j & 0x1f),range);
+            if (image->alpha_trait != UndefinedPixelTrait)
+              pixel.alpha=(MagickRealType) ((k & 0x80) == 0 ? (Quantum)
+                TransparentAlpha : (Quantum) OpaqueAlpha);
             if (image->storage_class == PseudoClass)
-              index=ConstrainColormapIndex(image,(ssize_t) ((size_t) k << 8)+j);
+              index=(Quantum) ConstrainColormapIndex(image,((ssize_t) k << 8)+
+                j,exception);
             break;
           }
           case 24:
@@ -507,9 +507,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             */
             if (ReadBlob(image,3,pixels) != 3)
               ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-            pixel.blue=ScaleCharToQuantum(pixels[0]);
-            pixel.green=ScaleCharToQuantum(pixels[1]);
-            pixel.red=ScaleCharToQuantum(pixels[2]);
+            pixel.blue=(MagickRealType) ScaleCharToQuantum(pixels[0]);
+            pixel.green=(MagickRealType) ScaleCharToQuantum(pixels[1]);
+            pixel.red=(MagickRealType) ScaleCharToQuantum(pixels[2]);
             break;
           }
           case 32:
@@ -519,35 +519,29 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             */
             if (ReadBlob(image,4,pixels) != 4)
               ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-            pixel.blue=ScaleCharToQuantum(pixels[0]);
-            pixel.green=ScaleCharToQuantum(pixels[1]);
-            pixel.red=ScaleCharToQuantum(pixels[2]);
-            pixel.opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
-              pixels[3]));
+            pixel.blue=(MagickRealType) ScaleCharToQuantum(pixels[0]);
+            pixel.green=(MagickRealType) ScaleCharToQuantum(pixels[1]);
+            pixel.red=(MagickRealType) ScaleCharToQuantum(pixels[2]);
+            pixel.alpha=(MagickRealType) ScaleCharToQuantum(pixels[3]);
             break;
           }
         }
       if (status == MagickFalse)
         ThrowReaderException(CorruptImageError,"UnableToReadImageData");
       if (image->storage_class == PseudoClass)
-        SetPixelIndex(indexes+x,index);
-      SetPixelRed(q,pixel.red);
-      SetPixelGreen(q,pixel.green);
-      SetPixelBlue(q,pixel.blue);
-      if (image->matte != MagickFalse)
-        SetPixelOpacity(q,pixel.opacity);
-      q++;
+        SetPixelIndex(image,index,q);
+      SetPixelRed(image,ClampToQuantum(pixel.red),q);
+      SetPixelGreen(image,ClampToQuantum(pixel.green),q);
+      SetPixelBlue(image,ClampToQuantum(pixel.blue),q);
+      if (image->alpha_trait != UndefinedPixelTrait)
+        SetPixelAlpha(image,ClampToQuantum(pixel.alpha),q);
+      q+=GetPixelChannels(image);
     }
-    /*
-      if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 4)
-        offset+=4;
-      else
-    */
-      if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 2)
-        offset+=2;
-      else
-        offset++;
-    if (offset >= image->rows)
+    if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 2)
+      offset+=2;
+    else
+      offset++;
+    if (offset >= (ssize_t) image->rows)
       {
         base++;
         offset=base;
@@ -597,37 +591,29 @@ ModuleExport size_t RegisterTGAImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("ICB");
+  entry=AcquireMagickInfo("TGA","ICB","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("TGA");
+  entry=AcquireMagickInfo("TGA","TGA","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("VDA");
+  entry=AcquireMagickInfo("TGA","VDA","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("VST");
+  entry=AcquireMagickInfo("TGA","VST","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
-  entry->seekable_stream=MagickTrue;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -675,7 +661,8 @@ ModuleExport void UnregisterTGAImage(void)
 %
 %  The format of the WriteTGAImage method is:
 %
-%      MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -685,11 +672,10 @@ ModuleExport void UnregisterTGAImage(void)
 %
 */
 static inline void WriteTGAPixel(Image *image,TGAImageType image_type,
-  const IndexPacket *indexes,const PixelPacket *p,const QuantumAny range,
-  const double midpoint)
+  const Quantum *p,const QuantumAny range,const double midpoint)
 {
   if (image_type == TGAColormap || image_type == TGARLEColormap)
-    (void) WriteBlobByte(image,(unsigned char) GetPixelIndex(indexes));
+    (void) WriteBlobByte(image,(unsigned char) ((ssize_t) GetPixelIndex(image,p)));
   else
     {
       if (image_type == TGAMonochrome || image_type == TGARLEMonochrome)
@@ -702,67 +688,74 @@ static inline void WriteTGAPixel(Image *image,TGAImageType image_type,
               green,
               value;
 
-              green=(unsigned char) ScaleQuantumToAny(GetPixelGreen(p),range);
-              value=((unsigned char) ScaleQuantumToAny(GetPixelBlue(p),range)) |
-                ((green & 0x07) << 5);
-              (void) WriteBlobByte(image,value);
-              value=(unsigned char) ((((image->matte != MagickFalse) && 
-                ((double) GetPixelOpacity(p) < midpoint)) ? 0x80 : 0) |
-                ((unsigned char) ScaleQuantumToAny(GetPixelRed(p),range) << 2) |
-                ((green & 0x18) >> 3));
-              (void) WriteBlobByte(image,value);
+            green=(unsigned char) ScaleQuantumToAny(GetPixelGreen(image,p),
+              range);
+            value=((unsigned char) ScaleQuantumToAny(GetPixelBlue(image,p),
+              range)) | ((green & 0x07) << 5);
+            (void) WriteBlobByte(image,value);
+            value=(((image->alpha_trait != UndefinedPixelTrait) &&
+              ((double) GetPixelAlpha(image,p) > midpoint)) ? 0x80 : 0) |
+              ((unsigned char) ScaleQuantumToAny(GetPixelRed(image,p),range) <<
+              2) | ((green & 0x18) >> 3);
+            (void) WriteBlobByte(image,value);
           }
         else
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(p)));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(p)));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
-            if (image->matte != MagickFalse)
-              (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelAlpha(p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              GetPixelBlue(image,p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              GetPixelGreen(image,p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              GetPixelRed(image,p)));
+            if (image->alpha_trait != UndefinedPixelTrait)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelAlpha(image,p)));
           }
     }
 }
 
-static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   CompressionType
     compression;
 
   const char
-    *comment,
-    *value;
+    *comment;
 
   const double
     midpoint = QuantumRange/2.0;
 
+  const Quantum
+    *p;
+
   MagickBooleanType
     status;
 
   QuantumAny
     range;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register ssize_t
+  ssize_t
     x;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
-    *q;
+  size_t
+    channels;
 
   ssize_t
+    base,
     count,
+    offset,
     y;
 
   TGAInfo
     tga_info;
 
+  unsigned char
+    *q;
+
   /*
     Open output image file.
   */
@@ -772,7 +765,9 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   /*
@@ -780,13 +775,14 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   */
   if ((image->columns > 65535L) || (image->rows > 65535L))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
-  (void) TransformImageColorspace(image,sRGBColorspace);
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(image,sRGBColorspace,exception);
   compression=image->compression;
   if (image_info->compression != UndefinedCompression)
     compression=image_info->compression;
   range=GetQuantumRange(5UL);
   tga_info.id_length=0;
-  comment=GetImageProperty(image,"comment");
+  comment=GetImageProperty(image,"comment",exception);
   if (comment != (const char *) NULL)
     tga_info.id_length=(unsigned char) MagickMin(strlen(comment),255);
   tga_info.colormap_type=0;
@@ -800,10 +796,10 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   tga_info.bits_per_pixel=8;
   tga_info.attributes=0;
   if ((image_info->type != TrueColorType) &&
-      (image_info->type != TrueColorMatteType) &&
+      (image_info->type != TrueColorAlphaType) &&
       (image_info->type != PaletteType) &&
-      (image->matte == MagickFalse) &&
-      (SetImageGray(image,&image->exception) != MagickFalse))
+      (image->alpha_trait == UndefinedPixelTrait) &&
+      (IdentifyImageCoderGray(image,exception) != MagickFalse))
     tga_info.image_type=compression == RLECompression ? TGARLEMonochrome :
       TGAMonochrome;
   else
@@ -816,13 +812,13 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         if (image_info->depth == 5)
           {
             tga_info.bits_per_pixel=16;
-            if (image->matte != MagickFalse)
+            if (image->alpha_trait != UndefinedPixelTrait)
               tga_info.attributes=1;  /* # of alpha bits */
           }
         else
           {
             tga_info.bits_per_pixel=24;
-            if (image->matte != MagickFalse)
+            if (image->alpha_trait != UndefinedPixelTrait)
               {
                 tga_info.bits_per_pixel=32;
                 tga_info.attributes=8;  /* # of alpha bits */
@@ -847,21 +843,9 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       (image->orientation == TopRightOrientation))
     tga_info.attributes|=(1UL << 4);
   if ((image->orientation == TopLeftOrientation) ||
+      (image->orientation == UndefinedOrientation) ||
       (image->orientation == TopRightOrientation))
     tga_info.attributes|=(1UL << 5);
-  value=GetImageArtifact(image,"tga:image-origin");  /* deprecated */
-  if (value != (const char *) NULL)
-    {
-      OrientationType
-        origin;
-
-      origin=(OrientationType) ParseCommandOption(MagickOrientationOptions,
-        MagickFalse,value);
-      if (origin == BottomRightOrientation || origin == TopRightOrientation)
-        tga_info.attributes|=(1UL << 4);
-      if (origin == TopLeftOrientation || origin == TopRightOrientation)
-        tga_info.attributes|=(1UL << 5);
-    }
   if ((image->columns > 65535) || (image->rows > 65535))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   /*
@@ -891,8 +875,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         Dump colormap to file (blue, green, red byte order).
       */
       targa_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
-        tga_info.colormap_length,(tga_info.colormap_size/8)*sizeof(
-        *targa_colormap));
+        tga_info.colormap_length,(tga_info.colormap_size/8)*
+        sizeof(*targa_colormap));
       if (targa_colormap == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       q=targa_colormap;
@@ -900,20 +884,20 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       {
         if (image_info->depth == 5)
           {
-            green=(unsigned char) ScaleQuantumToAny(image->colormap[i].green,
-              range);
-            *q++=((unsigned char) ScaleQuantumToAny(image->colormap[i].blue,
-              range)) | ((green & 0x07) << 5);
-            *q++=(((image->matte != MagickFalse) && (
-              (double) image->colormap[i].opacity < midpoint)) ? 0x80 : 0) |
-              ((unsigned char) ScaleQuantumToAny(image->colormap[i].red,
-              range) << 2) | ((green & 0x18) >> 3);
+            green=(unsigned char) ScaleQuantumToAny(ClampToQuantum(
+              image->colormap[i].green),range);
+            *q++=((unsigned char) ScaleQuantumToAny(ClampToQuantum(
+              image->colormap[i].blue),range)) | ((green & 0x07) << 5);
+            *q++=(((image->alpha_trait != UndefinedPixelTrait) && ((double)
+              ClampToQuantum(image->colormap[i].alpha) > midpoint)) ? 0x80 : 0) |
+              ((unsigned char) ScaleQuantumToAny(ClampToQuantum(
+              image->colormap[i].red),range) << 2) | ((green & 0x18) >> 3);
           }
         else
           {
-            *q++=ScaleQuantumToChar(image->colormap[i].blue);
-            *q++=ScaleQuantumToChar(image->colormap[i].green);
-            *q++=ScaleQuantumToChar(image->colormap[i].red);
+            *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].blue));
+            *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].green));
+            *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].red));
           }
       }
       (void) WriteBlob(image,(size_t) ((tga_info.colormap_size/8)*
@@ -923,12 +907,14 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   /*
     Convert MIFF to TGA raster pixels.
   */
-  for (y=(ssize_t) (image->rows-1); y >= 0; y--)
+  base=0;
+  offset=0;
+  channels=GetPixelChannels(image);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,offset,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-    indexes=GetVirtualIndexQueue(image);
     if (compression == RLECompression)
       {
         x=0;
@@ -941,23 +927,29 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           {
             if (tga_info.image_type == TGARLEColormap)
               {
-                if (GetPixelIndex(indexes+i) != GetPixelIndex(indexes+(i-1)))
+                if (GetPixelIndex(image,p+(i*channels)) !=
+                    GetPixelIndex(image,p+((i-1)*channels)))
                   break;
               }
             else
               if (tga_info.image_type == TGARLEMonochrome)
                 {
-                  if (GetPixelLuma(image,p+i) != GetPixelLuma(image,p+(i-1)))
+                  if (GetPixelLuma(image,p+(i*channels)) !=
+                      GetPixelLuma(image,p+((i-1)*channels)))
                     break;
                 }
               else
                 {
-                  if ((GetPixelBlue(p+i) != GetPixelBlue(p+(i-1))) ||
-                      (GetPixelGreen(p+i) != GetPixelGreen(p+(i-1))) ||
-                      (GetPixelRed(p+i) != GetPixelRed(p+(i-1))))
+                  if ((GetPixelBlue(image,p+(i*channels)) !=
+                       GetPixelBlue(image,p+((i-1)*channels))) ||
+                      (GetPixelGreen(image,p+(i*channels)) !=
+                       GetPixelGreen(image,p+((i-1)*channels))) ||
+                      (GetPixelRed(image,p+(i*channels)) !=
+                       GetPixelRed(image,p+((i-1)*channels))))
                     break;
-                  if ((image->matte != MagickFalse) &&
-                      (GetPixelAlpha(p+i) != GetPixelAlpha(p+(i-1))))
+                  if ((image->alpha_trait != UndefinedPixelTrait) &&
+                      (GetPixelAlpha(image,p+(i*channels)) !=
+                       GetPixelAlpha(image,p+(i-1)*channels)))
                     break;
                 }
             i++;
@@ -965,8 +957,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           if (i < 3)
             {
               count+=i;
-              p+=i;
-              indexes+=i;
+              p+=(i*channels);
             }
           if ((i >= 3) || (count == 128) ||
               ((x + i) == (ssize_t) image->columns))
@@ -976,8 +967,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
                   (void) WriteBlobByte(image,(unsigned char) (--count));
                   while (count >= 0)
                   {
-                    WriteTGAPixel(image,tga_info.image_type,indexes-(count+1),
-                      p-(count+1),range,midpoint);
+                    WriteTGAPixel(image,tga_info.image_type,p-((count+1)*
+                      channels),range,midpoint);
                     count--;
                   }
                   count=0;
@@ -986,18 +977,27 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           if (i >= 3)
             {
               (void) WriteBlobByte(image,(unsigned char) ((i-1) | 0x80));
-              WriteTGAPixel(image,tga_info.image_type,indexes,p,range,midpoint);
-              p+=i;
-              indexes+=i;
+              WriteTGAPixel(image,tga_info.image_type,p,range,midpoint);
+              p+=(i*channels);
             }
           x+=i;
         }
       }
     else
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        for (x=0; x < (ssize_t) image->columns; x++)
-          WriteTGAPixel(image,tga_info.image_type,indexes+x,p++,range,midpoint);
+        WriteTGAPixel(image,tga_info.image_type,p,range,midpoint);
+        p+=channels;
       }
+     if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 2)
+       offset+=2;
+     else
+       offset++;
+      if (offset >= (ssize_t) image->rows)
+        {
+          base++;
+          offset=base;
+        }
     if (image->previous == (Image *) NULL)
       {
         status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
