@@ -18,7 +18,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -36,29 +36,27 @@
 %
 %
 */
-
-
+
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/constitute.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/pixel-accessor.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/resource_.h"
-#include "magick/string_.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/utility.h"
 #if defined(MAGICKCORE_XML_DELEGATE)
 #  if defined(MAGICKCORE_WINDOWS_SUPPORT)
 #    if !defined(__MINGW32__)
@@ -132,16 +130,16 @@ static void GetFTPData(void *userdata,const char *data,int size)
 
 static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-#define MaxBufferExtent  8192
-
   char
-    filename[MaxTextExtent];
+    filename[MagickPathExtent];
 
   FILE
     *file;
 
   Image
-    *image;
+    *image,
+    *images,
+    *next;
 
   ImageInfo
     *read_info;
@@ -149,16 +147,47 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     unique_file;
 
+  images=(Image *) NULL;
+  image=AcquireImage(image_info,exception);
   read_info=CloneImageInfo(image_info);
   SetImageInfoBlob(read_info,(void *) NULL,0);
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__MINGW32__)
+  if (LocaleCompare(read_info->magick,"https") == 0)
+    {
+      MagickBooleanType
+        status;
+
+      /*
+        Leverage delegate to read HTTPS link.
+      */
+      status=InvokeDelegate(read_info,image,"https:decode",(char *) NULL,
+        exception);
+      if (status != MagickFalse)
+        {
+          (void) FormatLocaleString(read_info->filename,MagickPathExtent,
+            "%s.dat",read_info->unique);
+          *read_info->magick='\0';
+          images=ReadImage(read_info,exception);
+          (void) RelinquishUniqueFileResource(read_info->filename);
+          if (images != (Image *) NULL)
+            for (next=images; next != (Image *) NULL; next=next->next)
+              (void) CopyMagickString(next->filename,image->filename,
+                MagickPathExtent);
+        }
+      read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
+      return(images);
+    }
+#endif
   if (LocaleCompare(read_info->magick,"file") == 0)
     {
       (void) CopyMagickString(read_info->filename,image_info->filename+2,
-        MaxTextExtent);
+        MagickPathExtent);
       *read_info->magick='\0';
-      image=ReadImage(read_info,exception);
+      images=ReadImage(read_info,exception);
       read_info=DestroyImageInfo(read_info);
-      return(GetFirstImageInList(image));
+      image=DestroyImage(image);
+      return(GetFirstImageInList(images));
     }
   file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(read_info->filename);
@@ -169,14 +198,15 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
         read_info->filename);
       read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
-  (void) CopyMagickString(filename,image_info->magick,MaxTextExtent);
-  (void) ConcatenateMagickString(filename,":",MaxTextExtent);
+  (void) CopyMagickString(filename,image_info->magick,MagickPathExtent);
+  (void) ConcatenateMagickString(filename,":",MagickPathExtent);
   LocaleLower(filename);
-  (void) ConcatenateMagickString(filename,image_info->filename,MaxTextExtent);
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)
+  (void) ConcatenateMagickString(filename,image_info->filename,
+    MagickPathExtent);
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
   (void) fclose(file);
   if (URLDownloadToFile(NULL,filename,read_info->filename,0,NULL) != S_OK)
     {
@@ -184,6 +214,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         filename);
       (void) RelinquishUniqueFileResource(read_info->filename);
       read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
 #else
@@ -208,7 +239,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (LocaleCompare(read_info->magick,"http") == 0)
     {
       char
-        buffer[MaxBufferExtent],
+        buffer[8192],
         *type;
 
       int
@@ -225,7 +256,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
           ssize_t
             count;
 
-          while ((bytes=xmlNanoHTTPRead(context,buffer,MaxBufferExtent)) > 0)
+          while ((bytes=xmlNanoHTTPRead(context,buffer,sizeof(buffer))) > 0)
             count=(ssize_t) fwrite(buffer,bytes,1,file);
           (void) count;
           xmlNanoHTTPClose(context);
@@ -237,18 +268,22 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) fclose(file);
 #endif
   *read_info->magick='\0';
-  image=ReadImage(read_info,exception);
+  images=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  if (images != (Image *) NULL)
+    for (next=images; next != (Image *) NULL; next=next->next)
+      (void) CopyMagickString(next->filename,image->filename,MagickPathExtent);
   read_info=DestroyImageInfo(read_info);
-  if (image != (Image *) NULL)
-    GetPathComponent(image_info->filename,TailPath,image->filename);
+  image=DestroyImage(image);
+  if (images != (Image *) NULL)
+    GetPathComponent(image_info->filename,TailPath,images->filename);
   else
     {
       (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
         "NoDataReturned","`%s'",filename);
       return((Image *) NULL);
     }
-  return(GetFirstImageInList(image));
+  return(GetFirstImageInList(images));
 }
 
 /*
@@ -279,45 +314,33 @@ ModuleExport size_t RegisterURLImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("HTTP");
+  entry=AcquireMagickInfo("URL","HTTP","Uniform Resource Locator (http://)");
 #if (defined(MAGICKCORE_WINDOWS_SUPPORT) && \
     !defined(__MINGW32__)) || \
     (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
-  entry->description=ConstantString("Uniform Resource Locator (http://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("HTTPS");
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)
+  entry=AcquireMagickInfo("URL","HTTPS","Uniform Resource Locator (https://)");
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
-#endif
-  entry->description=ConstantString("Uniform Resource Locator (https://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("FTP");
+  entry=AcquireMagickInfo("URL","FTP","Uniform Resource Locator (ftp://)");
 #if (defined(MAGICKCORE_WINDOWS_SUPPORT) && \
     !defined(__MINGW32__)) || \
     (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
-  entry->description=ConstantString("Uniform Resource Locator (ftp://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("FILE");
+  entry=AcquireMagickInfo("URL","FILE","Uniform Resource Locator (file://)");
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
-  entry->description=ConstantString("Uniform Resource Locator (file://)");
-  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -340,7 +363,6 @@ ModuleExport size_t RegisterURLImage(void)
 ModuleExport void UnregisterURLImage(void)
 {
   (void) UnregisterMagickInfo("HTTP");
-  (void) UnregisterMagickInfo("HTTPS");
   (void) UnregisterMagickInfo("FTP");
   (void) UnregisterMagickInfo("FILE");
 }
