@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -165,7 +165,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register Quantum
+  Quantum
     *q;
 
   size_t
@@ -208,16 +208,16 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   length=0;
   image->columns=0;
   image->rows=0;
-  while (isgraph(c) && ((image->columns == 0) || (image->rows == 0)))
+  while (isgraph((int) ((unsigned char) c)) && ((image->columns == 0) || (image->rows == 0)))
   {
-    if (isalnum(c) == MagickFalse)
+    if (isalnum((int) ((unsigned char) c)) == MagickFalse)
       {
         c=ReadBlobByte(image);
         count++;
       }
     else
       {
-        register char
+        char
           *p;
 
         /*
@@ -230,10 +230,10 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
             *p++=c;
           c=ReadBlobByte(image);
           count++;
-        } while (isalnum(c) || (c == '_'));
+        } while (isalnum((int) ((unsigned char) c)) || (c == '_'));
         *p='\0';
         value_expected=MagickFalse;
-        while ((isspace(c) != 0) || (c == '='))
+        while ((isspace((int) ((unsigned char) c)) != 0) || (c == '='))
         {
           if (c == '=')
             value_expected=MagickTrue;
@@ -243,7 +243,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         if (value_expected == MagickFalse)
           continue;
         p=value;
-        while (isalnum(c))
+        while (isalnum((int) ((unsigned char) c)))
         {
           if ((size_t) (p-value) < (MagickPathExtent-1))
             *p++=c;
@@ -254,12 +254,15 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         /*
           Assign a value to the specified keyword.
         */
-        if (LocaleCompare(keyword,"Label_RECORDS") == 0)
-          length=(ssize_t) StringToLong(value);
+        if (LocaleCompare(keyword,"LABEL_RECORDS") == 0)
+          length*=(ssize_t) StringToLong(value);
         if (LocaleCompare(keyword,"LBLSIZE") == 0)
           length=(ssize_t) StringToLong(value);
         if (LocaleCompare(keyword,"RECORD_BYTES") == 0)
-          image->columns=StringToUnsignedLong(value);
+          {
+            image->columns=StringToUnsignedLong(value);
+            length=(ssize_t) image->columns;
+          }
         if (LocaleCompare(keyword,"NS") == 0)
           image->columns=StringToUnsignedLong(value);
         if (LocaleCompare(keyword,"LINES") == 0)
@@ -267,7 +270,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         if (LocaleCompare(keyword,"NL") == 0)
           image->rows=StringToUnsignedLong(value);
       }
-    while (isspace(c) != 0)
+    while (isspace((int) ((unsigned char) c)) != 0)
     {
       c=ReadBlobByte(image);
       count++;
@@ -439,7 +442,7 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   QuantumInfo
     *quantum_info;
 
-  register const Quantum
+  const Quantum
     *p;
 
   size_t
@@ -465,7 +468,8 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  (void) TransformImageColorspace(image,sRGBColorspace,exception);
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+    (void) TransformImageColorspace(image,sRGBColorspace,exception);
   /*
     Write header.
   */
@@ -473,8 +477,8 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   (void) FormatLocaleString(header,MagickPathExtent,
     "LBLSIZE=%.20g FORMAT='BYTE' TYPE='IMAGE' BUFSIZE=20000 DIM=2 EOL=0 "
     "RECSIZE=%.20g ORG='BSQ' NL=%.20g NS=%.20g NB=1 N1=0 N2=0 N3=0 N4=0 NBB=0 "
-    "NLB=0 TASK='ImageMagick'",(double) MagickPathExtent,(double) image->columns,
-    (double) image->rows,(double) image->columns);
+    "NLB=0 TASK='ImageMagick'",(double) MagickPathExtent,(double)
+    image->columns,(double) image->rows,(double) image->columns);
   (void) WriteBlob(image,MagickPathExtent,(unsigned char *) header);
   /*
     Write VICAR pixels.
