@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "MagickCore/log.h"
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
+#include "MagickCore/option.h"
 #include "MagickCore/resource_.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
@@ -62,6 +63,16 @@
 #include "MagickCore/utility.h"
 #include "MagickCore/utility-private.h"
 
+/*
+  Global declarations.
+*/
+static const char*
+  intermediate_formats[] =
+  {
+    "pam",
+    "webp"
+  };
+
 /*
   Forward declarations.
 */
@@ -150,10 +161,26 @@ static MagickBooleanType IsVIDEO(const unsigned char *magick,
 %    o exception: return any errors or warnings in this structure.
 %
 */
+static const char *GetIntermediateFormat(const ImageInfo *image_info)
+{
+  const char
+    *option;
+ 
+  option=GetImageOption(image_info,"video:intermediate-format");
+  if (LocaleCompare(option,"pam") == 0)
+    return(intermediate_formats[0]);
+#if defined(MAGICKCORE_WEBP_DELEGATE)
+  return(intermediate_formats[1]);
+#else
+  return(intermediate_formats[0]);
+#endif
+}
+
 static Image *ReadVIDEOImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define ReadVIDEOIntermediateFormat "pam"
+  const DelegateInfo
+    *delegate_info;
 
   Image
     *image,
@@ -190,25 +217,81 @@ static Image *ReadVIDEOImage(const ImageInfo *image_info,
   */
   images=(Image *) NULL;
   read_info=CloneImageInfo(image_info);
-  image=AcquireImage(image_info,exception);
-  status=InvokeDelegate(read_info,image,"video:decode",(char *) NULL,exception);
-  if (status != MagickFalse)
+  delegate_info=GetDelegateInfo("video:decode",(char *) NULL,exception);
+  if (delegate_info != (const DelegateInfo *) NULL)
     {
-      (void) FormatLocaleString(read_info->filename,MagickPathExtent,"%s.%s",
-        read_info->unique,ReadVIDEOIntermediateFormat);
-      *read_info->magick='\0';
-      images=ReadImage(read_info,exception);
+      char
+        command[MagickPathExtent],
+        message[MagickPathExtent];
+
+      char
+        *options,
+        *sanitized_option;
+
+      const char
+        *intermediate_format,
+        *option;
+
+      int
+        exit_code;
+
+      options=AcquireString("");
+      if (image_info->number_scenes > 0)
+        (void) FormatLocaleString(options,MagickPathExtent,"-vframes %i",
+          (int) image_info->number_scenes);
+      option=GetImageOption(image_info,"video:vsync");
+      if (option != (const char *) NULL)
+        {
+          sanitized_option=SanitizeDelegateString(option);
+          (void) FormatLocaleString(command,MagickPathExtent," -vsync %s",
+            sanitized_option);
+          DestroyString(sanitized_option);
+          (void) ConcatenateMagickString(options,command,MagickPathExtent);
+        }
+      option=GetImageOption(image_info,"video:pixel-format");
+      if (option != (const char *) NULL)
+        {
+          sanitized_option=SanitizeDelegateString(option);
+          (void) FormatLocaleString(command,MagickPathExtent," -pix_fmt %s",
+            sanitized_option);
+          DestroyString(sanitized_option);
+          (void) ConcatenateMagickString(options,command,MagickPathExtent);
+        }
+      else
+        if (LocaleNCompare(image_info->magick,"APNG",MagickPathExtent) == 0)
+          (void) ConcatenateMagickString(options," -pix_fmt rgba",
+            MagickPathExtent);
+      intermediate_format=GetIntermediateFormat(image_info);
+      (void) FormatLocaleString(command,MagickPathExtent," -vcodec %s",
+        intermediate_format);
+      (void) ConcatenateMagickString(options,command,MagickPathExtent);
+      AcquireUniqueFilename(read_info->unique);
+      (void) FormatLocaleString(command,MagickPathExtent,
+        GetDelegateCommands(delegate_info),read_info->filename,options,
+        read_info->unique);
+      options=DestroyString(options);
+      (void) CopyMagickString(read_info->magick,intermediate_format,
+        MagickPathExtent);
+      (void) CopyMagickString(read_info->filename,read_info->unique,
+        MagickPathExtent);
+      exit_code=ExternalDelegateCommand(MagickFalse,image_info->verbose,
+        command,message,exception);
+      if (exit_code == 0)
+        images=ReadImage(read_info,exception);
+      else if (*message != '\0')
+        (void) ThrowMagickException(exception,GetMagickModule(),
+          DelegateError,"VideoDelegateFailed","`%s'",message);
+      (void) RelinquishUniqueFileResource(read_info->unique);
       if (images != (Image *) NULL)
         for (next=images; next != (Image *) NULL; next=next->next)
         {
-          (void) CopyMagickString(next->filename,image->filename,
+          (void) CopyMagickString(next->filename,image_info->filename,
+            MagickPathExtent);
+          (void) CopyMagickString(next->magick,image_info->magick,
             MagickPathExtent);
-          (void) CopyMagickString(next->magick,image->magick,MagickPathExtent);
         }
-      (void) RelinquishUniqueFileResource(read_info->filename);
     }
   read_info=DestroyImageInfo(read_info);
-  image=DestroyImage(image);
   return(images);
 }
 
@@ -395,10 +478,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
     destination_file,
     source_file;
 
-  MagickBooleanType
-    status;
-
-  register size_t
+  size_t
     i;
 
   size_t
@@ -414,22 +494,16 @@ static MagickBooleanType CopyDelegateFile(const char *source,
   unsigned char
     *buffer;
 
-  /*
-    Return if destination file already exists and is not empty.
-  */
   assert(source != (const char *) NULL);
   assert(destination != (char *) NULL);
-  status=GetPathAttributes(destination,&attributes);
-  if ((status != MagickFalse) && (attributes.st_size > 0))
-    return(MagickTrue);
   /*
     Copy source file to destination.
   */
   if (strcmp(destination,"-") == 0)
     destination_file=fileno(stdout);
   else
-    destination_file=open_utf8(destination,O_WRONLY | O_BINARY | O_CREAT,
-      S_MODE);
+    destination_file=open_utf8(destination,O_WRONLY | O_BINARY | O_CREAT |
+      O_TRUNC,S_MODE);
   if (destination_file == -1)
     return(MagickFalse);
   source_file=open_utf8(source,O_RDONLY | O_BINARY,0);
@@ -470,12 +544,16 @@ static MagickBooleanType CopyDelegateFile(const char *source,
 static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
   Image *image,ExceptionInfo *exception)
 {
-#define WriteVIDEOIntermediateFormat "pam"
-
   char
     basename[MagickPathExtent],
     filename[MagickPathExtent];
 
+  const char
+    *intermediate_format;
+
+  const DelegateInfo
+    *delegate_info;
+
   double
     delay;
 
@@ -491,10 +569,10 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register Image
+  Image
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -516,10 +594,6 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
-  if (status == MagickFalse)
-    return(status);
-  (void) CloseBlob(image);
   /*
     Write intermediate files.
   */
@@ -533,7 +607,10 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     basename);
   count=0;
   write_info=CloneImageInfo(image_info);
+  write_info->file=(FILE *) NULL;
   *write_info->magick='\0';
+  status=MagickTrue;
+  intermediate_format=GetIntermediateFormat(image_info);
   for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     char
@@ -556,12 +633,11 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
             *frame;
 
           (void) FormatLocaleString(p->filename,MagickPathExtent,"%s%.20g.%s",
-            basename,(double) p->scene,WriteVIDEOIntermediateFormat);
+            basename,(double) p->scene,intermediate_format);
           (void) FormatLocaleString(filename,MagickPathExtent,"%s%.20g.%s",
-            basename,(double) p->scene,WriteVIDEOIntermediateFormat);
+            basename,(double) p->scene,intermediate_format);
           (void) FormatLocaleString(previous_image,MagickPathExtent,
-            "%s%.20g.%s",basename,(double) p->scene,
-            WriteVIDEOIntermediateFormat);
+            "%s%.20g.%s",basename,(double) p->scene,intermediate_format);
           frame=CloneImage(p,0,0,MagickTrue,exception);
           if (frame == (Image *) NULL)
             break;
@@ -577,7 +653,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
         default:
         {
           (void) FormatLocaleString(filename,MagickPathExtent,"%s%.20g.%s",
-            basename,(double) p->scene,WriteVIDEOIntermediateFormat);
+            basename,(double) p->scene,intermediate_format);
           if (length > 0)
             status=BlobToFile(filename,blob,length,exception);
           break;
@@ -588,11 +664,11 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
           if (status != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "%.20g. Wrote %s file for scene %.20g:",(double) i,
-              WriteVIDEOIntermediateFormat,(double) p->scene);
+              intermediate_format,(double) p->scene);
           else
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "%.20g. Failed to write %s file for scene %.20g:",(double) i,
-              WriteVIDEOIntermediateFormat,(double) p->scene);
+              intermediate_format,(double) p->scene);
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",filename);
         }
     }
@@ -605,19 +681,57 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
   /*
     Convert PAM to VIDEO.
   */
-  (void) CopyMagickString(coalesce_image->magick_filename,basename,
-    MagickPathExtent);
-  (void) CopyMagickString(coalesce_image->filename,basename,MagickPathExtent);
-  GetPathComponent(image_info->filename,ExtensionPath,coalesce_image->magick);
-  if (*coalesce_image->magick == '\0')
-    (void) CopyMagickString(coalesce_image->magick,image->magick,
-      MagickPathExtent);
-  status=InvokeDelegate(write_info,coalesce_image,(char *) NULL,"video:encode",
-    exception);
-  (void) FormatLocaleString(write_info->filename,MagickPathExtent,"%s.%s",
-    write_info->unique,coalesce_image->magick);
-  status=CopyDelegateFile(write_info->filename,image->filename);
-  (void) RelinquishUniqueFileResource(write_info->filename);
+  delegate_info=GetDelegateInfo((char *) NULL,"video:encode",exception);
+  if (delegate_info != (const DelegateInfo *) NULL)
+    {
+      char
+        command[MagickPathExtent],
+        message[MagickPathExtent];
+
+      char
+        *options,
+        *sanitized_option;
+
+      const char
+        *option;
+
+      int
+        exit_code;
+
+      options=AcquireString("");
+      (void) FormatLocaleString(options,MagickPathExtent,"-plays %i",
+        (int) coalesce_image->iterations);
+      option=GetImageOption(image_info,"video:pixel-format");
+      if (option != (const char *) NULL)
+        {
+          sanitized_option=SanitizeDelegateString(option);
+          (void) FormatLocaleString(command,MagickPathExtent," -pix_fmt %s",
+            sanitized_option);
+          DestroyString(sanitized_option);
+          (void) ConcatenateMagickString(options,command,MagickPathExtent);
+        }
+      AcquireUniqueFilename(write_info->unique);
+      (void) FormatLocaleString(command,MagickPathExtent,
+        GetDelegateCommands(delegate_info),basename,intermediate_format,
+        options,write_info->unique,image_info->magick);
+      options=DestroyString(options);
+      exit_code=ExternalDelegateCommand(MagickFalse,image_info->verbose,
+        command,message,exception);
+      status=exit_code == 0 ? MagickTrue : MagickFalse;
+      if (status != MagickFalse)
+        {
+          (void) FormatLocaleString(filename,MagickPathExtent,"%s.%s",
+            write_info->unique,image_info->magick);
+          status=CopyDelegateFile(filename,image->filename);
+          (void) RelinquishUniqueFileResource(filename);
+        }
+      else if (*message != '\0')
+        {
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            DelegateError,"VideoDelegateFailed","`%s'",message);
+        }
+      (void) RelinquishUniqueFileResource(write_info->unique);
+  }
   write_info=DestroyImageInfo(write_info);
   /*
     Relinquish resources.
@@ -629,14 +743,12 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     for (i=0; i < (ssize_t) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
     {
       (void) FormatLocaleString(p->filename,MagickPathExtent,"%s%.20g.%s",
-        basename,(double) count++,WriteVIDEOIntermediateFormat);
+        basename,(double) count++,intermediate_format);
       (void) RelinquishUniqueFileResource(p->filename);
     }
     (void) CopyMagickString(p->filename,image_info->filename,MagickPathExtent);
   }
   (void) RelinquishUniqueFileResource(basename);
   coalesce_image=DestroyImageList(coalesce_image);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"exit");
   return(status);
 }
