@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -37,7 +37,7 @@
 %  This coder is a kind of backdoor used by the COM object that allows it to  %
 %  pass blobs back and forth using the coder interface. It simply encodes and %
 %  decodes the filename as a comma delimited string and extracts the info it  %
-%  needs. The five methods of passing images are:                             %
+%  needs.                                                                     %
 %
 %
 */
@@ -45,21 +45,22 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/constitute.h"
-#include "magick/delegate.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/static.h"
-#include "magick/string_.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/delegate.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/magick-private.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
 #define WIN32_LEAN_AND_MEAN
 #define VC_EXTRALEAN
@@ -70,7 +71,7 @@
   Forward declarations.
 */
 static MagickBooleanType
-  WriteXTRNImage(const ImageInfo *,Image *);
+  WriteXTRNImage(const ImageInfo *,Image *,ExceptionInfo *exception);
 #endif
 
 /*
@@ -106,7 +107,7 @@ static Image *ReadXTRNImage(const ImageInfo *image_info,
 {
   char
     *blob_data,
-    filename[MaxTextExtent];
+    filename[MagickPathExtent];
 
   HRESULT
     hr;
@@ -152,19 +153,19 @@ static Image *ReadXTRNImage(const ImageInfo *image_info,
             {
               blob_length=lBoundu-lBoundl+1;
               hr=SafeArrayAccessData(pSafeArray,(void**) &blob_data);
-              if(SUCCEEDED(hr))
+              if (SUCCEEDED(hr))
                 {
                   *clone_info->filename='\0';
                   *clone_info->magick='\0';
                   if (*filename != '\0')
                     (void) CopyMagickString(clone_info->filename,filename,
-                      MaxTextExtent);
+                      MagickPathExtent);
                   image=BlobToImage(clone_info,blob_data,blob_length,
                     exception);
                   hr=SafeArrayUnaccessData(pSafeArray);
                   CatchException(exception);
                 }
-              }
+            }
         }
     }
   clone_info=DestroyImageInfo(clone_info);
@@ -200,16 +201,14 @@ ModuleExport size_t RegisterXTRNImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("XTRNARRAY");
+  entry=AcquireMagickInfo("XTRN","XTRNARRAY",
+    "External transfer via a smart array interface");
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
   entry->decoder=ReadXTRNImage;
   entry->encoder=WriteXTRNImage;
 #endif
-  entry->adjoin=MagickFalse;
-  entry->stealth=MagickTrue;
-  entry->description=ConstantString(
-    "External transfer via a smart array interface");
-  entry->magick_module=ConstantString("XTRN");
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderStealthFlag;
   RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -256,7 +255,7 @@ ModuleExport void UnregisterXTRNImage(void)
 %  The format of the WriteXTRNImage method is:
 %
 %      MagickBooleanType WriteXTRNImage(const ImageInfo *image_info,
-%        Image *image)
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -264,6 +263,8 @@ ModuleExport void UnregisterXTRNImage(void)
 %
 %    o image:  A pointer to a Image structure.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
@@ -311,13 +312,10 @@ static size_t SafeArrayFifo(const Image *image,const void *data,
 }
 
 static MagickBooleanType WriteXTRNImage(const ImageInfo *image_info,
-  Image *image)
+  Image *image,ExceptionInfo *exception)
 {
   char
-    filename[MaxTextExtent];
-
-  ExceptionInfo
-    *sans_exception;
+    filename[MagickPathExtent];
 
   Image
     *p;
@@ -340,7 +338,7 @@ static MagickBooleanType WriteXTRNImage(const ImageInfo *image_info,
   void
     *param1;
 
-  param1=(void *) NULL;
+  param1 = (void *) NULL;
   status=MagickTrue;
   clone_info=CloneImageInfo(image_info);
   if (*clone_info->filename != '\0')
@@ -348,18 +346,18 @@ static MagickBooleanType WriteXTRNImage(const ImageInfo *image_info,
       (void) sscanf(clone_info->filename,"%p,%2048s",&param1,filename);
       image->client_data=param1;
       scene=0;
-      (void) CopyMagickString(clone_info->filename,filename,MaxTextExtent);
+      (void) CopyMagickString(clone_info->filename,filename,
+        MagickPathExtent);
       for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
       {
-        (void) CopyMagickString(p->filename,filename,MaxTextExtent);
+        (void) CopyMagickString(p->filename,filename,MagickPathExtent);
         p->scene=scene++;
       }
-      SetImageInfo(clone_info,1,&image->exception);
+      SetImageInfo(clone_info,1,exception);
       (void) CopyMagickString(image->magick,clone_info->magick,
-        MaxTextExtent);
-      sans_exception=AcquireExceptionInfo();
-      blob_data=ImageToBlob(clone_info,image,&blob_length,sans_exception);
-      sans_exception=DestroyExceptionInfo(sans_exception);
+        MagickPathExtent);
+      blob_data=ImageToBlob(clone_info,image,&blob_length,
+        exception);
       if (blob_data == (unsigned char *) NULL)
         status=MagickFalse;
       else
